@@ -19,7 +19,8 @@ static void apply_case(RAnal *anal, RAnalBlock *block, ut64 switch_addr, ut64 of
 	}
 	if (anal->flb.set) {
 		char flagname[0x30];
-		snprintf (flagname, sizeof (flagname), "case.0x%"PFMT64x ".%d", (ut64)switch_addr, (int)id);
+		int iid = R_ABS((int)id);
+		snprintf (flagname, sizeof (flagname), "case.0x%"PFMT64x ".%d", (ut64)switch_addr, iid);
 		anal->flb.set (anal->flb.f, flagname, case_addr, 1);
 	}
 }
@@ -39,26 +40,40 @@ static void apply_switch(RAnal *anal, ut64 switch_addr, ut64 jmptbl_addr, ut64 c
 	}
 }
 
-// analyze a jmptablle inside a function // maybe rename to r_anal_fcn_jmptbl() ?
+// analyze a jmptablle inside a function // maybe rename to r_anal_function_jmptbl() ?
 R_API bool r_anal_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, ut64 jmpaddr, ut64 table, ut64 tablesize, ut64 default_addr) {
 	const int depth = 50;
 	return try_walkthrough_jmptbl (anal, fcn, block, depth, jmpaddr, 0, table, table, tablesize, tablesize, default_addr, false);
 }
 
 static inline void analyze_new_case(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, ut64 ip, ut64 jmpptr, int depth) {
-       const ut64 block_size = block->size;
-       (void)r_anal_fcn_bb (anal, fcn, jmpptr, depth - 1);
-       if (block->size != block_size) {
-               // block was be split during anal and does not contain the
-               // jmp instruction anymore, so we need to search for it and get it again
-               RAnalSwitchOp *sop = block->switch_op;
-	       block = r_anal_get_block_at (anal, ip);
-               if (!block) {
-                       r_warn_if_reached ();
-                       return;
-               }
-               block->switch_op = sop;
-       }
+	const ut64 block_size = block->size;
+	(void)r_anal_function_bb (anal, fcn, jmpptr, depth - 1);
+	if (block->size != block_size) {
+		// block was be split during anal and does not contain the
+		// jmp instruction anymore, so we need to search for it and get it again
+		RAnalSwitchOp *sop = block->switch_op;
+		block = r_anal_get_block_at (anal, ip);
+		if (!block) {
+			block = r_anal_bb_from_offset (anal, ip);
+			if (block) {
+				if (block->addr != ip) {
+					st64 d = block->addr - ip;
+					eprintf ("Cannot find basic block for switch case at 0x%08"PFMT64x" bbdelta = %d\n", ip, (int)R_ABS (d));
+					block = NULL;
+					return;
+				} else {
+					eprintf ("Inconsistent basicblock storage issue at 0x%08"PFMT64x"\n", ip);
+				}
+			} else {
+				eprintf ("Major disaster at 0x%08"PFMT64x"\n", ip);
+				return;
+			}
+			// analyze at given address
+			// block = r_anal_create_block(RAnal *anal, ut64 addr, ut64 size) {
+		}
+		block->switch_op = sop;
+	}
 }
 
 R_API bool try_walkthrough_casetbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *block, int depth, ut64 ip, st64 start_casenum_shift, ut64 jmptbl_loc, ut64 casetbl_loc, ut64 jmptbl_off, ut64 sz, ut64 jmptbl_size, ut64 default_case, bool ret0) {
@@ -165,14 +180,22 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 		return false;
 	}
 	ut64 jmpptr, offs;
+	int jmptblsz = jmptbl_size * sz;
+	if (jmptblsz < 1) {
+		if (anal->verbose) {
+			eprintf ("Invalid jump table size\n");
+		}
+		return false;
+	}
 	ut8 *jmptbl = calloc (jmptbl_size, sz);
 	if (!jmptbl) {
 		return false;
 	}
 	bool is_arm = anal->cur->arch && !strncmp (anal->cur->arch, "arm", 3);
-	const bool is_v850 = !is_arm && ((anal->cur->arch && !strncmp (anal->cur->arch, "v850", 4)) || !strncmp (anal->coreb.cfgGet (anal->coreb.core, "asm.cpu"), "v850", 4));
+	bool is_x86 = !is_arm && anal->cur->arch && !strncmp (anal->cur->arch, "x86", 3);
+	const bool is_v850 = !is_arm && !is_x86 && ((anal->cur->arch && !strncmp (anal->cur->arch, "v850", 4)) || !strncmp (anal->coreb.cfgGet (anal->coreb.core, "asm.cpu"), "v850", 4));
 	// eprintf ("JMPTBL AT 0x%"PFMT64x"\n", jmptbl_loc);
-	anal->iob.read_at (anal->iob.io, jmptbl_loc, jmptbl, jmptbl_size * sz);
+	anal->iob.read_at (anal->iob.io, jmptbl_loc, jmptbl, jmptblsz);
 	for (offs = 0; offs + sz - 1 < jmptbl_size * sz; offs += sz) {
 		switch (sz) {
 		case 1:
@@ -209,6 +232,10 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 			if (!anal->iob.is_valid_offset (anal->iob.io, jmpptr, 0)) {
 				break;
 			}
+		} else if (sz == 2 && is_x86) {
+			st32 jmpdelta = (st32)jmpptr;
+			// jump tables where sign extended movs are used
+			jmpptr = jmptbl_off + jmpdelta;
 		}
 		if (anal->limit) {
 			if (jmpptr < anal->limit->from || jmpptr > anal->limit->to) {
@@ -216,7 +243,7 @@ R_API bool try_walkthrough_jmptbl(RAnal *anal, RAnalFunction *fcn, RAnalBlock *b
 			}
 		}
 		//apply_case (anal, block, ip, sz, jmpptr, offs / sz, jmptbl_loc + offs);
-		//(void)r_anal_fcn_bb (anal, fcn, jmpptr, depth - 1);
+		//(void)r_anal_function_bb (anal, fcn, jmpptr, depth - 1);
 		int case_idx = offs / sz;
 		int casenum = case_idx + start_casenum_shift;
 		apply_case (anal, block, ip, sz, jmpptr, casenum, jmptbl_loc + offs);
@@ -402,8 +429,8 @@ R_API bool try_get_jmptbl_info(RAnal *anal, RAnalFunction *fcn, ut64 addr, RAnal
 	/* if UJMP is in .plt section just skip it */
 	RBinSection *s = anal->binb.get_vsect_at (anal->binb.bin, addr);
 	if (s && s->name[0]) {
-		bool in_plt = strstr (s->name, ".plt") != NULL;
-		if (!in_plt && strstr (s->name, "_stubs") != NULL) {
+		bool in_plt = strstr (s->name, ".plt");
+		if (!in_plt && strstr (s->name, "_stubs")) {
 			/* for mach0 */
 			in_plt = true;
 		}
@@ -428,15 +455,6 @@ R_API bool try_get_jmptbl_info(RAnal *anal, RAnalFunction *fcn, ut64 addr, RAnal
 	// default case is the jump target of the unconditional jump
 	*default_case = prev_bb->jump == my_bb->addr ? prev_bb->fail : prev_bb->jump;
 
-	RAnalOp tmp_aop = {0};
-	ut8 *bb_buf = calloc (1, prev_bb->size);
-	if (!bb_buf) {
-		return false;
-	}
-	// search for a cmp register with a reasonable size
-	anal->iob.read_at (anal->iob.io, prev_bb->addr, (ut8 *) bb_buf, prev_bb->size);
-	isValid = false;
-
 	RAnalHint *hint = r_anal_hint_get (anal, addr);
 	if (hint) {
 		ut64 val = hint->val;
@@ -447,6 +465,15 @@ R_API bool try_get_jmptbl_info(RAnal *anal, RAnalFunction *fcn, ut64 addr, RAnal
 		}
 	}
 
+	RAnalOp tmp_aop = {0};
+	ut8 *bb_buf = calloc (1, prev_bb->size);
+	if (!bb_buf) {
+		return false;
+	}
+	// search for a cmp register with a reasonable size
+	anal->iob.read_at (anal->iob.io, prev_bb->addr, (ut8 *) bb_buf, prev_bb->size);
+	isValid = false;
+
 	RRegItem *cmp_reg = NULL;
 	for (i = prev_bb->ninstr - 1; i >= 0; i--) {
 		const ut64 prev_pos = r_anal_bb_offset_inst (prev_bb, i);
