@@ -5,6 +5,7 @@
 #include <r_types.h>
 #include <r_util.h>
 #include <r_asm.h>
+#include <r_anal.h> // for RAnalBind
 #define USE_R2 1
 #include <spp/spp.h>
 #include <config.h>
@@ -26,7 +27,8 @@ static void parseHeap(RParse *p, RStrBuf *s) {
 	if (out) {
 		*out = 0;
 		strcpy (out , op_buf_asm);
-	// XXX we shouldn't pad here because we have t orefactor the RParse API to handle boundaries and chunks properly
+		// XXX we shouldn't pad here because we have to refactor
+		// the RParse API to handle boundaries and chunks properly
 		r_parse_parse (p, op_buf_asm, out);
 		r_strbuf_set (s, out);
 		free (out);
@@ -328,32 +330,78 @@ R_API bool r_asm_use_assembler(RAsm *a, const char *name) {
 	return false;
 }
 
+static void load_asm_descriptions(RAsm *a, RAsmPlugin *p) {
+	const char *arch = p->arch;
+	if (!arch || !strcmp (p->name, "r2ghidra")) {
+		if (a->cpu) {
+			arch = a->cpu;
+		} else {
+			arch = p->name;
+		}
+	}
+#if HAVE_GPERF
+	SdbGperf *gp = r_asm_get_gperf (arch); // p->name);
+	if (gp) {
+		sdb_free (a->pair);
+		a->pair = sdb_new0 ();
+		sdb_open_gperf (a->pair, gp);
+		return;
+	}
+#endif
+	char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
+	char *file = r_str_newf ("%s/%s.sdb", r_str_getf (r2prefix), arch);
+	if (file) {
+		sdb_free (a->pair);
+		a->pair = sdb_new (NULL, file, 0);
+		free (file);
+	}
+	free (r2prefix);
+}
+
 // TODO: this can be optimized using r_str_hash()
 R_API bool r_asm_use(RAsm *a, const char *name) {
-	RAsmPlugin *h;
-	RListIter *iter;
-	if (!a || !name) {
+	r_return_val_if_fail (a, false);
+	if (R_STR_ISEMPTY (name)) {
+		// that shouldnt be permitted imho, keep for backward compat
 		return false;
 	}
+	RAsmPlugin *h;
+	RListIter *iter;
 	r_list_foreach (a->plugins, iter, h) {
-		if (!strcmp (h->name, name) && h->arch) {
-			if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
-				char *r2prefix = r_str_r2_prefix (R2_SDB_OPCODES);
-				char *file = r_str_newf ("%s/%s.sdb", r_str_getf (r2prefix), h->arch);
-				if (file) {
+		if (h->arch) {
+			if (!strcmp (h->name, name)) {
+				if (!a->cur || (a->cur && strcmp (a->cur->arch, h->arch))) {
+					load_asm_descriptions (a, h);
 					r_asm_set_cpu (a, NULL);
-					sdb_free (a->pair);
-					a->pair = sdb_new (NULL, file, 0);
-					free (file);
 				}
-				free (r2prefix);
+				a->cur = h;
+				return true;
+			}
+		} else {
+			char *vv = strchr (name, '.');
+			if (vv) {
+				if (!strcmp (vv + 1, h->name)) {
+					char *cpu = r_str_ndup (name, vv - name);
+					r_asm_set_cpu (a, cpu);
+					a->cur = h;
+					free (cpu);
+					return true;
+				}
+			} else {
+				if (!strcmp (name, h->name)) {
+					h->arch = name;
+					r_asm_set_cpu (a, NULL);
+					a->cur = h;
+					return true;
+				}
 			}
-			a->cur = h;
-			return true;
 		}
 	}
 	sdb_free (a->pair);
 	a->pair = NULL;
+	if (strcmp (name, "null")) {
+		return r_asm_use (a, "null");
+	}
 	return false;
 }
 
@@ -418,7 +466,7 @@ R_API int r_asm_set_pc(RAsm *a, ut64 pc) {
 	return true;
 }
 
-static bool __isInvalid (RAsmOp *op) {
+static bool __isInvalid(RAsmOp *op) {
 	const char *buf_asm = r_strbuf_get (&op->buf_asm);
 	return (buf_asm && *buf_asm && !strcmp (buf_asm, "invalid"));
 }
@@ -454,6 +502,14 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 		} else {
 			ret = a->cur->disassemble (a, op, buf, len);
 		}
+	} else if (a->analb.anal) {
+		// disassemble using the analysis plugin if found
+		RAnalOp aop;
+		a->analb.opinit (&aop);
+		ret = a->analb.decode (a->analb.anal, &aop, a->pc, buf, len, R_ANAL_OP_MASK_DISASM);
+		op->size = aop.size;
+		r_strbuf_set (&op->buf_asm, aop.mnemonic);
+		a->analb.opfini (&aop);
 	}
 	if (ret < 0) {
 		ret = 0;
@@ -470,12 +526,13 @@ R_API int r_asm_disassemble(RAsm *a, RAsmOp *op, const ut8 *buf, int len) {
 
 	if (op->size < 1 || __isInvalid (op)) {
 		if (a->invhex) {
+			r_strf_buffer (32);
 			if (a->bits == 16) {
 				ut16 b = r_read_le16 (buf);
-				r_strbuf_set (&op->buf_asm, sdb_fmt (".word 0x%04x", b));
+				r_strbuf_set (&op->buf_asm, r_strf (".word 0x%04x", b));
 			} else {
 				ut32 b = r_read_le32 (buf);
-				r_strbuf_set (&op->buf_asm, sdb_fmt (".dword 0x%08x", b));
+				r_strbuf_set (&op->buf_asm, r_strf (".dword 0x%08x", b));
 			}
 			// TODO: something for 64bits too?
 		} else {
@@ -496,7 +553,7 @@ static bool assemblerMatches(RAsm *a, RAsmPlugin *h) {
 	if (!a || !h->arch || !h->assemble || !has_bits (h, a->bits)) {
 		return false;
 	}
-	return (!strncmp (a->cur->arch, h->arch, strlen (a->cur->arch)));
+	return (a->cur->arch && !strncmp (a->cur->arch, h->arch, strlen (a->cur->arch)));
 }
 
 static Ase findAssembler(RAsm *a, const char *kw) {
@@ -600,6 +657,16 @@ R_API int r_asm_assemble(RAsm *a, RAsmOp *op, const char *buf) {
 					ase = findAssembler (a, NULL);
 				}
 			}
+			if (!ase && a->analb.anal) {
+				// disassemble using the analysis plugin if found
+				ase = NULL;
+				RAnalOp aop;
+				a->analb.opinit (&aop);
+				ut8 buf[256] = {0};
+				ret = a->analb.encode (a->analb.anal, a->pc, b, buf, sizeof (buf));
+				r_strbuf_setbin (&op->buf, buf, R_MIN (ret, sizeof (buf)));
+				a->analb.opfini (&aop);
+			}
 		} else {
 			ase = a->cur->assemble;
 		}
@@ -625,7 +692,7 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 	RStrBuf *buf_asm;
 	RAsmCode *acode;
 	ut64 pc = a->pc;
-	RAsmOp op;
+	RAsmOp op = {0};
 	ut64 idx;
 	size_t ret;
 	const size_t addrbytes = a->user? ((RCore *)a->user)->io->addrbytes: 1;
@@ -652,6 +719,7 @@ R_API RAsmCode* r_asm_mdisassemble(RAsm *a, const ut8 *buf, int len) {
 		r_strbuf_append (buf_asm, r_strbuf_get (&op.buf_asm));
 		r_strbuf_append (buf_asm, "\n");
 	}
+	r_asm_op_fini (&op);
 	acode->assembly = r_strbuf_drain (buf_asm);
 	acode->len = idx;
 	return acode;
@@ -815,7 +883,7 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 				continue;
 			}
 			// XXX TODO remove arch-specific hacks
-			if (!strncmp (a->cur->arch, "avr", 3)) {
+			if (a->cur->arch && !strncmp (a->cur->arch, "avr", 3)) {
 				for (ptr_start = buf_token; *ptr_start && isavrseparator (*ptr_start); ptr_start++);
 			} else {
 				for (ptr_start = buf_token; *ptr_start && IS_SEPARATOR (*ptr_start); ptr_start++);
@@ -1017,6 +1085,9 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 						continue;
 					}
 					ret = r_asm_assemble (a, &op, ptr_start);
+					// XXX This fixes a leak, unsure
+					// why op_fini below doesn't catch it
+					r_strbuf_fini (&op.buf_asm);
 				}
 			}
 			if (stage == STAGES - 1) {
@@ -1046,10 +1117,12 @@ R_API RAsmCode *r_asm_massemble(RAsm *a, const char *assembly) {
 	}
 	free (lbuf);
 	free (tokens);
+	r_asm_op_fini (&op);
 	return acode;
 fail:
 	free (lbuf);
 	free (tokens);
+	r_asm_op_fini (&op);
 	return r_asm_code_free (acode);
 }
 
@@ -1171,3 +1244,28 @@ R_API RAsmCode* r_asm_rasm_assemble(RAsm *a, const char *buf, bool use_spp) {
 	free (lbuf);
 	return acode;
 }
+
+R_API RList *r_asm_cpus(RAsm *a) {
+	RList *list = NULL;
+	RListIter *iter;
+	char *item;
+	// get asm plugin
+	if (a->cur && a->cur->cpus) {
+		list = r_str_split_duplist (a->cur->cpus, ",", 0);
+	} else {
+		list = r_list_newf (free);
+	}
+	// get anal plugin
+	if (a->analb.anal && a->analb.anal->cur && a->analb.anal->cur->cpus) {
+		char *cpus = a->analb.anal->cur->cpus;
+		RList *al = r_str_split_duplist (cpus, ",", 0);
+		r_list_foreach (al, iter, item) {
+			if (!r_list_find (list, item, (RListComparator)strcmp)) {
+				r_list_append (list, strdup (item));
+			}
+		}
+		r_list_free (al);
+	}
+	r_list_sort (list, (RListComparator)strcmp);
+	return list;
+}
