@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2019 - pancake, nibble, dso */
+/* radare2 - LGPL - Copyright 2009-2022 - pancake, nibble, dso */
 
 #include <r_bin.h>
 #include <r_hash.h>
@@ -15,7 +15,7 @@ static RBinClass *__getClass(RBinFile *bf, const char *name) {
 
 static RBinSymbol *__getMethod(RBinFile *bf, const char *klass, const char *method) {
 	r_return_val_if_fail (bf && bf->o && bf->o->methods_ht && klass && method, NULL);
-	const char *name = sdb_fmt ("%s::%s", klass, method);
+	r_strf_var (name, 128, "%s::%s", klass, method);
 	return ht_pp_find (bf->o->methods_ht, name, NULL);
 }
 
@@ -26,20 +26,18 @@ static RBinString *__stringAt(RBinFile *bf, RList *ret, ut64 addr) {
 	return NULL;
 }
 
-static ut64 binobj_a2b(RBinObject *o, ut64 addr) {
-	return o ? addr + o->baddr_shift : addr;
+static ut64 binobj_a2b(RBinObject *bo, ut64 addr) {
+	return addr + (bo ? bo->baddr_shift : 0);
 }
 
 static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
 	r_return_if_fail (bf && string);
 
 	int mode = bf->strmode;
-	ut64 addr, vaddr;
 	RBin *bin = bf->rbin;
 	if (!bin) {
 		return;
 	}
-	const char *section_name, *type_string;
 	RIO *io = bin->iob.io;
 	if (!io) {
 		return;
@@ -48,9 +46,10 @@ static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
 	if (s) {
 		string->vaddr = s->vaddr + (string->paddr - s->paddr);
 	}
-	section_name = s ? s->name : "";
-	type_string = r_bin_string_type (string->type);
-	vaddr = addr = r_bin_get_vaddr (bin, string->paddr, string->vaddr);
+	const char *section_name = s ? s->name : "";
+	const char *type_string = r_bin_string_type (string->type);
+	ut64 vaddr = r_bin_get_vaddr (bin, string->paddr, string->vaddr);
+	ut64 addr = vaddr;
 
 	// If raw string dump mode, use printf to dump directly to stdout.
 	//  PrintfCallback temp = io->cb_printf;
@@ -83,7 +82,7 @@ static void print_string(RBinFile *bf, RBinString *string, int raw, PJ *pj) {
 		break;
 	case R_MODE_RADARE: {
 		char *f_name = strdup (string->string);
-		r_name_filter (f_name, 32);
+		r_name_filter (f_name, -1);
 		if (bin->prefix) {
 			io->cb_printf ("f %s.str.%s %u @ 0x%08"PFMT64x"\n"
 					"Cs %u @ 0x%08"PFMT64x"\n",
@@ -129,7 +128,11 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		eprintf ("Invalid range to find strings 0x%"PFMT64x" .. 0x%"PFMT64x"\n", from, to);
 		return -1;
 	}
-	int len = to - from;
+	st64 len = (st64)(to - from);
+	if (len > ST32_MAX) {
+		eprintf ("String scan range is too large\n");
+		return -1;
+	}
 	ut8 *buf = calloc (len, 1);
 	if (!buf || !min) {
 		free (buf);
@@ -146,6 +149,33 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		}
 	}
 	r_buf_read_at (bf->buf, from, buf, len);
+	char *charset = r_sys_getenv ("RABIN2_CHARSET");
+	if (!R_STR_ISEMPTY (charset)) {
+		RCharset *ch = r_charset_new ();
+		if (r_charset_use (ch, charset)) {
+			int outlen = len * 4;
+			ut8 *out = calloc (len, 4);
+			if (out) {
+				int res = r_charset_encode_str (ch, out, outlen, buf, len);
+				int i;
+				// TODO unknown chars should be translated to null bytes
+				for (i = 0; i < res; i++) {
+					if (out[i] == '?') {
+						out[i] = 0;
+					}
+				}
+				len = res;
+				free (buf);
+				buf = out;
+			} else {
+				eprintf ("Cannot allocate\n");
+			}
+		} else {
+			eprintf ("Invalid value for RABIN2_CHARSET.\n");
+		}
+		r_charset_free (ch);
+	}
+	free (charset);
 	// may oobread
 	while (needle < to) {
 		if (bin && bin->consb.is_breaked) {
@@ -153,23 +183,49 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 				break;
 			}
 		}
+		// smol optimization
+		if (needle + 4 < to) {
+			ut32 n1 = r_read_le32 (buf + needle - from);
+			if (!n1) {
+				needle += 4;
+				continue;
+			}
+		}
 		rc = r_utf8_decode (buf + needle - from, to - needle, NULL);
 		if (!rc) {
 			needle++;
 			continue;
 		}
+		bool addr_aligned = !(needle % 4);
+
 		if (type == R_STRING_TYPE_DETECT) {
 			char *w = (char *)buf + needle + rc - from;
-			if ((to - needle) > 5 + rc) {
-				bool is_wide32 = (needle + rc + 2 < to) && (!w[0] && !w[1] && !w[2] && w[3] && !w[4]);
-				if (is_wide32) {
-					str_type = R_STRING_TYPE_WIDE32;
+			if (((to - needle) > 8 + rc)) {
+				// TODO: support le and be
+				bool is_wide32le = (needle + rc + 2 < to) && (!w[0] && !w[1] && !w[2] && w[3] && !w[4]);
+				// reduce false positives
+				if (is_wide32le) {
+					if (!w[5] && !w[6] && w[7] && w[8]) {
+						is_wide32le = false;
+					}
+				}
+				if (!addr_aligned) {
+					is_wide32le = false;
+				}
+				///is_wide32be &= (n1 < 0xff && n11 < 0xff); // false; // n11 < 0xff;
+				if (is_wide32le  && addr_aligned) {
+					str_type = R_STRING_TYPE_WIDE32; // asume big endian,is there little endian w32?
 				} else {
+					// bool is_wide = (n1 && n2 && n1 < 0xff && (!n2 || n2 < 0xff));
 					bool is_wide = needle + rc + 4 < to && !w[0] && w[1] && !w[2] && w[3] && !w[4];
 					str_type = is_wide? R_STRING_TYPE_WIDE: R_STRING_TYPE_ASCII;
 				}
 			} else {
-				str_type = R_STRING_TYPE_ASCII;
+				if (rc > 1) {
+					str_type = R_STRING_TYPE_UTF8; // could be charset if set :?
+				} else {
+					str_type = R_STRING_TYPE_ASCII;
+				}
 			}
 		} else if (type == R_STRING_TYPE_UTF8) {
 			str_type = R_STRING_TYPE_ASCII; // initial assumption
@@ -182,7 +238,6 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 		/* Eat a whole C string */
 		for (i = 0; i < sizeof (tmp) - 4 && needle < to; i += rc) {
 			RRune r = {0};
-
 			if (str_type == R_STRING_TYPE_WIDE32) {
 				rc = r_utf32le_decode (buf + needle - from, to - needle, &r);
 				if (rc) {
@@ -262,7 +317,7 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 			case R_STRING_TYPE_WIDE32:
 				num_blocks = 0;
 				block_list = r_utf_block_list ((const ut8*)tmp, i - 1,
-				                               str_type == R_STRING_TYPE_WIDE ? &freq_list : NULL);
+						str_type == R_STRING_TYPE_WIDE? &freq_list: NULL);
 				if (block_list) {
 					for (j = 0; block_list[j] != -1; j++) {
 						num_blocks++;
@@ -329,8 +384,9 @@ static int string_scan_range(RList *list, RBinFile *bf, int min,
 					pdelta = s->paddr;
 				}
 			}
-			bs->paddr = str_start;
-			bs->vaddr = str_start - pdelta + vdelta;
+			ut64 baddr = bf->loadaddr && bf->o? bf->o->baddr: bf->loadaddr;
+			bs->paddr = str_start + baddr;
+			bs->vaddr = str_start - pdelta + vdelta + baddr;
 			bs->string = r_str_ndup ((const char *)tmp, i);
 			if (list) {
 				r_list_append (list, bs);
@@ -367,10 +423,10 @@ static bool __isDataSection(RBinFile *a, RBinSection *s) {
 		return true;
 	}
  	// Rust
-	return strstr (s->name, "_const") != NULL;
+	return strstr (s->name, "_const");
 }
 
-static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64 from, ut64 to, RBinSection * section) {
+static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64 from, ut64 to, RBinSection *section) {
 	r_return_if_fail (bf && bf->buf);
 
 	RBinPlugin *plugin = r_bin_file_cur_plugin (bf);
@@ -382,12 +438,12 @@ static void get_strings_range(RBinFile *bf, RList *list, int min, int raw, ut64
 		min = plugin? plugin->minstrlen: 4;
 	}
 	/* Some plugins return zero, fix it up */
-	if (!min) {
-		min = 4;
-	}
 	if (min < 0) {
 		return;
 	}
+	if (!min) {
+		min = 4;
+	}
 	{
 		RIO *io = bf->rbin->iob.io;
 		RCoreBind *cb = &io->corebind;
@@ -457,7 +513,7 @@ R_IPI RBinFile *r_bin_file_new(RBin *bin, const char *file, ut64 file_sz, int ra
 	return bf;
 }
 
-static RBinPlugin *get_plugin_from_buffer(RBin *bin, const char *pluginname, RBuffer *buf) {
+static RBinPlugin *get_plugin_from_buffer(RBin *bin, RBinFile *bf, const char *pluginname, RBuffer *buf) {
 	RBinPlugin *plugin = bin->force? r_bin_get_binplugin_by_name (bin, bin->force): NULL;
 	if (plugin) {
 		return plugin;
@@ -466,7 +522,7 @@ static RBinPlugin *get_plugin_from_buffer(RBin *bin, const char *pluginname, RBu
 	if (plugin) {
 		return plugin;
 	}
-	plugin = r_bin_get_binplugin_by_buffer (bin, buf);
+	plugin = r_bin_get_binplugin_by_buffer (bin, bf, buf);
 	if (plugin) {
 		return plugin;
 	}
@@ -479,7 +535,7 @@ R_API bool r_bin_file_object_new_from_xtr_data(RBin *bin, RBinFile *bf, ut64 bas
 	ut64 offset = data->offset;
 	ut64 sz = data->size;
 
-	RBinPlugin *plugin = get_plugin_from_buffer (bin, NULL, data->buf);
+	RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, NULL, data->buf);
 	bf->buf = r_buf_ref (data->buf);
 
 	RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, offset, sz);
@@ -525,7 +581,7 @@ R_IPI RBinFile *r_bin_file_new_from_buffer(RBin *bin, const char *file, RBuffer
 	if (bf) {
 		RListIter *item = r_list_append (bin->binfiles, bf);
 		bf->buf = r_buf_ref (buf);
-		RBinPlugin *plugin = get_plugin_from_buffer (bin, pluginname, bf->buf);
+		RBinPlugin *plugin = get_plugin_from_buffer (bin, bf, pluginname, bf->buf);
 		RBinObject *o = r_bin_object_new (bf, plugin, baseaddr, loadaddr, 0, r_buf_size (bf->buf));
 		if (!o) {
 			r_list_delete (bin->binfiles, item);
@@ -768,7 +824,7 @@ R_IPI bool r_bin_file_set_bytes(RBinFile *bf, const ut8 *bytes, ut64 sz, bool st
 	} else {
 		bf->buf = r_buf_new_with_bytes (bytes, sz);
 	}
-	return bf->buf != NULL;
+	return bf->buf;
 }
 
 R_API RBinPlugin *r_bin_file_cur_plugin(RBinFile *bf) {
@@ -1018,8 +1074,9 @@ R_API RBinSymbol *r_bin_file_add_method(RBinFile *bf, const char *klass, const c
 		if (sym) {
 			sym->name = strdup (method);
 			r_list_append (c->methods, sym);
-			const char *name = sdb_fmt ("%s::%s", klass, method);
+			char *name = r_str_newf ("%s::%s", klass, method);
 			ht_pp_insert (bf->o->methods_ht, name, sym);
+			free (name);
 		}
 	}
 	return sym;
