@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2015-2018 - inisider */
+/* radare - LGPL - Copyright 2015-2020 - inisider, pancake */
 
 #include "../../i/private.h"
 #include "mach0_classes.h"
@@ -6,6 +6,18 @@
 #define RO_META (1 << 0)
 #define MAX_CLASS_NAME_LEN 256
 
+#ifdef R_BIN_MACH064
+#define FAST_DATA_MASK 0x00007ffffffffff8UL
+#else
+#define FAST_DATA_MASK 0xfffffffcUL
+#endif
+
+#define METHOD_LIST_FLAG_IS_SMALL 0x80000000
+#define METHOD_LIST_FLAG_IS_PREOPT 0x3
+#define METHOD_LIST_ENTSIZE_FLAG_MASK 0xffff0003
+
+#define RO_DATA_PTR(x) ((x) & FAST_DATA_MASK)
+
 struct MACH0_(SMethodList) {
 	ut32 entsize;
 	ut32 count;
@@ -96,10 +108,10 @@ static mach0_ut va2pa(mach0_ut p, ut32 *offset, ut32 *left, RBinFile *bf);
 static void copy_sym_name_with_namespace(char *class_name, char *read_name, RBinSymbol *sym);
 static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass);
 static void get_objc_property_list(mach0_ut p, RBinFile *bf, RBinClass *klass);
-static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static);
-static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass);
-static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass);
-static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs);
+static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static, objc_cache_opt_info *oi);
+static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass, objc_cache_opt_info *oi);
+static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass, objc_cache_opt_info *oi);
+static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs, objc_cache_opt_info *oi);
 static bool read_ptr_pa(RBinFile *bf, ut64 paddr, mach0_ut *out);
 static bool read_ptr_va(RBinFile *bf, ut64 vaddr, mach0_ut *out);
 static char *read_str(RBinFile *bf, mach0_ut p, ut32 *offset, ut32 *left);
@@ -171,17 +183,21 @@ static void copy_sym_name_with_namespace(char *class_name, char *read_name, RBin
 	sym->name = strdup (read_name);
 }
 
+static int sort_by_offset(const void *_a , const void *_b) {
+	RBinField *a = (RBinField*)_a;
+	RBinField *b = (RBinField*)_b;
+	return a->offset - b->offset;
+}
+
 static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
-	struct MACH0_(SIVarList) il;
+	struct MACH0_(SIVarList) il = {0};
 	struct MACH0_(SIVar) i;
 	mach0_ut r;
 	ut32 offset, left, j;
-	char *name = NULL;
-	char *type = NULL;
 
 	int len;
 	bool bigendian;
-	mach0_ut ivar_offset_p, ivar_offset;
+	mach0_ut ivar_offset;
 	RBinField *field = NULL;
 	ut8 sivarlist[sizeof (struct MACH0_(SIVarList))] = {0};
 	ut8 sivar[sizeof (struct MACH0_(SIVar))] = {0};
@@ -195,7 +211,6 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 	if (!(r = va2pa (p, &offset, &left, bf))) {
 		return;
 	}
-	memset (&il, '\0', sizeof (struct MACH0_(SIVarList)));
 	if (r + left < r || r + sizeof (struct MACH0_(SIVarList)) < r) {
 		return;
 	}
@@ -219,16 +234,13 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 	il.count = r_read_ble (&sivarlist[4], bigendian, 32);
 	p += sizeof (struct MACH0_(SIVarList));
 	offset += sizeof (struct MACH0_(SIVarList));
+
 	for (j = 0; j < il.count; j++) {
 		r = va2pa (p, &offset, &left, bf);
 		if (!r) {
 			return;
 		}
-		if (!(field = R_NEW0 (RBinField))) {
-			// retain just for debug
-			// eprintf ("RBinField allocation error\n");
-			return;
-		}
+		field = R_NEW0 (RBinField);
 		memset (&i, '\0', sizeof (struct MACH0_(SIVar)));
 		if (r + left < r || r + sizeof (struct MACH0_(SIVar)) < r) {
 			goto error;
@@ -249,44 +261,39 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 				goto error;
 			}
 		}
-		switch (sizeof (mach0_ut)) {
-		case 4:
-			i.offset = r_read_ble (&sivar[0], bigendian, 32);
-			i.name = r_read_ble (&sivar[4], bigendian, 32);
-			i.type = r_read_ble (&sivar[8], bigendian, 32);
-			i.alignment = r_read_ble (&sivar[12], bigendian, 32);
-			i.size = r_read_ble (&sivar[16], bigendian, 32);
-			break;
-		case 8:
-			i.offset = r_read_ble (&sivar[0], bigendian, 64);
-			i.name = r_read_ble (&sivar[8], bigendian, 64);
-			i.type = r_read_ble (&sivar[16], bigendian, 64);
-			i.alignment = r_read_ble (&sivar[24], bigendian, 32);
-			i.size = r_read_ble (&sivar[28], bigendian, 32);
-			break;
-		}
+#if R_BIN_MACH064
+		i.offset = r_read_ble (&sivar[0], bigendian, 64);
+		i.name = r_read_ble (&sivar[8], bigendian, 64);
+		i.type = r_read_ble (&sivar[16], bigendian, 64);
+		i.alignment = r_read_ble (&sivar[24], bigendian, 32);
+		i.size = r_read_ble (&sivar[28], bigendian, 32);
+#else
+		i.offset = r_read_ble (&sivar[0], bigendian, 32);
+		i.name = r_read_ble (&sivar[4], bigendian, 32);
+		i.type = r_read_ble (&sivar[8], bigendian, 32);
+		i.alignment = r_read_ble (&sivar[12], bigendian, 32);
+		i.size = r_read_ble (&sivar[16], bigendian, 32);
+#endif
+		field->vaddr = i.offset;
+		mach0_ut offset_at = va2pa (i.offset, NULL, &left, bf);
 
-
-		ivar_offset_p = va2pa (i.offset, NULL, &left, bf);
-
-		if (ivar_offset_p > bf->size) {
+		if (offset_at > bf->size) {
 			goto error;
 		}
-		if (ivar_offset_p + sizeof (ivar_offset) > bf->size) {
+		if (offset_at + sizeof (ivar_offset) > bf->size) {
 			goto error;
 		}
-		if (ivar_offset_p != 0 && left >= sizeof (mach0_ut)) {
-			len = r_buf_read_at (bf->buf, ivar_offset_p, offs, sizeof (mach0_ut));
+		if (offset_at != 0 && left >= sizeof (mach0_ut)) {
+			len = r_buf_read_at (bf->buf, offset_at, offs, sizeof (mach0_ut));
 			if (len != sizeof (mach0_ut)) {
 				eprintf ("Error reading\n");
 				goto error;
 			}
 			ivar_offset = r_read_ble (offs, bigendian, 8 * sizeof (mach0_ut));
-			field->vaddr = ivar_offset;
+			field->offset = ivar_offset;
 		}
-
 		r = va2pa (i.name, NULL, &left, bf);
-		if (r != 0) {
+		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *)bf->o->bin_obj;
 			if (r + left < r) {
 				goto error;
@@ -294,6 +301,7 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			if (r > bf->size || r + left > bf->size) {
 				goto error;
 			}
+			char *name;
 			if (bin->has_crypto) {
 				name = strdup ("some_encrypted_data");
 				left = strlen (name) + 1;
@@ -308,12 +316,13 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 				}
 				name[name_len] = 0;
 			}
+			// XXX the field name shouldnt contain the class name
 			field->name = r_str_newf ("%s::%s%s", klass->name, "(ivar)", name);
 			R_FREE (name);
 		}
 
 		r = va2pa (i.type, NULL, &left, bf);
-		if (r != 0) {
+		if (r) {
 			struct MACH0_(obj_t) *bin = (struct MACH0_(obj_t) *) bf->o->bin_obj;
 			int is_crypted = bin->has_crypto;
 			if (r + left < r) {
@@ -322,25 +331,42 @@ static void get_ivar_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 			if (r > bf->size || r + left > bf->size) {
 				goto error;
 			}
+			char *type = NULL;
 			if (is_crypted == 1) {
 				type = strdup ("some_encrypted_data");
 			// 	left = strlen (name) + 1;
 			} else {
 				int type_len = R_MIN (MAX_CLASS_NAME_LEN, left);
 				type = calloc (1, type_len + 1);
-				r_buf_read_at (bf->buf, r, (ut8 *)type, type_len);
-				type[type_len] = 0;
+				if (type) {
+					r_buf_read_at (bf->buf, r, (ut8 *)type, type_len);
+					type[type_len] = 0;
+				}
 			}
-      			field->type = strdup (type);
-			R_FREE (type);
+			if (type) {
+				field->type = type;
+				type = NULL;
+			} else {
+				field->type = NULL;
+			}
+			r_list_append (klass->fields, field);
+		} else {
+			goto error;
 		}
-
-		r_list_append (klass->fields, field);
 		p += sizeof (struct MACH0_(SIVar));
 		offset += sizeof (struct MACH0_(SIVar));
 	}
+	if (!r_list_empty (klass->fields)) {
+		r_list_sort (klass->fields, sort_by_offset);
+	}
+	RBinField *isa_field = R_NEW0 (RBinField);
+	isa_field->name = strdup ("isa");
+	isa_field->size = sizeof (mach0_ut);
+	isa_field->type = strdup ("struct objc_class *");
+	isa_field->vaddr = 0;
+	isa_field->offset = 0;
+	r_list_prepend (klass->fields, isa_field);
 	return;
-
 error:
 	r_bin_field_free (field);
 }
@@ -489,13 +515,12 @@ error:
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static) {
+static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinClass *klass, bool is_static, objc_cache_opt_info *oi) {
 	struct MACH0_(SMethodList) ml;
-	struct MACH0_(SMethod) m;
 	mach0_ut r;
 	ut32 offset, left, i;
 	char *name = NULL;
-  	char *rtype = NULL;
+	char *rtype = NULL;
 	int len;
 	bool bigendian;
 	ut8 sml[sizeof (struct MACH0_(SMethodList))] = {0};
@@ -534,12 +559,25 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 	}
 	ml.entsize = r_read_ble (&sml[0], bigendian, 32);
 	ml.count = r_read_ble (&sml[4], bigendian, 32);
+	if (ml.count < 1 || ml.count > ST32_MAX) {
+		return;
+	}
+	if (r + (ml.count * (ml.entsize & ~METHOD_LIST_ENTSIZE_FLAG_MASK)) > bf->size) {
+		return;
+	}
+
+	bool is_small = (ml.entsize & METHOD_LIST_FLAG_IS_SMALL) != 0;
+	ut8 mlflags = ml.entsize & 0x3;
 
 	p += sizeof (struct MACH0_(SMethodList));
 	offset += sizeof (struct MACH0_(SMethodList));
+
+	size_t read_size = is_small ? 3 * sizeof (ut32) :
+			sizeof (struct MACH0_(SMethod));
+
 	for (i = 0; i < ml.count; i++) {
 		r = va2pa (p, &offset, &left, bf);
-		if (!r) {
+		if (!r || r == -1) {
 			return;
 		}
 
@@ -548,29 +586,57 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 			// eprintf ("RBinClass allocation error\n");
 			return;
 		}
+		struct MACH0_(SMethod) m;
 		memset (&m, '\0', sizeof (struct MACH0_(SMethod)));
-		if (r + left < r || r + sizeof (struct MACH0_(SMethod)) < r) {
+		if (r + left < r || r + read_size < r) {
 			goto error;
 		}
 		if (r > bf->size) {
 			goto error;
 		}
-		if (r + sizeof (struct MACH0_(SMethod)) > bf->size) {
+		if (r + read_size > bf->size) {
 			goto error;
 		}
-		if (left < sizeof (struct MACH0_(SMethod))) {
+		if (left < read_size) {
 			if (r_buf_read_at (bf->buf, r, sm, left) != left) {
 				goto error;
 			}
 		} else {
-			len = r_buf_read_at (bf->buf, r, sm, sizeof (struct MACH0_(SMethod)));
-			if (len != sizeof (struct MACH0_(SMethod))) {
+			len = r_buf_read_at (bf->buf, r, sm, read_size);
+			if (len != read_size) {
 				goto error;
 			}
 		}
-		m.name = r_read_ble (&sm[0], bigendian, 8 * sizeof (mach0_ut));
-		m.types = r_read_ble (&sm[sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
-		m.imp = r_read_ble (&sm[2 * sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+		if (!is_small) {
+			m.name = r_read_ble (&sm[0], bigendian, 8 * sizeof (mach0_ut));
+			m.types = r_read_ble (&sm[sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+			m.imp = r_read_ble (&sm[2 * sizeof (mach0_ut)], bigendian, 8 * sizeof (mach0_ut));
+		} else {
+			st64 name_offset = (st32) r_read_ble (&sm[0], bigendian, 8 * sizeof (ut32));
+			mach0_ut name;
+			if (oi && oi->sel_string_base) {
+				name = oi->sel_string_base + name_offset;
+			} else {
+				name = p + name_offset;
+			}
+			if (mlflags != METHOD_LIST_FLAG_IS_PREOPT) {
+				r = va2pa (name, &offset, &left, bf);
+				if (!r) {
+					goto error;
+				}
+				ut8 tmp[8];
+				if (r_buf_read_at (bf->buf, r, tmp, sizeof (mach0_ut)) != sizeof (mach0_ut)) {
+					goto error;
+				}
+				m.name = r_read_ble (tmp, bigendian, 8 * sizeof (mach0_ut));
+			} else {
+				m.name = name;
+			}
+			st64 types_offset = (st32) r_read_ble (&sm[sizeof (ut32)], bigendian, 8 * sizeof (ut32));
+			m.types = p + types_offset + 4;
+			st64 imp_offset = (st32) r_read_ble (&sm[2 * sizeof (ut32)], bigendian, 8 * sizeof (ut32));
+			m.imp = p + imp_offset + 8;
+		}
 
 		r = va2pa (m.name, NULL, &left, bf);
 		if (r) {
@@ -621,7 +687,7 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 				}
 				rtype[left] = 0;
 			}
-      			method->rtype = strdup (rtype);
+			method->rtype = strdup (rtype);
 			R_FREE (rtype);
 		}
 
@@ -643,8 +709,8 @@ static void get_method_list_t(mach0_ut p, RBinFile *bf, char *class_name, RBinCl
 		}
 		r_list_append (klass->methods, method);
 next:
-		p += sizeof (struct MACH0_(SMethod));
-		offset += sizeof (struct MACH0_(SMethod));
+		p += read_size;
+		offset += read_size;
 	}
 	return;
 error:
@@ -654,8 +720,8 @@ error:
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
-	struct MACH0_(SProtocolList) pl = { 0 };
+static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass, objc_cache_opt_info *oi) {
+	struct MACH0_(SProtocolList) pl = {0};
 	struct MACH0_(SProtocol) pc;
 	char *class_name = NULL;
 	ut32 offset, left, i, j;
@@ -791,10 +857,10 @@ static void get_protocol_list_t(mach0_ut p, RBinFile *bf, RBinClass *klass) {
 		}
 
 		if (pc.instanceMethods > 0) {
-			get_method_list_t (pc.instanceMethods, bf, class_name, klass, false);
+			get_method_list_t (pc.instanceMethods, bf, class_name, klass, false, oi);
 		}
 		if (pc.classMethods > 0) {
-			get_method_list_t (pc.classMethods, bf, class_name, klass, true);
+			get_method_list_t (pc.classMethods, bf, class_name, klass, true, oi);
 		}
 		R_FREE (class_name);
 		p += sizeof (ut32);
@@ -943,9 +1009,9 @@ static char *get_class_name(mach0_ut p, RBinFile *bf) {
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass) {
+static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinClass *klass, objc_cache_opt_info *oi) {
 	struct MACH0_(obj_t) *bin;
-	struct MACH0_(SClassRoT) cro = { 0 };
+	struct MACH0_(SClassRoT) cro = {0};
 	ut32 offset, left, i;
 	ut64 r, s;
 	int len;
@@ -1032,20 +1098,22 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 			}
 		}
 		//eprintf ("0x%x  %s\n", s, klass->name);
-		sdb_num_set (bin->kv, sdb_fmt ("objc_class_%s.offset", klass->name), s, 0);
+		char *k = r_str_newf ("objc_class_%s.offset", klass->name);
+		sdb_num_set (bin->kv, k, s, 0);
+		free (k);
 	}
 #ifdef R_BIN_MACH064
-	sdb_set (bin->kv, sdb_fmt ("objc_class.format", 0), "lllll isa super cache vtable data", 0);
+	sdb_set (bin->kv, "objc_class.format", "lllll isa super cache vtable data", 0);
 #else
-	sdb_set (bin->kv, sdb_fmt ("objc_class.format", 0), "xxxxx isa super cache vtable data", 0);
+	sdb_set (bin->kv, "objc_class.format", "xxxxx isa super cache vtable data", 0);
 #endif
 
 	if (cro.baseMethods > 0) {
-		get_method_list_t (cro.baseMethods, bf, klass->name, klass, (cro.flags & RO_META) ? true : false);
+		get_method_list_t (cro.baseMethods, bf, klass->name, klass, (cro.flags & RO_META) ? true : false, oi);
 	}
 
 	if (cro.baseProtocols > 0) {
-		get_protocol_list_t (cro.baseProtocols, bf, klass);
+		get_protocol_list_t (cro.baseProtocols, bf, klass, oi);
 	}
 
 	if (cro.ivars > 0) {
@@ -1061,13 +1129,13 @@ static void get_class_ro_t(mach0_ut p, RBinFile *bf, ut32 *is_meta_class, RBinCl
 	}
 }
 
-static mach0_ut get_isa_value() {
+static mach0_ut get_isa_value(void) {
 	// TODO: according to otool sources this is taken from relocs
 	return 0;
 }
 
-void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe, RSkipList *relocs) {
-	struct MACH0_(SClass) c = { 0 };
+void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe, RSkipList *relocs, objc_cache_opt_info *oi) {
+	struct MACH0_(SClass) c = {0};
 	const int size = sizeof (struct MACH0_(SClass));
 	mach0_ut r = 0;
 	ut32 offset = 0, left = 0;
@@ -1130,7 +1198,7 @@ void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe,
 			}
 		}
 	}
-	get_class_ro_t (c.data & ~0x3, bf, &is_meta_class, klass);
+	get_class_ro_t (RO_DATA_PTR (c.data), bf, &is_meta_class, klass, oi);
 
 #if SWIFT_SUPPORT
 	if (q (c.data + n_value) & 7) {
@@ -1140,7 +1208,7 @@ void MACH0_(get_class_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, bool dupe,
 	if (!is_meta_class && !dupe) {
 		mach0_ut isa_n_value = get_isa_value ();
 		ut64 tmp = klass->addr;
-		MACH0_(get_class_t) (c.isa + isa_n_value, bf, klass, true, relocs);
+		MACH0_(get_class_t) (c.isa + isa_n_value, bf, klass, true, relocs, oi);
 		klass->addr = tmp;
 	}
 }
@@ -1181,7 +1249,7 @@ static RList *parse_swift_classes(RBinFile *bf) {
 }
 #endif
 
-RList *MACH0_(parse_classes)(RBinFile *bf) {
+RList *MACH0_(parse_classes)(RBinFile *bf, objc_cache_opt_info *oi) {
 	RList /*<RBinClass>*/ *ret = NULL;
 	ut64 num_of_unnamed_class = 0;
 	RBinClass *klass = NULL;
@@ -1205,7 +1273,7 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 
 	RSkipList *relocs = MACH0_(get_relocs) (bf->o->bin_obj);
 
-	ret = MACH0_(parse_categories) (bf, relocs);
+	ret = MACH0_(parse_categories) (bf, relocs, oi);
 
 	/* check if it's Swift */
 	// ret = parse_swift_classes (bf);
@@ -1214,6 +1282,7 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 
 	struct section_t *sections = NULL;
 	if (!(sections = MACH0_(get_sections) (bf->o->bin_obj))) {
+		r_skiplist_free (relocs);
 		return ret;
 	}
 
@@ -1274,7 +1343,7 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 			goto get_classes_error;
 		}
 		p = r_read_ble (&pp[0], bigendian, 8 * sizeof (mach0_ut));
-		MACH0_(get_class_t) (p, bf, klass, false, relocs);
+		MACH0_(get_class_t) (p, bf, klass, false, relocs, oi);
 		if (!klass->name) {
 			klass->name = r_str_newf ("UnnamedClass%" PFMT64d, num_of_unnamed_class);
 			if (!klass->name) {
@@ -1284,16 +1353,18 @@ RList *MACH0_(parse_classes)(RBinFile *bf) {
 		}
 		r_list_append (ret, klass);
 	}
+	r_skiplist_free (relocs);
 	return ret;
 
 get_classes_error:
 	r_list_free (sctns);
 	r_list_free (ret);
+	r_skiplist_free (relocs);
 	// XXX DOUBLE FREE r_bin_class_free (klass);
 	return NULL;
 }
 
-static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs) {
+static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs, objc_cache_opt_info *oi) {
 	r_return_val_if_fail (bf && bf->o && bf->o->bin_obj && bf->o->info, NULL);
 
 	RList /*<RBinClass>*/ *ret = NULL;
@@ -1355,7 +1426,7 @@ static RList *MACH0_(parse_categories)(RBinFile *bf, RSkipList *relocs) {
 			R_FREE (klass);
 			goto error;
 		}
-		MACH0_(get_category_t) (p, bf, klass, relocs);
+		MACH0_(get_category_t) (p, bf, klass, relocs, oi);
 		if (!klass->name) {
 			R_FREE (klass);
 			continue;
@@ -1369,10 +1440,10 @@ error:
 	return NULL;
 }
 
-void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipList *relocs) {
+void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipList *relocs, objc_cache_opt_info *oi) {
 	r_return_if_fail (bf && bf->o && bf->o->info);
 
-	struct MACH0_(SCategory) c = { 0 };
+	struct MACH0_(SCategory) c = {0};
 	const int size = sizeof (struct MACH0_(SCategory));
 	mach0_ut r = 0;
 	ut32 offset = 0, left = 0;
@@ -1453,8 +1524,7 @@ void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipLis
 			R_FREE (category_name);
 			return;
 		}
-
-		mach0_ut name_field = ro_data + 3 * 4 + ptr_size;
+		mach0_ut name_field = RO_DATA_PTR (ro_data) + 3 * 4 + ptr_size;
 #ifdef R_BIN_MACH064
 		name_field += 4;
 #endif
@@ -1469,7 +1539,7 @@ void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipLis
 		if (target_class_name) {
 			demangled = demangle_classname (target_class_name);
 		}
-		klass->name = r_str_newf ("%s(%s)", demangled ? demangled : "(null)", category_name);
+		klass->name = r_str_newf ("%s(%s)", r_str_getf (demangled), category_name);
 		R_FREE (target_class_name);
 		R_FREE (demangled);
 	}
@@ -1479,15 +1549,15 @@ void MACH0_(get_category_t)(mach0_ut p, RBinFile *bf, RBinClass *klass, RSkipLis
 	R_FREE (category_name);
 
 	if (c.instanceMethods > 0) {
-		get_method_list_t (c.instanceMethods, bf, klass->name, klass, false);
+		get_method_list_t (c.instanceMethods, bf, klass->name, klass, false, oi);
 	}
 
 	if (c.classMethods > 0) {
-		get_method_list_t (c.classMethods, bf, klass->name, klass, true);
+		get_method_list_t (c.classMethods, bf, klass->name, klass, true, oi);
 	}
 
 	if (c.protocols > 0) {
-		get_protocol_list_t (c.protocols, bf, klass);
+		get_protocol_list_t (c.protocols, bf, klass, oi);
 	}
 
 	if (c.properties > 0) {
