@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2021 - pancake */
+/* radare - LGPL - Copyright 2007-2022 - pancake */
 /* dietline is a lightweight and portable library similar to GNU readline */
 
 #include <r_cons.h>
@@ -13,22 +13,23 @@
 static int r_line_readchar_win(ut8 *s, int slen);
 #else
 #include <sys/ioctl.h>
+#ifndef HAVE_PTY
 #include <termios.h>
+#endif
 #include <signal.h>
 #define USE_UTF8 1
 #endif
 
-static char *r_line_nullstr = "";
 static const char word_break_characters[] = "\t\n ~`!@#$%^&*()-_=+[]{}\\|;:\"'<>,./";
 
+// TODO: remove global variables
+static R_TH_LOCAL bool enable_yank_pop = false;
 
 typedef enum {
 	MINOR_BREAK,
 	MAJOR_BREAK
 } BreakMode;
 
-bool enable_yank_pop = false;
-
 static inline bool is_word_break_char(char ch, bool mode) {
 	int i;
 	if (mode == MAJOR_BREAK) {
@@ -259,14 +260,14 @@ static int r_line_readchar_utf8(ut8 *s, int slen) {
 
 #if __WINDOWS__
 static int r_line_readchar_win(ut8 *s, int slen) { // this function handle the input in console mode
-	INPUT_RECORD irInBuf = { { 0 } };
-	BOOL ret, bCtrl = FALSE;
+	r_sys_backtrace();
+	INPUT_RECORD irInBuf = { {0} };
+	BOOL ret;
 	DWORD mode, out;
 	char buf[5] = {0};
-	HANDLE h;
 	void *bed;
 
-	h = GetStdHandle (STD_INPUT_HANDLE);
+	HANDLE h = GetStdHandle (STD_INPUT_HANDLE);
 	DWORD new_mode = I.vtmode == 2 ? ENABLE_VIRTUAL_TERMINAL_INPUT : 0;
 	GetConsoleMode (h, &mode);
 	SetConsoleMode (h, new_mode);
@@ -295,19 +296,19 @@ do_it_again:
 	if (irInBuf.EventType == KEY_EVENT) {
 		if (irInBuf.Event.KeyEvent.bKeyDown) {
 			if (irInBuf.Event.KeyEvent.uChar.UnicodeChar) {
-				char *tmp = r_sys_conv_win_to_utf8_l ((PTCHAR)&irInBuf.Event.KeyEvent.uChar, 1);
-				if (!tmp) {
-					return 0;
+				ut8 chbuf[4] = {0};
+				memcpy (chbuf, &(irInBuf.Event.KeyEvent.uChar), 2);
+				char *tmp = r_sys_conv_win_to_utf8_l ((PTCHAR)&chbuf, 1);
+				if (tmp) {
+					r_str_ncpy (buf, tmp, sizeof (buf));
+					free (tmp);
 				}
-				strncpy_s (buf, sizeof (buf), tmp, strlen (tmp));
-				free (tmp);
 			} else {
 				int idx = 0;
 				buf[idx++] = 27;
 				buf[idx++] = '['; // Simulate escaping
-				bCtrl = irInBuf.Event.KeyEvent.dwControlKeyState & 8;
-				if (bCtrl) {
-					buf[idx++] = 0x31;
+				if (irInBuf.Event.KeyEvent.dwControlKeyState & 8) {
+					buf[idx++] = '1'; // control key
 				}
 				switch (irInBuf.Event.KeyEvent.wVirtualKeyCode) {
 				case VK_UP: buf[idx++] = 'A'; break;
@@ -327,7 +328,7 @@ do_it_again:
 	if (!buf[0]) {
 		goto do_it_again;
 	}
-	strncpy_s ((char *)s, slen, buf, sizeof (buf));
+	r_str_ncpy ((char *)s, buf, slen);
 	SetConsoleMode (h, mode);
 	return strlen ((char *)s);
 }
@@ -343,7 +344,12 @@ R_API int r_line_set_hist_callback(RLine *line, RLineHistoryUpCb up, RLineHistor
 	return 1;
 }
 
-static inline bool match_hist_line(char *hist_line, char *cur_line) {
+static inline bool match_hist_line(RLine *line, int i) {
+	const char *hist_line = line->history.data[i];
+	char *cur_line = line->history.match;
+	if (!line->histfilter) {
+		return true;
+	}
 	// Starts with but not equal to
 	return r_str_startswith (hist_line, cur_line) && strcmp (hist_line, cur_line);
 }
@@ -370,7 +376,7 @@ R_API int r_line_hist_cmd_up(RLine *line) {
 		if (line->history.match) {
 			int i;
 			for (i= line->history.index - 1; i >= 0; i--) {
-				if (match_hist_line (line->history.data[i], line->history.match)) {
+				if (match_hist_line (line, i)) {
 					line->history.index = i;
 					break;
 				}
@@ -399,7 +405,7 @@ R_API int r_line_hist_cmd_down(RLine *line) {
 	if (line->history.match) {
 		int i;
 		for (i = line->history.index + 1; i < line->history.top; i++) {
-			if (match_hist_line (line->history.data[i], line->history.match)) {
+			if (match_hist_line (line, i)) {
 				break;
 			}
 		}
@@ -519,7 +525,7 @@ R_API int r_line_hist_load(const char *file) {
 		free (path);
 		return false;
 	}
-	while (fgets (buf, sizeof (buf), fd) != NULL) {
+	while (fgets (buf, sizeof (buf), fd)) {
 		r_str_trim_tail (buf);
 		r_line_hist_add (buf);
 	}
@@ -528,25 +534,23 @@ R_API int r_line_hist_load(const char *file) {
 	return true;
 }
 
-R_API int r_line_hist_save(const char *file) {
-	FILE *fd;
-	int i, ret = false;
-	if (!file || !*file) {
-		return false;
-	}
+R_API bool r_line_hist_save(const char *file) {
+	r_return_val_if_fail (file && *file, false);
+	int i;
+	bool ret = false;
 	char *p, *path = r_str_home (file);
-	if (path != NULL) {
+	if (path) {
 		p = (char *) r_str_lastbut (path, R_SYS_DIR[0], NULL);	// TODO: use fs
 		if (p) {
 			*p = 0;
 			if (!r_sys_mkdirp (path)) {
-				eprintf ("could not save history into %s\n", path);
+				eprintf ("Could not save history into %s\n", path);
 				goto end;
 			}
 			*p = R_SYS_DIR[0];
 		}
-		fd = r_sandbox_fopen (path, "w");
-		if (fd != NULL) {
+		FILE *fd = r_sandbox_fopen (path, "w");
+		if (fd) {
 			if (I.history.data) {
 				for (i = 0; i < I.history.index; i++) {
 					fputs (I.history.data[i], fd);
@@ -664,10 +668,7 @@ static void selection_widget_down(int steps) {
 }
 
 static void print_rline_task(void *_core) {
-	RCore *core =(RCore *)_core;
-	if (core->cons->context->color_mode) {
-		r_cons_clear_line (0);
-	}
+	r_cons_clear_line (0);
 	r_cons_printf ("%s%s%s", Color_RESET, I.prompt,  I.buffer.data);
 	r_cons_flush ();
 }
@@ -777,11 +778,11 @@ R_API void r_line_autocomplete(void) {
 	if (argc == 1) {
 		const char *end_word = r_sub_str_rchr (I.buffer.data,
 			I.buffer.index, strlen (I.buffer.data), ' ');
-		const char *t = end_word != NULL?
+		const char *t = end_word?
 				end_word: I.buffer.data + I.buffer.index;
 		int largv0 = strlen (r_str_get (argv[0]));
 		size_t len_t = strlen (t);
-		p[largv0]='\0';
+		p[largv0] = '\0';
 
 		if ((p - I.buffer.data) + largv0 + 1 + len_t < plen) {
 			if (len_t > 0) {
@@ -934,21 +935,22 @@ static inline void delete_till_end(void) {
 }
 
 static void __print_prompt(void) {
-        RCons *cons = r_cons_singleton ();
+	RCons *cons = r_cons_singleton ();
 	int columns = r_cons_get_size (NULL) - 2;
-	int chars = R_MAX (1, strlen (I.buffer.data));
 	int len, i, cols = R_MAX (1, columns - r_str_ansi_len (I.prompt) - 2);
 	if (cons->line->prompt_type == R_LINE_PROMPT_OFFSET) {
-                r_cons_gotoxy (0,  cons->rows);
-                r_cons_flush ();
+		r_cons_gotoxy (0,  cons->rows);
+		r_cons_flush ();
 	}
+	r_cons_clear_line (0);
 	if (cons->context->color_mode > 0) {
-		r_cons_clear_line (0);
 		printf ("\r%s%s", Color_RESET, I.prompt);
 	} else {
 		printf ("\r%s", I.prompt);
 	}
-	fwrite (I.buffer.data, 1, R_MIN (cols, chars), stdout);
+	if (I.buffer.length > 0) {
+		fwrite (I.buffer.data, I.buffer.length, 1, stdout);
+	}
 	printf ("\r%s", I.prompt);
 	if (I.buffer.index > cols) {
 		printf ("< ");
@@ -1073,7 +1075,7 @@ static inline void vi_cmd_e(void) {
 	}
 }
 
-static void __update_prompt_color (void) {
+static void __update_prompt_color(void) {
 	RCons *cons = r_cons_singleton ();
 	const char *BEGIN = "", *END = "";
 	if (cons->context->color_mode) {
@@ -1108,7 +1110,7 @@ static void __vi_mode(void) {
 		if (I.echo) {
 			__print_prompt ();
 		}
-		if (I.vi_mode != CONTROL_MODE) {	// exit if insert mode is selected
+		if (I.vi_mode != CONTROL_MODE) { // exit if insert mode is selected
 			__update_prompt_color ();
 			break;
 		}
@@ -1313,6 +1315,30 @@ static void __vi_mode(void) {
 	}
 }
 
+static void dietline_print_risprompt(const char *gcomp_line) {
+	RCons *cons = r_cons_singleton ();
+	if (cons->context->color_mode && *gcomp_line && I.buffer.length > 0) {
+		printf ("\r (ri-search): ");
+		const char *line = gcomp_line;
+		while (line) {
+			char *m = strstr (line, I.buffer.data);
+			if (m) {
+				fwrite (line, m - line, 1, stdout);
+				printf (Color_INVERT);
+				fwrite (m, I.buffer.length, 1, stdout);
+				printf (Color_RESET);
+				line = m + I.buffer.length;
+			} else {
+				printf ("%s", line);
+				line = NULL;
+			}
+		}
+		printf ("\r");
+	} else {
+		printf ("\r(ri-search (%s)): %s\r", I.buffer.data, gcomp_line);
+	}
+}
+
 R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 	int rows;
 	const char *gcomp_line = "";
@@ -1350,7 +1376,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 		if (!fgets (I.buffer.data, R_LINE_BUFSIZE, stdin)) {
 			return NULL;
 		}
-		return (*I.buffer.data)? I.buffer.data: r_line_nullstr;
+		return (*I.buffer.data)? I.buffer.data: "";
 	}
 
 	memset (&buf, 0, sizeof buf);
@@ -1462,7 +1488,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 		case 4:	// ^D
 			if (!I.buffer.data[0]) {/* eof */
 				if (I.echo) {
-					printf ("^D\n");
+					eprintf ("^D\n");
 				}
 				r_cons_set_raw (false);
 				r_cons_break_pop ();
@@ -1601,16 +1627,24 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			break;
 		case 27: // esc-5b-41-00-00 alt/meta key
 #if __WINDOWS__
+			// always skip escape
+			memmove (buf, buf + 1, strlen ((char *)buf));
+#if 0
 			if (I.vtmode != 2) {
-				memmove (buf, buf + 1, strlen (buf));
+				if (buf[1] == '[') {
+					memmove (buf, buf + 2, strlen (buf));
+				} else {
+					memmove (buf, buf + 1, strlen (buf));
+				}
 				if (!buf[0]) {
 					buf[0] = -1;
 				}
 			} else {
-#endif
 				buf[0] = r_cons_readchar_timeout (50);
-#if __WINDOWS__
 			}
+#endif
+#else
+			buf[0] = r_cons_readchar_timeout (50);
 #endif
 			switch (buf[0]) {
 			case 127: // alt+bkspace
@@ -1665,6 +1699,7 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 				}
 				break;
 			default:
+#if !__WINDOWS__
 				if (I.vtmode == 2) {
 					buf[1] = r_cons_readchar_timeout (50);
 					if (buf[1] == -1) { // alt+e
@@ -1672,18 +1707,37 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						__print_prompt ();
 						continue;
 					}
+				} else {
+					buf[1] = r_cons_readchar_timeout (50);
 				}
-				if (buf[0] == 0x5b) {	// [
+#endif
+				if (buf[0] == '[') { // [
 					switch (buf[1]) {
-					case '3': // supr
+					case '3': // supr or mouse click
 						__delete_next_char ();
 						if (I.vtmode == 2) {
 							buf[1] = r_cons_readchar ();
+							if (buf[1] == 126) {
+								// handle SUPR key
+								r_cons_break_pop ();
+								__print_prompt ();
+								continue;
+							}
 							if (buf[1] == -1) {
 								r_cons_break_pop ();
 								return NULL;
 							}
 						}
+						for (;;) {
+							ch = r_cons_readchar ();
+							if (ch < 20) {
+								r_cons_break_pop ();
+								return NULL;
+							}
+							if (isupper (ch)) { // read until 'M'
+								break;
+							}
+						}
 						break;
 					case '5': // pag up
 						if (I.vtmode == 2) {
@@ -1970,8 +2024,8 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 			if (gcomp) {
 				gcomp_line = "";
 				int counter = 0;
-				if (I.history.data != NULL) {
-					for (i = I.history.size-1; i >= 0; i--) {
+				if (I.history.data) {
+					for (i = I.history.size - 1; i >= 0; i--) {
 						if (!I.history.data[i]) {
 							continue;
 						}
@@ -1986,9 +2040,9 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user) {
 						}
 					}
 				}
-				printf ("\r (reverse-i-search (%s)): %s\r", I.buffer.data, gcomp_line);
+				dietline_print_risprompt (gcomp_line);
 			} else {
-			        __print_prompt ();
+				__print_prompt ();
 			}
 			fflush (stdout);
 		}
@@ -2012,7 +2066,7 @@ _end:
 	if (!memcmp (I.buffer.data, "!history", 8)) {
 		// if (I.buffer.data[0]=='!' && I.buffer.data[1]=='\0') {
 		r_line_hist_list ();
-		return r_line_nullstr;
+		return "";
 	}
-	return I.buffer.data[0] != '\0'? I.buffer.data: r_line_nullstr;
+	return I.buffer.data[0] != '\0'? I.buffer.data: "";
 }
