@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nibble */
 
 #include <r_cons.h>
 #include <r_util/r_print.h>
@@ -6,7 +6,7 @@
 
 #define I(x) r_cons_singleton ()->x
 
-static char *strchr_ns (char *s, const char ch) {
+static char *strchr_ns(char *s, const char ch) {
 	char *p = strchr (s, ch);
 	if (p && p > s) {
 		char *prev = p - 1;
@@ -32,9 +32,12 @@ static const char *help_detail_tilde[] = {
 	" ?",        "", "count number of matching lines",
 	" ?.",       "", "count number chars",
 	" ??",       "", "show this help message",
+	" ?ea",      "", "convert text into seven segment style ascii art",
 	" :s..e",    "", "show lines s-e",
 	" ..",       "", "internal 'less'",
 	" ...",      "", "internal 'hud' (like V_)",
+	" ....",     "", "internal 'hud' in one line",
+	" :)",       "", "parse C-like output from decompiler",
 	" {:",       "", "human friendly indentation (yes, it's a smiley)",
 	" {:..",     "", "less the output of {:",
 	" {:...",    "", "hud the output of {:",
@@ -59,11 +62,6 @@ static const char *help_detail_tilde[] = {
 	NULL
 };
 
-/* TODO: remove globals */
-static RList *sorted_lines = NULL;
-static RList *unsorted_lines = NULL;
-static int sorted_column = -1;
-
 R_API void r_cons_grep_help(void) {
 	r_cons_cmd_help (help_detail_tilde, true);
 }
@@ -77,13 +75,17 @@ static void parse_grep_expression(const char *str) {
 	ut64 range_begin, range_end;
 	size_t ptrs_length;
 
-	if (!str || !*str) {
+	if (R_STR_ISEMPTY (str)) {
 		return;
 	}
 	RCons *cons = r_cons_singleton ();
-	RConsGrep *grep = &cons->context->grep;
+	RConsContext *ctx = cons->context;
+	RConsGrep *grep = &ctx->grep;
 
 	len = strlen (str) - 1;
+	if (len < 0) {
+		len = 0;
+	}
 	if (len > R_CONS_GREP_BUFSIZE - 1) {
 		eprintf ("r_cons_grep: too long!\n");
 		return;
@@ -115,17 +117,27 @@ static void parse_grep_expression(const char *str) {
 	free (grep->str);
 	grep->str = NULL;
 	bool first = true;
-	sorted_column = 0;
+	ctx->sorted_column = 0;
 	size_t i;
 	for (i = 0; i < ptrs_length; i++) {
 		ptr = ptrs[i];
 		end_ptr = ptr2 = ptr3 = NULL;
 		while (*ptr) {
 			switch (*ptr) {
+			case ':':
+				if (ptr[1] == ')') { // ":)"
+					grep->code = true;
+					ptr++;
+				}
+				goto while_end;
 			case '.':
 				if (ptr[1] == '.') {
 					if (ptr[2] == '.') {
-						grep->less = 2;
+						if (ptr[3] == '.') {
+							grep->less = 3;
+						} else {
+							grep->less = 2;
+						}
 					} else {
 						grep->less = 1;
 					}
@@ -193,18 +205,18 @@ static void parse_grep_expression(const char *str) {
 			case '+':
 				if (first) {
 					ptr++;
-					grep->icase = 1;
+					grep->icase = true;
 				} else {
 					goto while_end;
 				}
 				break;
 			case '^':
 				ptr++;
-				grep->begin = 1;
+				grep->begin[grep->nstrings] = true;
 				break;
 			case '!':
 				ptr++;
-				grep->neg = 1;
+				grep->neg[grep->nstrings] = true;
 				break;
 			case '?':
 				ptr++;
@@ -212,8 +224,11 @@ static void parse_grep_expression(const char *str) {
 				if (*ptr == '.') {
 					grep->charCounter = true;
 					ptr++;
+				} else if (!strcmp (ptr, "ea")) {
+					// "?ea"
+					grep->ascart = true;
 				} else if (*ptr == '?') {
-					cons->filter = true;
+					cons->context->filter = true;
 					r_cons_grep_help ();
 					return;
 				}
@@ -315,7 +330,7 @@ static void parse_grep_expression(const char *str) {
 
 		len = strlen (ptr) - 1;
 		if (len > 1 && ptr[len] == '$' && ptr[len - 1] != '\\') {
-			grep->end = 1;
+			grep->end[i] = 1;
 			ptr[len] = '\0';
 		}
 
@@ -358,7 +373,7 @@ static void parse_grep_expression(const char *str) {
 	}
 }
 
-// Finds and returns next intgerp expression,
+// Finds and returns next intgrep expression,
 // unescapes escaped twiddles
 static char *find_next_intgrep(char *cmd, const char *quotes) {
 	char *p;
@@ -454,13 +469,14 @@ static int cmp(const void *a, const void *b) {
 	if (!a || !b) {
 		return (int) (size_t) ((char *) a - (char *) b);
 	}
-	if (sorted_column > 0) {
+	RConsContext *ctx = r_cons_context ();
+	if (ctx->sorted_column > 0) {
 		da = strdup (ca);
 		db = strdup (cb);
 		int colsa = r_str_word_set0 (da);
 		int colsb = r_str_word_set0 (db);
-		ca = (colsa > sorted_column)? r_str_word_get0 (da, sorted_column): "";
-		cb = (colsb > sorted_column)? r_str_word_get0 (db, sorted_column): "";
+		ca = (colsa > ctx->sorted_column)? r_str_word_get0 (da, ctx->sorted_column): "";
+		cb = (colsb > ctx->sorted_column)? r_str_word_get0 (db, ctx->sorted_column): "";
 	}
 	if (IS_DIGIT (*ca) && IS_DIGIT (*cb)) {
 		ut64 na = r_num_get (NULL, ca);
@@ -487,21 +503,50 @@ R_API void r_cons_grepbuf(void) {
 	const int len = cons->context->buffer_len;
 	RConsGrep *grep = &cons->context->grep;
 	const char *in = buf;
-	int ret, total_lines = 0, buffer_len = 0, l = 0, tl = 0;
+	int ret, total_lines = 0, l = 0, tl = 0;
 	bool show = false;
-	if (cons->filter) {
+	if (cons->context->filter) {
 		cons->context->buffer_len = 0;
 		R_FREE (cons->context->buffer);
 		return;
 	}
+	if (grep->code) {
+		char *buf = r_str_ndup (cons->context->buffer, cons->context->buffer_len);
+		if (buf) {
+			char *res = r_str_tokenize_json (buf);
+			free (buf);
+			if (res) {
+				cons->context->buffer_len = strlen (res);
+				cons->context->buffer_sz = cons->context->buffer_len;
+				free (cons->context->buffer);
+				cons->context->buffer = res;
+			} else {
+				cons->context->buffer_len = 0;
+				cons->context->buffer_sz = 0;
+				free (cons->context->buffer);
+				cons->context->buffer = strdup ("");
+			}
+		}
+		return;
+	}
 
-	if ((!len || !buf || buf[0] == '\0') && (grep->json || grep->less)) {
+	if ((!len || !buf || !*buf) && (grep->json || grep->less)) {
 		grep->json = 0;
 		grep->less = 0;
 		grep->hud = 0;
 		return;
 	}
-
+	if (grep->ascart) {
+		char *buf = strdup (cons->context->buffer);
+		r_str_ansi_filter (buf, NULL, NULL, -1);
+		char *out = r_str_ss (buf, NULL, 0);
+		free (cons->context->buffer);
+		free (buf);
+		cons->context->buffer = out;
+		cons->context->buffer_len = strlen (out);
+		cons->context->buffer_sz = cons->context->buffer_len;
+		return;
+	}
 	if (grep->zoom) {
 		char *in = calloc (cons->context->buffer_len + 2, 4);
 		strcpy (in, cons->context->buffer);
@@ -565,7 +610,13 @@ R_API void r_cons_grepbuf(void) {
 	if (grep->less) {
 		int less = grep->less;
 		grep->less = 0;
-		if (less == 2) {
+		if (less == 3) {
+			char *res = r_cons_hud_line_string (buf);
+			if (res) {
+				r_cons_println (res);
+				free (res);
+			}
+		} else if (less == 2) {
 			char *res = r_cons_hud_string (buf);
 			if (res) {
 				r_cons_println (res);
@@ -581,11 +632,6 @@ R_API void r_cons_grepbuf(void) {
 		}
 		return;
 	}
-	if (!cons->context->buffer) {
-		cons->context->buffer_len = len + 20;
-		cons->context->buffer = malloc (cons->context->buffer_len);
-		cons->context->buffer[0] = 0;
-	}
 	RStrBuf *ob = r_strbuf_new ("");
 	// if we modify cons->lines we should update I.context->buffer too
 	cons->lines = 0;
@@ -614,7 +660,7 @@ R_API void r_cons_grepbuf(void) {
 			grep->l_line = total_lines + grep->l_line;
 		}
 	}
-	bool is_range_line_grep_only = grep->range_line != 2 && !*grep->str;
+	bool is_range_line_grep_only = grep->range_line != 2 && grep->str && *grep->str == '\0';
 	in = buf;
 	while ((int) (size_t) (in - buf) < len) {
 		char *p = strchr (in, '\n');
@@ -624,7 +670,7 @@ R_API void r_cons_grepbuf(void) {
 		l = p - in;
 		if ((!l && is_range_line_grep_only) || l > 0) {
 			char *tline = r_str_ndup (in, l);
-			if (cons->grep_color) {
+			if (cons->context->grep_color) {
 				tl = l;
 			} else {
 				tl = r_str_ansi_filter (tline, NULL, NULL, l);
@@ -651,7 +697,7 @@ R_API void r_cons_grepbuf(void) {
 			if ((!ret && is_range_line_grep_only) || ret > 0) {
 				if (show) {
 					char *str = r_str_ndup (tline, ret);
-					if (cons->grep_highlight) {
+					if (cons->context->grep_highlight) {
 						int i;
 						for (i = 0; i < grep->nstrings; i++) {
 							char *newstr = r_str_newf (Color_INVERT"%s"Color_RESET, grep->strings[i]);
@@ -669,7 +715,6 @@ R_API void r_cons_grepbuf(void) {
 						r_strbuf_append (ob, str);
 						r_strbuf_append (ob, "\n");
 					}
-					buffer_len += ret + 1;
 					free (str);
 				}
 				if (!grep->range_line) {
@@ -678,6 +723,7 @@ R_API void r_cons_grepbuf(void) {
 				cons->lines++;
 			} else if (ret < 0) {
 				free (tline);
+				r_strbuf_free (ob);
 				return;
 			}
 			free (tline);
@@ -690,11 +736,12 @@ R_API void r_cons_grepbuf(void) {
 	cons->context->buffer_len = r_strbuf_length (ob);
 	if (grep->counter) {
 		int cnt = grep->charCounter? strlen (cons->context->buffer): cons->lines;
-		if (cons->context->buffer_len < 10) {
-			cons->context->buffer_len = 10; // HACK
+		if (cons->context->buffer) {
+			free (cons->context->buffer);
 		}
-		snprintf (cons->context->buffer, cons->context->buffer_len, "%d\n", cnt);
+		cons->context->buffer = r_str_newf ("%d\n", cnt);
 		cons->context->buffer_len = strlen (cons->context->buffer);
+		cons->context->buffer_sz = cons->context->buffer_len+1;
 		cons->num->value = cons->lines;
 		r_strbuf_free (ob);
 		return;
@@ -728,27 +775,30 @@ R_API void r_cons_grepbuf(void) {
 		int nl = 0;
 		char *ptr = cons->context->buffer;
 		char *str;
-		sorted_column = grep->sort;
-		r_list_sort (sorted_lines, cmp);
+		RConsContext *ctx = cons->context;
+		ctx->sorted_column = grep->sort;
+
+		r_list_sort (ctx->sorted_lines, cmp);
 		if (grep->sort_invert) {
-			r_list_reverse (sorted_lines);
+			r_list_reverse (ctx->sorted_lines);
 		}
-		INSERT_LINES (unsorted_lines);
-		INSERT_LINES (sorted_lines);
+		INSERT_LINES (ctx->unsorted_lines);
+		INSERT_LINES (ctx->sorted_lines);
 		cons->lines = nl;
-		r_list_free (sorted_lines);
-		sorted_lines = NULL;
-		r_list_free (unsorted_lines);
-		unsorted_lines = NULL;
+		r_list_free (ctx->sorted_lines);
+		ctx->sorted_lines = NULL;
+		r_list_free (ctx->unsorted_lines);
+		ctx->unsorted_lines = NULL;
 	}
 }
 
 R_API int r_cons_grep_line(char *buf, int len) {
+	r_return_val_if_fail (buf && len >= 0, 0);
 	RCons *cons = r_cons_singleton ();
 	RConsGrep *grep = &cons->context->grep;
 	const char *delims = " |,;=\t";
 	char *tok = NULL;
-	bool hit = grep->neg;
+	bool hit = true;
 	int outlen = 0;
 	bool use_tok = false;
 	size_t i;
@@ -765,7 +815,7 @@ R_API int r_cons_grep_line(char *buf, int len) {
 	memcpy (in, buf, len);
 
 	if (grep->nstrings > 0) {
-		int ampfail = grep->amp;
+		bool all_hits = true;
 		if (grep->icase) {
 			r_str_case (in, false);
 		}
@@ -776,29 +826,33 @@ R_API int r_cons_grep_line(char *buf, int len) {
 			}
 			const char *p = r_strstr_ansi (in, grep->strings[i]);
 			if (!p) {
-				ampfail = 0;
+				hit = grep->neg[i];
+				all_hits &= hit;
 				continue;
 			}
-			if (grep->begin) {
-				hit = (p == in);
-			} else {
-				hit = !grep->neg;
-			}
+			hit = grep->begin[i]
+				? grep->neg[i]
+					? p != in
+					: p == in
+				: !grep->neg[i];
+
 			// TODO: optimize without strlen without breaking t/feat_grep (grep end)
-			if (grep->end && (strlen (grep->strings[i]) != strlen (p))) {
-				hit = 0;
+			if (grep->end[i] && (strlen (grep->strings[i]) != strlen (p))) {
+				hit = false;
 			}
+			all_hits &= hit;
 			if (!grep->amp) {
 				break;
 			}
 		}
 		if (grep->amp) {
-			hit = ampfail;
+			hit = all_hits;
 		}
 	} else {
-		hit = 1;
+		hit = true;
 	}
 
+	RConsContext *ctx = r_cons_context ();
 	if (hit) {
 		if (!grep->range_line) {
 			if (grep->line == cons->lines) {
@@ -851,16 +905,16 @@ R_API int r_cons_grep_line(char *buf, int len) {
 	if (grep->sort != -1) {
 		char ch = buf[len];
 		buf[len] = 0;
-		if (!sorted_lines) {
-			sorted_lines = r_list_newf (free);
+		if (!ctx->sorted_lines) {
+			ctx->sorted_lines = r_list_newf (free);
 		}
-		if (!unsorted_lines) {
-			unsorted_lines = r_list_newf (free);
+		if (!ctx->unsorted_lines) {
+			ctx->unsorted_lines = r_list_newf (free);
 		}
 		if (cons->lines >= grep->sort_row) {
-			r_list_append (sorted_lines, strdup (buf));
+			r_list_append (ctx->sorted_lines, strdup (buf));
 		} else {
-			r_list_append (unsorted_lines, strdup (buf));
+			r_list_append (ctx->unsorted_lines, strdup (buf));
 		}
 		buf[len] = ch;
 	}
@@ -869,6 +923,7 @@ R_API int r_cons_grep_line(char *buf, int len) {
 }
 
 R_API void r_cons_grep(const char *grep) {
+	r_return_if_fail (grep);
 	parse_grep_expression (grep);
 	r_cons_grepbuf ();
 }
