@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #include "r_types.h"
 #include "r_config.h"
@@ -7,10 +7,12 @@
 #include "r_debug.h"
 #include "r_io.h"
 
+static void __core_cmd_search_backward_prelude(RCore *core, bool doseek, bool forward);
+
 static const char *help_msg_s[] = {
 	"Usage: s", "", " # Help for the seek commands. See ?$? to see all variables",
 	"s", "", "Print current address",
-	"s.", "hexoff", "Seek honoring a base from core->offset",
+	"s.", "[?]hexoff", "Seek honoring a base from core->offset",
 	"s:", "pad", "Print current address with N padded zeros (defaults to 8)",
 	"s", " addr", "Seek to address",
 	"s-", "", "Undo seek",
@@ -32,13 +34,22 @@ static const char *help_msg_s[] = {
 	"sg/sG", "", "Seek begin (sg) or end (sG) of section or file",
 	"sl", "[?] [+-]line", "Seek to line",
 	"sn/sp", " ([nkey])", "Seek to next/prev location, as specified by scr.nkey",
+	"snp", "", "Seek to next function prelude",
+	"spp", "", "Seek to prev function prelude",
 	"so", " [N]", "Seek to N next opcode(s)",
 	"sr", " pc", "Seek to register",
-	"ss", "", "Seek silently (without adding an entry to the seek history)",
+	"ss", "[?]", "Seek silently (without adding an entry to the seek history)",
 	// "sp [page]  seek page N (page = block)",
 	NULL
 };
 
+static const char *help_msg_sdot[] = {
+	"Usage:", "s.", "seek here or there (near seeks)",
+	"s.", "", "Seek here, same as 's $$'",
+	"s..", "32a8", "Seek to the same address but replacing the lower nibbles",
+	NULL
+};
+
 static const char *help_msg_sC[] = {
 	"Usage:", "sC", "Comment grep",
 	"sC", "*", "List all comments",
@@ -46,13 +57,27 @@ static const char *help_msg_sC[] = {
 	NULL
 };
 
+static const char *help_msg_sn[] = {
+	"Usage:", "sn[p]", "",
+	"sn", " [line]", "Seek to next address",
+	"snp", "", "Seek to next prelude",
+	NULL
+};
+
+static const char *help_msg_sp[] = {
+	"Usage:", "sp[p]", "",
+	"sp", " [line]", "Seek to previous address",
+	"spp", "", "Seek to previous prelude",
+	NULL
+};
+
 static const char *help_msg_sl[] = {
 	"Usage:", "sl+ or sl- or slc", "",
 	"sl", " [line]", "Seek to absolute line",
 	"sl", "[+-][line]", "Seek to relative line",
 	"slc", "", "Clear line cache",
 	"sll", "", "Show total number of lines",
-	"sleep", " [seconds]", "Sleep for an specific amount of time",
+	"sleep", " [seconds]", "Sleep for an specific amount of time (support decimal values)",
 	NULL
 };
 
@@ -62,13 +87,6 @@ static const char *help_msg_ss[] = {
 	NULL
 };
 
-static void cmd_seek_init(RCore *core, RCmdDesc *parent) {
-	DEFINE_CMD_DESCRIPTOR (core, s);
-	DEFINE_CMD_DESCRIPTOR (core, sC);
-	DEFINE_CMD_DESCRIPTOR (core, sl);
-	DEFINE_CMD_DESCRIPTOR (core, ss);
-}
-
 static void __init_seek_line(RCore *core) {
 	ut64 from, to;
 
@@ -285,7 +303,7 @@ static int cmd_seek_opcode_backward(RCore *core, int numinstr) {
 	return val;
 }
 
-static int cmd_seek_opcode_forward (RCore *core, int n) {
+static int cmd_seek_opcode_forward(RCore *core, int n) {
 	// N forward instructions
 	int i, ret, val = 0;
 	for (val = i = 0; i < n; i++) {
@@ -330,7 +348,7 @@ static int cmd_seek(void *data, const char *input) {
 		return 0;
 	}
 	char *ptr;
-	if ((ptr = strstr (input, "+.")) != NULL) {
+	if ((ptr = strstr (input, "+."))) {
 		char *dup = strdup (input);
 		dup[ptr - input] = '\x00';
 		off = r_num_math (core->num, dup + 1);
@@ -462,11 +480,19 @@ static int cmd_seek(void *data, const char *input) {
 	}
 	break;
 	case '.': // "s." "s.."
-		for (input++; *input == '.'; input++) {
-			;
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_sdot);
+		} else if (input[1]) {
+			for (input++; *input == '.'; input++) {
+				;
+			}
+			r_core_seek_base (core, input);
+			r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
+		} else {
+			// just re-read the current block
+			r_core_seek (core, core->offset, true);
+			r_core_block_read (core);
 		}
-		r_core_seek_base (core, input);
-		r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
 		break;
 	case 'j':  // "sj"
 		{
@@ -612,9 +638,9 @@ static int cmd_seek(void *data, const char *input) {
 		case '-': // "s--"
 		default:
 			{
-				int delta = -off;
+				st64 delta = -(st64)off;
 				if (input[1] == '-') {
-					delta = -core->blocksize;
+					delta = -(st64)core->blocksize;
 					int mult = r_num_math (core->num, input + 2);
 					if (mult > 0) {
 						delta /= mult;
@@ -631,7 +657,11 @@ static int cmd_seek(void *data, const char *input) {
 		}
 		break;
 	case 'n': // "sn"
-		{
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_sn);
+		} else if (input[1] == 'p') { // "snp" - seek next prelude
+			__core_cmd_search_backward_prelude (core, true, true);
+		} else {
 			if (!silent) {
 				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
 			}
@@ -642,7 +672,11 @@ static int cmd_seek(void *data, const char *input) {
 		}
 		break;
 	case 'p': // "sp"
-		{
+		if (input[1] == '?') {
+			r_core_cmd_help (core, help_msg_sp);
+		} else if (input[1] == 'p') { // "spp" - seek previous prelude
+			__core_cmd_search_backward_prelude (core, true, false);
+		} else {
 			if (!silent) {
 				r_io_sundo_push (core->io, core->offset, r_print_get_cursor (core->print));
 			}
@@ -755,8 +789,15 @@ static int cmd_seek(void *data, const char *input) {
 			{
 				const char *arg = strchr (input, ' ');
 				if (arg) {
+					arg++;
 					void *bed = r_cons_sleep_begin ();
-					r_sys_sleep (atoi (arg + 1));
+					if (strchr (arg, '.')) {
+						double d = 0;
+						sscanf (arg, "%lf", &d);
+						r_sys_usleep ((int)(d * 1000000));
+					} else {
+						r_sys_sleep (atoi (arg));
+					}
 					r_cons_sleep_end (bed);
 				} else {
 					eprintf ("Usage: sleep [seconds]\n");
