@@ -1,16 +1,16 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #include "r_crypto.h"
 #include "r_config.h"
 #include "r_cons.h"
 #include "r_core.h"
 #include "r_io.h"
-#include "cmd_helps.h"
 
 static const char *help_msg_w[] = {
 	"Usage:","w[x] [str] [<file] [<<EOF] [@addr]","",
 	"w","[1248][+-][n]","increment/decrement byte,word..",
 	"w"," foobar","write string 'foobar'",
+	"w+","string","write string and seek at the end of it",
 	"w0"," [len]","write 'len' bytes with value 0x00",
 	"w6","[de] base64/hex","write base64 [d]ecoded or [e]ncoded string",
 	"wa","[?] push ebp","write opcode, separated by ';' (use '\"' around the command)",
@@ -20,8 +20,7 @@ static const char *help_msg_w[] = {
 	"wA","[?] r 0","alter/modify opcode at current seek (see wA?)",
 	"wb"," 010203","fill current block with cyclic hexpairs",
 	"wB","[-]0xVALUE","set or unset bits with given value",
-	"wc","","list all write changes",
-	"wc","[?][jir+-*?]","write cache undo/commit/reset/list (io.cache)",
+	"wc","[?][jir+-*?]","write cache list/undo/commit/reset (io.cache)",
 	"wd"," [off] [n]","duplicate N bytes from offset at current seek (memcpy) (see y?)",
 	"we","[?] [nNsxX] [arg]","extend write operations (insert instead of replace)",
 	"wf","[fs] -|file","write contents of file at current offset",
@@ -30,9 +29,8 @@ static const char *help_msg_w[] = {
 	"wo","[?] hex","write in block with operation. 'wo?' fmi",
 	"wp","[?] -|file","apply radare patch file. See wp? fmi",
 	"wr"," 10","write 10 random bytes",
-	"ws"," pstring","write 1 byte for length and then the string",
-	"wt[f]","[?] file [sz]","write to file (from current seek, blocksize or sz bytes)",
-	"wts"," host:port [sz]", "send data to remote host:port via tcp://",
+	"ws","[?] pstring","write pascal string: 1 byte for length + N for the string",
+	"wt","[?] file [sz]","write to file (from current seek, blocksize or sz bytes)",
 	"ww"," foobar","write wide string 'f\\x00o\\x00o\\x00b\\x00a\\x00r\\x00'",
 	"wx","[?][fs] 9090","write two intel nops (from wxfile or wxseek)",
 	"wv","[?] eip+34","write 32-64 bit value honoring cfg.bigendian",
@@ -40,10 +38,20 @@ static const char *help_msg_w[] = {
 	NULL
 };
 
+static const char *help_msg_ws[] = {
+	"Usage:", "ws[124?] [string]", "Pascal strings are not null terminated and store the length in binary at the beginning",
+	"ws", " str", "write pascal string using first byte as length",
+	"ws1", " str", "same as above",
+	"ws2", " str", "same as above but using ut16 as length (honors cfg.bigendian)",
+	"ws4", " str", "same, but using ut32 (honors cfg.bigendian)",
+	NULL
+};
+
 static const char *help_msg_wa[] = {
 	"Usage:", "wa[of*] [arg]", "",
 	"wa", " nop", "write nopcode using asm.arch and asm.bits",
 	"wai", " jmp 0x8080", "write inside this op (fill with nops or error if doesnt fit)",
+	"wan", " jmp 0x8080", "write instruction(s) nopping the trailing bytes",
 	"wa*", " mov eax, 33", "show 'wx' op with hexpair bytes of assembled opcode",
 	"\"wa nop;nop\"", "" , "assemble more than one instruction (note the quotes)",
 	"waf", " f.asm" , "assemble file and write bytes",
@@ -74,6 +82,7 @@ static const char *help_msg_wc[] = {
 	"wc*","","\"\" in radare commands",
 	"wcr","","reset all write changes in cache",
 	"wci","","commit write cache",
+	"wcf"," [file]","commit write cache into given file",
 	"wcp"," [fd]", "list all cached write-operations on p-layer for specified fd or current fd",
 	"wcp*"," [fd]","list all cached write-operations on p-layer in radare commands",
 	"wcpi"," [fd]", "commit and invalidate pcache for specified fd or current fd",
@@ -102,6 +111,7 @@ static const char *help_msg_wo[] = {
 	"woD","[algo] [key] [IV]","decrypt current block with given algo and key",
 	"woe"," [from to] [step] [wsz=1]","..  create sequence",
 	"woE"," [algo] [key] [IV]", "encrypt current block with given algo and key",
+	"woi","", "inverse bytes in current block",
 	"wol"," [val]","<<= shift left",
 	"wom"," [val]", "*=  multiply",
 	"woo"," [val]","|=  or",
@@ -160,7 +170,7 @@ static const char *help_msg_wv[] = {
 	"wv", " 0x834002", "write dword with this value",
 	"wv1", " 234", "write one byte with this value",
 	"wv2", " 234", "write unsigned short (2 bytes) with this number",
-	"wv4", " 234", "write dword (4 bytes) with this number",
+	"wv4", " 1 2 3", "write N space-separated dword (4 bytes)",
 	"wv8", " 234", "write qword (8 bytes) with this number",
 	"wvf", " 3.14", "write float value (4 bytes)",
 	"wvF", " 3.14", "write double value (8 bytes)",
@@ -172,13 +182,13 @@ static const char *help_msg_wv[] = {
 static const char *help_msg_wx[] = {
 	"Usage:", "wx[f] [arg]", "",
 	"wx", " 9090", "write two intel nops",
+	"wx+", " 9090", "write hexpairs and seek forward",
 	"wxf", " -|file", "write contents of hexpairs file here",
-	"wxs", " 9090", "write hexpairs and seek at the end",
 	NULL
 };
 
 static void cmd_write_fail(RCore *core) {
-	eprintf ("Failed to write\n");
+	eprintf ("ERROR: Cannot write in here, check map permissions or reopen the file with oo+\n");
 	core->num->value = 1;
 }
 
@@ -225,18 +235,17 @@ static bool encrypt_or_decrypt_block(RCore *core, const char *algo, const char *
 		binkey = (ut8 *)strdup (key);
 		keylen = r_hex_str2bin (key, binkey);
 	}
+	if (!binkey) {
+		eprintf ("Cannot allocate %d byte(s)\n", keylen);
+		return false;
+	}
 	if (!no_key_mode && keylen < 1) {
-		eprintf ("%s key not defined. Use -S [key]\n", ((!direction) ? "Encryption" : "Decryption"));
+		eprintf ("%s key not defined. Use -S [key]\n", ((!direction)? "Encryption": "Decryption"));
 		free (binkey);
 		return false;
 	}
 	RCrypto *cry = r_crypto_new ();
 	if (r_crypto_use (cry, algo)) {
-		if (!binkey) {
-			eprintf ("Cannot allocate %d byte(s)\n", keylen);
-			r_crypto_free (cry);
-			return false;
-		}
 		if (r_crypto_set_key (cry, binkey, keylen, 0, direction)) {
 			if (iv) {
 				ut8 *biniv = malloc (strlen (iv) + 1);
@@ -251,7 +260,6 @@ static bool encrypt_or_decrypt_block(RCore *core, const char *algo, const char *
 				}
 			}
 			r_crypto_update (cry, (const ut8*)core->block, core->blocksize);
-			r_crypto_final (cry, NULL, 0);
 
 			int result_size = 0;
 			ut8 *result = r_crypto_get_output (cry, &result_size);
@@ -312,27 +320,28 @@ static int wo_handler_old(void *data, const char *input) {
 	int len;
 	int value;
 	switch (input[0]) {
-	case 'e':
+	case 'e': // "woe"
 		if (input[1]!=' ') {
 			r_cons_printf ("Usage: 'woe from-to step'\n");
 			return -1;
 		}
-		/* fallthru */
-	case 'a':
-	case 's':
-	case 'A':
-	case 'x':
-	case 'r':
-	case 'l':
-	case 'm':
-	case 'd':
-	case 'o':
-	case 'w':
+		/* fallthrough */
+	case 'a': // "woa"
+	case 's': // "wos"
+	case 'A': // "woA"
+	case 'x': // "wox"
+	case 'r': // "wor"
+	case 'l': // "wol"
+	case 'm': // "wom"
+	case 'i': // "woi"
+	case 'd': // "wod"
+	case 'o': // "woo"
+	case 'w': // "wow"
 	case '2': // "wo2"
 	case '4': // "wo4"
 	case '8': // "wo8"
 		if (input[1]) {  // parse val from arg
-			r_core_write_op (core, input + 2, input[0]);
+			r_core_write_op (core, r_str_trim_head_ro (input + 2), input[0]);
 		} else {  // use clipboard instead of val
 			r_core_write_op (core, NULL, input[0]);
 		}
@@ -503,127 +512,69 @@ static void cmd_write_value(RCore *core, const char *input) {
 	case '4': type = 4; break;
 	case '8': type = 8; break;
 	}
-	if (input[0] && input[1]) {
-		off = r_num_math (core->num, input+1);
-	}
-	if (core->io->desc) {
-		r_io_use_fd (core->io, core->io->desc->fd);
-	}
-	ut64 res = r_io_seek (core->io, core->offset, R_IO_SEEK_SET);
-	if (res == UT64_MAX) return;
-	if (type == 0) {
-		type = (off&UT64_32U)? 8: 4;
-	}
-	switch (type) {
-	case 1:
-		r_write_ble8 (buf, (ut8)(off & UT8_MAX));
-		if (!r_io_write (core->io, buf, 1)) {
-			cmd_write_fail (core);
-		} else {
-			WSEEK (core, 1);
+	ut64 addr = core->offset;
+	char *inp = r_str_trim_dup (input + 1);
+	RList *list = r_str_split_list (inp, " ", 0); // or maybe comma :?
+	char *cinp;
+	RListIter *iter;
+	r_list_foreach (list, iter, cinp) {
+		if (input[0] && input[1]) {
+			off = r_num_math (core->num, cinp);
 		}
-		break;
-	case 2:
-		r_write_ble16 (buf, (ut16)(off & UT16_MAX), be);
-		if (!r_io_write (core->io, buf, 2)) {
-			cmd_write_fail (core);
-		} else {
-			WSEEK (core, 2);
+		if (core->io->desc) {
+			r_io_use_fd (core->io, core->io->desc->fd);
 		}
-		break;
-	case 4:
-		r_write_ble32 (buf, (ut32)(off & UT32_MAX), be);
-		if (!r_io_write (core->io, buf, 4)) {
-			cmd_write_fail (core);
-		} else {
-			WSEEK (core, 4);
+		ut64 res = r_io_seek (core->io, addr, R_IO_SEEK_SET);
+		if (res == UT64_MAX) {
+			return;
 		}
-		break;
-	case 8:
-		r_write_ble64 (buf, off, be);
-		if (!r_io_write (core->io, buf, 8)) {
-			cmd_write_fail (core);
-		} else {
-			WSEEK (core, 8);
+		if (type == 0) {
+			type = (off & UT64_32U)? 8: 4;
 		}
-		break;
+		switch (type) {
+		case 1:
+			r_write_ble8 (buf, (ut8)(off & UT8_MAX));
+			if (!r_io_write (core->io, buf, 1)) {
+				cmd_write_fail (core);
+			} else {
+				WSEEK (core, 1);
+			}
+			break;
+		case 2:
+			r_write_ble16 (buf, (ut16)(off & UT16_MAX), be);
+			if (!r_io_write (core->io, buf, 2)) {
+				cmd_write_fail (core);
+			} else {
+				WSEEK (core, 2);
+			}
+			break;
+		case 4:
+			r_write_ble32 (buf, (ut32)(off & UT32_MAX), be);
+			if (!r_io_write (core->io, buf, 4)) {
+				cmd_write_fail (core);
+			} else {
+				WSEEK (core, 4);
+			}
+			break;
+		case 8:
+			r_write_ble64 (buf, off, be);
+			if (!r_io_write (core->io, buf, 8)) {
+				cmd_write_fail (core);
+			} else {
+				WSEEK (core, 8);
+			}
+			break;
+		}
+		addr += type;
 	}
+	r_list_free (list);
+	free (inp);
 	r_core_block_read (core);
 }
 
-static RCmdStatus common_wv_handler(RCore *core, int argc, const char **argv, int type) {
-	ut64 off = 0LL;
-	ut8 buf[sizeof(ut64)];
-	int wseek = r_config_get_i (core->config, "cfg.wseek");
-	bool be = r_config_get_i (core->config, "cfg.bigendian");
-
-	core->num->value = 0;
-	if (argc != 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-
-	off = r_num_math (core->num, argv[1]);
-	if (core->io->desc) {
-		r_io_use_fd (core->io, core->io->desc->fd);
-	}
-
-	ut64 res = r_io_seek (core->io, core->offset, R_IO_SEEK_SET);
-	if (res == UT64_MAX) {
-		return R_CMD_STATUS_ERROR;
-	}
-	if (type == 0) {
-		type = off & UT64_32U? 8: 4;
-	}
-
-	switch (type) {
-	case 1:
-		r_write_ble8 (buf, (ut8)(off & UT8_MAX));
-		break;
-	case 2:
-		r_write_ble16 (buf, (ut16)(off & UT16_MAX), be);
-		break;
-	case 4:
-		r_write_ble32 (buf, (ut32)(off & UT32_MAX), be);
-		break;
-	case 8:
-		r_write_ble64 (buf, off, be);
-		break;
-	}
-
-	if (!r_io_write (core->io, buf, type)) {
-		cmd_write_fail (core);
-	} else if (wseek) {
-		r_core_seek_delta (core, type);
-	}
-
-	r_core_block_read (core);
-	return R_CMD_STATUS_OK;
-}
-
-static RCmdStatus wv_handler(RCore *core, int argc, const char **argv) {
-	return common_wv_handler (core, argc, argv, 0);
-}
-
-static RCmdStatus wv1_handler(RCore *core, int argc, const char **argv) {
-	return common_wv_handler (core, argc, argv, 1);
-}
-
-static RCmdStatus wv2_handler(RCore *core, int argc, const char **argv) {
-	return common_wv_handler (core, argc, argv, 2);
-}
-
-static RCmdStatus wv4_handler(RCore *core, int argc, const char **argv) {
-	return common_wv_handler (core, argc, argv, 4);
-}
-
-static RCmdStatus wv8_handler(RCore *core, int argc, const char **argv) {
-	return common_wv_handler (core, argc, argv, 8);
-}
-
 static bool cmd_wff(RCore *core, const char *input) {
-	ut8 *buf;
-	size_t size;
-	// XXX: file names cannot contain spaces
+	ut8 *buf = NULL;
+	size_t size = 0;
 	const char *arg = input + ((input[0] == ' ') ? 1 : 0);
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
 	char *p, *a = r_str_trim_dup (arg);
@@ -634,8 +585,7 @@ static bool cmd_wff(RCore *core, const char *input) {
 
 	if (*arg =='?' || !*arg) {
 		eprintf ("Usage: wf [file] ([size] ([offset]))\n");
-	}
-	if (!strcmp (arg, "-")) {
+	} else if (!strcmp (arg, "-")) {
 		char *out = r_core_editor (core, NULL, NULL);
 		if (out) {
 			if (!r_io_write_at (core->io, core->offset,
@@ -646,7 +596,28 @@ static bool cmd_wff(RCore *core, const char *input) {
 			free (out);
 		}
 	}
-	if ((buf = (ut8*) r_file_slurp (a, &size))) {
+
+	if (*a == '$' && !a[1]) {
+		eprintf ("No alias name given.\n");
+	} else if (*a == '$') {
+		RCmdAliasVal *v = r_cmd_alias_get (core->rcmd, a+1);
+		if (v) {
+			buf = malloc (v->sz);
+			if (buf) {
+				size = v->sz;
+				memcpy (buf, v->data, size);
+			} else {
+				size = 0;
+			}
+		} else {
+			eprintf ("No such alias \"$%s\"\n", a+1);
+		}
+	} else {
+		buf = (ut8*) r_file_slurp (a, &size);
+	}
+	if (size < 1) {
+		// nothing to write
+	} else if (buf) {
 		int u_offset = 0;
 		ut64 u_size = r_num_math (core->num, p);
 		if (u_size < 1) u_size = (ut64)size;
@@ -655,6 +626,7 @@ static bool cmd_wff(RCore *core, const char *input) {
 			u_offset = r_num_math (core->num, p);
 			if (u_offset > size) {
 				eprintf ("Invalid offset\n");
+				free (a);
 				free (buf);
 				return false;
 			}
@@ -664,15 +636,16 @@ static bool cmd_wff(RCore *core, const char *input) {
 			eprintf ("r_io_write_at failed at 0x%08"PFMT64x"\n", core->offset);
 		}
 		WSEEK (core, size);
-		free (buf);
 		r_core_block_read (core);
 	} else {
 		eprintf ("Cannot open file '%s'\n", arg);
 	}
+	free (a);
+	free (buf);
 	return true;
 }
 
-static bool ioMemcpy (RCore *core, ut64 dst, ut64 src, int len) {
+static bool ioMemcpy(RCore *core, ut64 dst, ut64 src, int len) {
 	bool ret = false;
 	if (len > 0) {
 		ut8 * buf = calloc (1, len);
@@ -777,7 +750,7 @@ static bool cmd_wfs(RCore *core, const char *input) {
 		if (r_io_write_at (core->io, core->offset, buf, done)) {
 			eprintf ("Written %d bytes\n", done);
 		} else {
-			eprintf ("Cannot write\n");
+			cmd_write_fail (core);
 		}
 	}
 	r_socket_free (s);
@@ -898,22 +871,6 @@ static int wB_handler_old(void *data, const char *input) {
 	return 0;
 }
 
-static RCmdStatus wB_handler(RCore *core, int argc, const char **argv) {
-	if (argc != 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-	cmd_write_bits (core, 1, r_num_math (core->num, argv[1]));
-	return R_CMD_STATUS_OK;
-}
-
-static RCmdStatus wB_minus_handler(RCore *core, int argc, const char **argv) {
-	if (argc != 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-	cmd_write_bits (core, 0, r_num_math (core->num, argv[1]));
-	return R_CMD_STATUS_OK;
-}
-
 static int w0_handler_common(RCore *core, ut64 len) {
 	int res = 0;
 	if (len > 0) {
@@ -939,14 +896,6 @@ static int w0_handler_old(void *data, const char *input) {
 	return w0_handler_common (core, len);
 }
 
-static RCmdStatus w0_handler(RCore *core, int argc, const char **argv) {
-	if (argc != 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-	ut64 len = r_num_math (core->num, argv[1]);
-	return r_cmd_int2status (w0_handler_common (core, len));
-}
-
 static int w_incdec_handler_old(void *data, const char *input, int inc) {
 	RCore *core = (RCore *)data;
 	st64 num = 1;
@@ -966,35 +915,6 @@ static int w_incdec_handler_old(void *data, const char *input, int inc) {
 	return 0;
 }
 
-static RCmdStatus w_incdec_handler(RCore *core, int argc, const char **argv, int inc_size) {
-	if (argc > 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-	st64 num = argc > 1? r_num_math (core->num, argv[1]): 1;
-	const char *command = argv[0];
-	if (command[strlen (command) - 1] == '-') {
-		num *= -1;
-	}
-	cmd_write_inc (core, inc_size, num);
-	return R_CMD_STATUS_OK;
-}
-
-static RCmdStatus w1_incdec_handler(RCore *core, int argc, const char **argv) {
-	return w_incdec_handler (core, argc, argv, 1);
-}
-
-static RCmdStatus w2_incdec_handler(RCore *core, int argc, const char **argv) {
-	return w_incdec_handler (core, argc, argv, 2);
-}
-
-static RCmdStatus w4_incdec_handler(RCore *core, int argc, const char **argv) {
-	return w_incdec_handler (core, argc, argv, 4);
-}
-
-static RCmdStatus w8_incdec_handler(RCore *core, int argc, const char **argv) {
-	return w_incdec_handler (core, argc, argv, 8);
-}
-
 static int w6_handler_old(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
@@ -1297,7 +1217,7 @@ static int wr_handler_old(void *data, const char *input) {
 	int len = (int)off;
 	if (len > 0) {
 		ut8 *buf = malloc (len);
-		if (buf != NULL) {
+		if (buf) {
 			int i;
 			r_num_irand ();
 			for (i = 0; i < len; i++)
@@ -1343,6 +1263,53 @@ static int wA_handler_old(void *data, const char *input) {
 	return 0;
 }
 
+static char *__current_filename(RCore *core) {
+	RIOMap *map = r_io_map_get_at (core->io, core->offset);
+	if (map) {
+		RIODesc *desc = r_io_desc_get (core->io, map->fd);
+		if (desc) {
+			return strdup (desc->uri);
+		}
+	}
+	return NULL;
+}
+
+static ut64 __va2pa(RCore *core, ut64 va) {
+	RIOMap *map = r_io_map_get_at (core->io, va);
+	if (map) {
+		return va - map->itv.addr + map->delta;
+	}
+	return va;
+}
+
+static void cmd_wcf(RCore *core, const char *dfn) {
+	char *sfn = __current_filename (core);
+	if (!sfn) {
+		eprintf ("Cannot determine source file.\n");
+		return;
+	}
+	size_t sfs;
+	ut8 *sfb = (ut8*)r_file_slurp (sfn, &sfs);
+	if (sfb) {
+		void **iter;
+		r_pvector_foreach (&core->io->cache, iter) {
+			RIOCache *c = *iter;
+			const ut64 ps = r_itv_size (c->itv);
+			const ut64 va = r_itv_begin (c->itv);
+			const ut64 pa = __va2pa (core, va);
+			if (pa + ps < sfs) {
+				memcpy (sfb + pa, c->data, ps);
+			} else {
+				eprintf ("Out of bounds patch at 0x%08"PFMT64x"\n", pa);
+			}
+		}
+		// patch buffer
+		r_file_dump (dfn, sfb, sfs, false);
+		free (sfb);
+	}
+	free (sfn);
+}
+
 static int wc_handler_old(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	switch (input[0]) {
@@ -1354,6 +1321,13 @@ static int wc_handler_old(void *data, const char *input) {
 	case '?': // "wc?"
 		r_core_cmd_help (core, help_msg_wc);
 		break;
+	case 'f': // "wcf"
+		if (input[1] == ' ') {
+			cmd_wcf (core, r_str_trim_head_ro (input + 1));
+		} else {
+			eprintf ("Usage: wcf [file]\n");
+		}
+		break;
 	case '*': // "wc*"
 		r_io_cache_list (core->io, 1);
 		break;
@@ -1362,13 +1336,13 @@ static int wc_handler_old(void *data, const char *input) {
 			//r_io_cache_reset (core->io, core->io->cached);
 			eprintf ("TODO\n");
 		} else if (input[1]==' ') { // "wc+ "
-			char *p = strchr (input+2, ' ');
+			char *p = strchr (input + 2, ' ');
 			ut64 to, from;
 			from = r_num_math (core->num, input+2);
 			if (p) {
 				*p = 0;
 				to = r_num_math (core->num, input+2);
-				if (to<from) {
+				if (to < from) {
 					eprintf ("Invalid range (from>to)\n");
 					return 0;
 				}
@@ -1433,35 +1407,22 @@ static int wc_handler_old(void *data, const char *input) {
 	return 0;
 }
 
-static void w_handler_common(RCore *core, const char *input) {
+static int w_handler(RCore *core, const char *input) {
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
 	char *str = strdup (input);
 	/* write string */
 	int len = r_str_unescape (str);
+	// handle charset logic here
 	if (!r_core_write_at (core, core->offset, (const ut8 *)str, len)) {
 		cmd_write_fail (core);
 	}
 	free (str);
 	WSEEK (core, len);
 	r_core_block_read (core);
-}
-
-static int w_handler_old(void *data, const char *input) {
-	RCore *core = (RCore *)data;
-	w_handler_common (core, input);
+	core->num->value = len;
 	return 0;
 }
 
-static RCmdStatus w_handler(RCore *core, int argc, const char **argv) {
-	if (argc < 2) {
-		return R_CMD_STATUS_WRONG_ARGS;
-	}
-	char *s = r_str_array_join (argv + 1, argc - 1, " ");
-	w_handler_common (core, s);
-	free (s);
-	return R_CMD_STATUS_OK;
-}
-
 static int wz_handler_old(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
@@ -1476,11 +1437,9 @@ static int wz_handler_old(void *data, const char *input) {
 	} else {
 		core->num->value = 0;
 	}
-#if 0
-		r_io_use_desc (core->io, core->file->desc);
-#endif
 	WSEEK (core, len + 1);
 	r_core_block_read (core);
+	free (str);
 	return 0;
 }
 
@@ -1488,10 +1447,11 @@ static int wt_handler_old(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	char *str = strdup (input);
 	char *ostr = str;
+	char *hfilename = NULL;
 	const char *filename = "";
 	char _fn[32];
 	_fn[0] = 0;
-	char *tmp;
+	char *size_sep;
 	if (*str == 's') { // "wts"
 		if (str[1] == ' ') {
 			eprintf ("Write to server\n");
@@ -1580,6 +1540,14 @@ static int wt_handler_old(void *data, const char *input) {
 						return 0;
 					}
 					filename = r_str_trim_head_ro (str);
+					if (r_str_startswith (filename, "base64:")) {
+						const char *encoded = filename + 7;
+						char *decoded = (char *)sdb_decode (encoded, NULL);
+						if (decoded) {
+							hfilename = decoded;
+							filename = decoded;
+						}
+					}
 				} else {
 					filename = "";
 				}
@@ -1601,27 +1569,47 @@ static int wt_handler_old(void *data, const char *input) {
 		} else {
 			filename = str + 1;
 		}
-		tmp = *str? strchr (str + 1, ' ') : NULL;
+		size_sep = *str? strchr (str + 1, ' ') : NULL;
 		if (!filename || !*filename) {
 			const char* prefix = r_config_get (core->config, "cfg.prefixdump");
 			snprintf (_fn, sizeof (_fn), "%s.0x%08"PFMT64x, prefix, poff);
 			filename = _fn;
 		}
-		if (tmp) {
+
+		/* TODO: I think this logic can be cleaned up, size_sep looks tacked-on */
+		if (size_sep) {
 			if (toend) {
 				sz = r_io_fd_size (core->io, core->io->desc->fd) - core->offset;
 				if (sz < 0) {
 					eprintf ("Warning: File size is unknown.");
 				}
 			} else {
-				sz = (st64) r_num_math (core->num, tmp + 1);
-				*tmp = 0;
+				sz = (st64) r_num_math (core->num, size_sep + 1);
+				/* Don't attempt to write if we can't parse size */
+				if (sz < 1) {
+					eprintf ("%s is not a valid size.\n", size_sep + 1);
+					sz = -1;
+				}
+				*size_sep = '\0';
 			}
-			if ((st64)sz < 1) {
-				// wtf?
-				sz = 0;
-			} else if (!r_core_dump (core, filename, poff, (ut64)sz, append)) {
-				sz = -1;
+
+			// XXX: this branch didn't handle aliases at all before
+			// but im not fixing the duplication rn
+			if (*filename == '$') {
+				if (append) {
+					if (sz > 0 && r_cmd_alias_append_raw (core->rcmd, filename+1, core->block, sz)) {
+						eprintf ("Alias \"$%s\" is a command - will not attempt to append.\n", filename+1);
+					}
+				} else {
+					if (sz > 0) {
+						r_cmd_alias_set_raw (core->rcmd, filename+1, core->block, sz);
+					}
+				}
+			} else {
+				/* XXX: r_core_dump vs r_file_dump below? */
+				if (sz > 0 && !r_core_dump (core, filename, poff, (ut64)sz, append)) {
+					sz = -1;
+				}
 			}
 		} else {
 			if (toend) {
@@ -1638,9 +1626,18 @@ static int wt_handler_old(void *data, const char *input) {
 					sz = -1;
 				}
 			} else {
-				sz = core->blocksize;
-				if (!r_file_dump (filename, core->block, sz, append)) {
-					sz = -1;
+				if (*filename == '$') {
+					if (append) {
+						if (r_cmd_alias_append_raw (core->rcmd, filename+1, core->block, sz)) {
+							eprintf ("Alias \"$%s\" is a command - will not attempt to append.\n", filename+1);
+						}
+					} else {
+						r_cmd_alias_set_raw (core->rcmd, filename+1, core->block, sz);
+					}
+				} else {
+					if (sz > 0 && !r_file_dump (filename, core->block, sz, append)) {
+						sz = -1;
+					}
 				}
 			}
 		}
@@ -1649,6 +1646,8 @@ static int wt_handler_old(void *data, const char *input) {
 					sz, poff, filename);
 		}
 	}
+	free (ostr);
+	free (hfilename);
 	return 0;
 }
 
@@ -1739,6 +1738,9 @@ static int wx_handler_old(void *data, const char *input) {
 		}
 		break;
 	case 's': // "wxs"
+		eprintf ("Warning: wxs has been renamed to wx+\n");
+		// fallthrough
+	case '+': // "wx+"
 		{
 			int len = cmd_write_hexpair (core, input + 1);
 			if (len > 0) {
@@ -1773,13 +1775,33 @@ static int wa_handler_old(void *data, const char *input) {
 		break;
 	case ' ':
 	case 'i':
+	case 'n':
 	case '*': {
 		const char *file = r_str_trim_head_ro (input + 1);
 		RAsmCode *acode;
 		r_asm_set_pc (core->rasm, core->offset);
 		acode = r_asm_massemble (core->rasm, file);
 		if (acode) {
-			if (input[0] == 'i') { // "wai"
+			if (input[0] == 'n') { // "wan"
+				int delta = 0;
+				RAnalOp analop;
+				ut64 at = core->offset;
+repeat:
+				if (!r_anal_op (core->anal, &analop, at, core->block + delta, core->blocksize - delta, R_ANAL_OP_MASK_BASIC)) {
+					eprintf ("Invalid instruction?\n");
+					break;
+				}
+				if (delta < acode->len) {
+					delta += analop.size;
+					at += analop.size;
+					r_anal_op_fini (&analop);
+					r_core_cmdf (core, "wao nop @ 0x%08"PFMT64x, at);
+					goto repeat;
+				}
+				r_anal_op_fini (&analop);
+				r_core_cmd0 (core, "wao nop");
+				input++;
+			} else if (input[0] == 'i') { // "wai"
 				RAnalOp analop;
 				if (!r_anal_op (core->anal, &analop, core->offset, core->block, core->blocksize, R_ANAL_OP_MASK_BASIC)) {
 					eprintf ("Invalid instruction?\n");
@@ -1920,7 +1942,7 @@ static int wb_handler_old(void *data, const char *input) {
 		}
 		free (buf);
 	} else {
-		eprintf ("Cannot malloc %zd\n", buf_size);
+		eprintf ("Cannot malloc %d\n", (int)buf_size);
 	}
 	return 0;
 }
@@ -1987,23 +2009,65 @@ static int ws_handler_old(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 	int wseek = r_config_get_i (core->config, "cfg.wseek");
 	char *str = strdup (input);
-	if (str && *str && str[1]) {
-		int len = r_str_unescape (str + 1);
-		if (len > 255) {
+	if (str && *str) {
+		char *arg = str;
+		int pss = 1;
+		int maxlen = 255;
+		if (*str == ' ') {
+			arg++;
+		} else switch (*str) {
+		case '1':
+			pss = 1;
+			break;
+		case '2':
+			pss = 2;
+			maxlen = UT16_MAX;
+			break;
+		case '4':
+			pss = 4;
+			maxlen = UT32_MAX;
+			break;
+		default:
+			pss = 0;
+			break;
+		}
+		arg = strchr (str, ' ');
+		if (!arg || !pss) {
+			r_core_cmd_help (core, help_msg_ws);
+			free (str);
+			return 0;
+		}
+		arg = (char *)r_str_trim_head_ro (arg + 1);
+		ut64 len = r_str_unescape ((char *)arg);
+		if (len > maxlen) {
 			eprintf ("Too large\n");
 		} else {
-			ut8 ulen = (ut8)len;
-			if (!r_core_write_at (core, core->offset, &ulen, 1) ||
-				!r_core_write_at (core, core->offset + 1, (const ut8 *)str + 1, len)) {
-				cmd_write_fail (core);
-			} else {
-				WSEEK (core, len);
+			ut8 lenbuf[4] = {0};
+			// write string length
+			switch (pss) {
+			case 1:
+				r_write_ble8 (lenbuf, len);
+				r_io_write_at (core->io, core->offset, lenbuf, 1);
+				break;
+			case 2:
+				r_write_ble16 (lenbuf, len, core->anal->big_endian);
+				r_io_write_at (core->io, core->offset, lenbuf, 2);
+				break;
+			case 4:
+				r_write_ble32 (lenbuf, len, core->anal->big_endian);
+				r_io_write_at (core->io, core->offset, lenbuf, 4);
+				break;
 			}
+			if (!r_core_write_at (core, core->offset + pss, (const ut8 *)arg, len)) {
+				cmd_write_fail (core);
+			}
+			WSEEK (core, len);
 			r_core_block_read (core);
 		}
 	} else {
-		eprintf ("Too short.\n");
+		r_core_cmd_help (core, help_msg_ws);
 	}
+	free (str);
 	return 0;
 }
 
@@ -2065,23 +2129,40 @@ static int cmd_write(void *data, const char *input) {
 		wA_handler_old (core, input + 1);
 		break;
 	case ' ': // "w"
+	case '+': // "w+"
 	{
 		size_t len = core->blocksize;
 		const char *curcs = r_config_get (core->config, "cfg.charset");
+		char *str = strdup (input);
+
+		r_str_trim_args (str);
+		r_str_trim_tail (str);
+
+		ut64 addr = core->offset;
 		if (R_STR_ISEMPTY (curcs)) {
-			w_handler_old (core, input + 1);
+			core->num->value = 0;
+			w_handler (core, str + 1);
+			addr += core->num->value;
 		} else {
 			if (len > 0) {
-				size_t in_len = strlen (input + 1);
+				size_t in_len = strlen (str + 1);
 				int max = core->print->charset->encode_maxkeylen;
+				int out_len = in_len * max;
+				int new_len = 0;
 				ut8 *out = malloc (in_len * max); //suppose in len = out len TODO: change it
 				if (out) {
-					r_charset_decode_str (core->print->charset, out, in_len, (const unsigned char *) input + 1, in_len);
-					w_handler_old (core, (const char *)out);
+					*out = 0;
+					new_len = r_charset_decode_str (core->print->charset, out, out_len, (const ut8*) str + 1, in_len);
+					w_handler (core, (const char *)out);
 					free (out);
 				}
+				addr += new_len;
 			}
 		}
+		free (str);
+		if (*input == '+') {
+			r_core_seek (core, addr, true);
+		}
 		break;
 	}
 	case 'z': // "wz"
@@ -2122,62 +2203,3 @@ static int cmd_write(void *data, const char *input) {
 	r_core_block_read (core);
 	return 0;
 }
-
-static void cmd_write_init(RCore *core, RCmdDesc *parent) {
-	DEFINE_CMD_DESCRIPTOR (core, w);
-	DEFINE_CMD_DESCRIPTOR (core, wa);
-	DEFINE_CMD_DESCRIPTOR (core, wA);
-	DEFINE_CMD_DESCRIPTOR (core, wc);
-	DEFINE_CMD_DESCRIPTOR (core, we);
-	DEFINE_CMD_DESCRIPTOR (core, wo);
-	DEFINE_CMD_DESCRIPTOR (core, wop);
-	DEFINE_CMD_DESCRIPTOR (core, wp);
-	DEFINE_CMD_DESCRIPTOR (core, wt);
-	DEFINE_CMD_DESCRIPTOR (core, wv);
-	DEFINE_CMD_DESCRIPTOR (core, wx);
-
-	DEFINE_CMD_ARGV_GROUP_WITH_CHILD (core, wB, parent);
-	DEFINE_CMD_ARGV_DESC_SPECIAL (core, wB-, wB_minus, wB_cd);
-
-	DEFINE_CMD_ARGV_GROUP_WITH_CHILD (core, wv, parent);
-	DEFINE_CMD_ARGV_DESC (core, wv1, wv_cd);
-	DEFINE_CMD_ARGV_DESC (core, wv2, wv_cd);
-	DEFINE_CMD_ARGV_DESC (core, wv4, wv_cd);
-	DEFINE_CMD_ARGV_DESC (core, wv8, wv_cd);
-
-	DEFINE_CMD_ARGV_DESC (core, w0, parent);
-
-	DEFINE_CMD_ARGV_DESC_INNER (core, w, w_incdec, parent);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w1, w1, w_incdec_cd, NULL, &w1_incdec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w1+, w1_inc, w1_cd, w1_incdec_handler, &w1_inc_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w1-, w1_dec, w1_cd, w1_incdec_handler, &w1_dec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w2, w2, w_incdec_cd, NULL, &w2_incdec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w2+, w2_inc, w2_cd, w2_incdec_handler, &w2_inc_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w2-, w2_dec, w2_cd, w2_incdec_handler, &w2_dec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w4, w4, w_incdec_cd, NULL, &w4_incdec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w4+, w4_inc, w4_cd, w4_incdec_handler, &w4_inc_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w4-, w4_dec, w4_cd, w4_incdec_handler, &w4_dec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w8, w8, w_incdec_cd, NULL, &w8_incdec_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w8+, w8_inc, w8_cd, w8_incdec_handler, &w8_inc_help);
-	DEFINE_CMD_ARGV_DESC_DETAIL (core, w8-, w8_dec, w8_cd, w8_incdec_handler, &w8_dec_help);
-
-	DEFINE_CMD_OLDINPUT_DESC (core, w6, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wh, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, we, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wp, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wu, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wr, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wA, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wc, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wz, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wt, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wf, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, ww, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wx, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wa, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wb, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wm, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wo, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, wd, parent);
-	DEFINE_CMD_OLDINPUT_DESC (core, ws, parent);
-}
