@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2021 - pancake, nibble */
 
 #include <r_core.h>
 #include <r_anal.h>
@@ -12,6 +12,7 @@ static const char *help_msg_z[] = {
 	"z", "", "show zignatures",
 	"z.", "", "find matching zignatures in current offset",
 	"zb", "[?][n=5]", "search for best match",
+	"zd", "zignature", "diff current function and signature",
 	"z*", "", "show zignatures in radare format",
 	"zq", "", "show zignatures in quiet mode",
 	"zj", "", "show zignatures in json format",
@@ -37,17 +38,20 @@ static const char *help_msg_zb[] = {
 };
 
 static const char *help_msg_z_slash[] = {
-	"Usage:", "z/[*] ", "# Search signatures (see 'e?search' for options)",
+	"Usage:", "z/[f*] ", "# Search signatures (see 'e?search' for options)",
 	"z/ ", "", "search zignatures on range and flag matches",
+	"z/f ", "", "zignature search on known functions",
 	"z/* ", "", "search zignatures on range and output radare commands",
 	NULL
 };
 
 static const char *help_msg_za[] = {
-	"Usage:", "za[fF?] [args] ", "# Add zignature",
+	"Usage:", "za[fFM?] [args] ", "# Add zignature",
 	"za ", "zigname type params", "add zignature",
+	"zac ", "", "Compute collisions between signatures",
 	"zaf ", "[fcnname] [zigname]", "create zignature for function",
 	"zaF ", "", "generate zignatures for all functions",
+	"zaM ", "", "Same as zaF but merge signatures of same name",
 	"za?? ", "", "show extended help",
 	NULL
 };
@@ -90,109 +94,6 @@ static const char *help_msg_zc[] = {
 	NULL
 };
 
-static void cmd_zign_init(RCore *core, RCmdDesc *parent) {
-	DEFINE_CMD_DESCRIPTOR (core, z);
-	DEFINE_CMD_DESCRIPTOR (core, zb);
-	DEFINE_CMD_DESCRIPTOR_SPECIAL (core, z/, z_slash);
-	DEFINE_CMD_DESCRIPTOR (core, za);
-	DEFINE_CMD_DESCRIPTOR (core, zf);
-	DEFINE_CMD_DESCRIPTOR (core, zo);
-	DEFINE_CMD_DESCRIPTOR (core, zs);
-	DEFINE_CMD_DESCRIPTOR (core, zc);
-}
-
-#if 0
-static char *getFcnComments(RCore *core, RAnalFunction *fcn) {
-	// XXX this is slow as hell on big binaries
-	char *r = r_core_cmd_strf (core, "CCf* @ 0x%08"PFMT64x, fcn->addr);
-	if (r && *r) {
-		return r;
-	}
-	//
-	return NULL;
-}
-#endif
-
-static void addFcnZign(RCore *core, RAnalFunction *fcn, const char *name) {
-	char *ptr = NULL;
-	char *zignspace = NULL;
-	char *zigname = NULL;
-	const RSpace *curspace = r_spaces_current (&core->anal->zign_spaces);
-	int len = 0;
-
-	if (name) {
-		zigname = r_str_new (name);
-	} else {
-		// If the user has set funtion names containing a single ':' then we assume
-		// ZIGNSPACE:FUNCTION, and for now we only support the 'zg' command
-		if ((ptr = strchr (fcn->name, ':')) != NULL) {
-			len = ptr - fcn->name;
-			zignspace = r_str_newlen (fcn->name, len);
-			r_spaces_push (&core->anal->zign_spaces, zignspace);
-		} else if (curspace) {
-			zigname = r_str_newf ("%s:", curspace->name);
-		}
-		zigname = r_str_appendf (zigname, "%s", fcn->name);
-	}
-
-	// create empty item
-	RSignItem *it = r_sign_item_new ();
-	if (!it) {
-		free (zigname);
-		return;
-	}
-	// add sig types info to item
-	it->name = zigname; // will be free'd when item is free'd
-	it->space = r_spaces_current (&core->anal->zign_spaces);
-	// TODO: sort fcn bbs
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_GRAPH);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_BYTES);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_XREFS);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_REFS);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_VARS);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_TYPES);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_BBHASH);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_OFFSET);
-	r_sign_addto_item (core->anal, it, fcn, R_SIGN_NAME);
-
-	/* r_sign_add_addr (core->anal, zigname, fcn->addr); */
-
-	// commit the item to anal
-	r_sign_add_item (core->anal, it);
-
-	/*
-	XXX this is very slow and poorly tested
-	char *comments = getFcnComments (core, fcn);
-	if (comments) {
-		r_sign_add_comment (core->anal, zigname, comments);
-	}
-	*/
-
-	r_sign_item_free (it); // causes zigname to be free'd
-	if (zignspace) {
-		r_spaces_pop (&core->anal->zign_spaces);
-		free (zignspace);
-	}
-}
-
-static bool addCommentZign(RCore *core, const char *name, RList *args) {
-	if (r_list_length (args) != 1) {
-		eprintf ("Invalid number of arguments\n");
-		return false;
-	}
-	const char *comment = (const char *)r_list_get_top (args);
-	return r_sign_add_comment (core->anal, name, comment);
-}
-
-static bool addNameZign(RCore *core, const char *name, RList *args) {
-	if (r_list_length (args) != 1) {
-		eprintf ("Invalid number of arguments\n");
-		return false;
-	}
-	const char *realname = (const char *)r_list_get_top (args);
-	return r_sign_add_name (core->anal, name, realname);
-}
-
 static bool addGraphZign(RCore *core, const char *name, RList *args) {
 	RSignGraph graph = { .cc = -1, .nbbs = -1, .edges = -1, .ebbs = -1, .bbsum = 0 };
 
@@ -216,225 +117,131 @@ static bool addGraphZign(RCore *core, const char *name, RList *args) {
 	return r_sign_add_graph (core->anal, name, graph);
 }
 
-static bool addHashZign(RCore *core, const char *name, int type, RList *args) {
-	if (r_list_length (args) != 1) {
-		eprintf ("error: invalid syntax\n");
-		return false;
-	}
-	const char *hash = (const char *)r_list_get_top (args);
-	int len = strlen (hash);
-	if (!len) {
-		return false;
-	}
-	return r_sign_add_hash (core->anal, name, type, hash, len);
-}
-
-static bool addBytesZign(RCore *core, const char *name, int type, RList *args) {
-	ut8 *mask = NULL, *bytes = NULL, *sep = NULL;
-	int size = 0;
-	bool retval = true;
-
-	if (r_list_length (args) != 1) {
-		eprintf ("error: invalid syntax\n");
+static inline bool za_add(RCore *core, const char *input) {
+	char *args = r_str_trim_dup (input + 1);
+	if (!args) {
 		return false;
 	}
 
-	const char *hexbytes = (const char *)r_list_get_top (args);
-	if ((sep = (ut8 *)strchr (hexbytes, ':'))) {
-		size_t blen = sep - (ut8 *)hexbytes;
-		sep++;
-		if (!blen || (blen & 1) || strlen ((char *)sep) != blen) {
-			eprintf ("error: cannot parse hexpairs\n");
-			return false;
-		}
-		bytes = calloc (1, blen + 1);
-		mask = calloc (1, blen + 1);
-		memcpy (bytes, hexbytes, blen);
-		memcpy (mask, sep, blen);
-		size = r_hex_str2bin ((char*) bytes, bytes);
-		if (size != blen / 2 || r_hex_str2bin ((char*)mask, mask) != size) {
-			eprintf ("error: cannot parse hexpairs\n");
-			retval = false;
-			goto out;
-		}
-	} else {
-		size_t blen = strlen (hexbytes) + 4;
-		bytes = malloc (blen);
-		mask = malloc (blen);
+	char *name = strtok (args, " ");
+	char *stype = strtok (NULL, " ");
+	char *sig = strtok (NULL, "");
 
-		size = r_hex_str2binmask (hexbytes, bytes, mask);
-		if (size <= 0) {
-			eprintf ("error: cannot parse hexpairs\n");
-			retval = false;
-			goto out;
-		}
+	if (!stype || !sig || stype[1] != '\0') {
+		eprintf ("Invalid input\n");
+		free (args);
+		return false;
 	}
+	char t = *stype;
 
-	switch (type) {
+	RList *lst = NULL;
+	bool ret = false;
+	switch (t) {
 	case R_SIGN_BYTES:
-		retval = r_sign_add_bytes (core->anal, name, size, bytes, mask);
+		ret = r_sign_add_bytes (core->anal, name, sig);
 		break;
 	case R_SIGN_ANAL:
-		retval = r_sign_add_anal (core->anal, name, size, bytes, 0);
+		ret = r_sign_add_anal (core->anal, name, sig);
 		break;
-	}
-
-out:
-	free (bytes);
-	free (mask);
-
-	return retval;
-}
-
-static bool addOffsetZign(RCore *core, const char *name, RList *args) {
-	if (r_list_length (args) != 1) {
-		eprintf ("error: invalid syntax\n");
-		return false;
-	}
-	const char *offstr = (const char *)r_list_get_top (args);
-	if (!offstr) {
-		return false;
-	}
-	ut64 offset = r_num_get (core->num, offstr);
-	return r_sign_add_addr (core->anal, name, offset);
-}
-
-static bool addZign(RCore *core, const char *name, int type, RList *args) {
-	switch (type) {
-	case R_SIGN_BYTES:
-	case R_SIGN_ANAL:
-		return addBytesZign (core, name, type, args);
 	case R_SIGN_GRAPH:
-		return addGraphZign (core, name, args);
+		lst = r_str_split_list (sig, " ", 0);
+		ret = addGraphZign (core, name, lst);
+		break;
 	case R_SIGN_COMMENT:
-		return addCommentZign (core, name, args);
+		ret = r_sign_add_comment (core->anal, name, sig);
+		break;
 	case R_SIGN_NAME:
-		return addNameZign (core, name, args);
-	case R_SIGN_OFFSET:
-		return addOffsetZign (core, name, args);
-	case R_SIGN_REFS:
-		return r_sign_add_refs (core->anal, name, args);
-	case R_SIGN_XREFS:
-		return r_sign_add_xrefs (core->anal, name, args);
-	case R_SIGN_VARS:
-		return r_sign_add_vars (core->anal, name, args);
+		ret = r_sign_add_name (core->anal, name, sig);
+		break;
 	case R_SIGN_TYPES:
-		return r_sign_add_types (core->anal, name, args);
+		ret = r_sign_add_types (core->anal, name, sig);
+		break;
+	case R_SIGN_OFFSET:
+		{
+			ut64 offset = r_num_get (core->num, sig);
+			ret = r_sign_add_addr (core->anal, name, offset);
+		}
+		break;
+	case R_SIGN_REFS:
+		lst = r_str_split_list (sig, " ", 0);
+		r_sign_add_refs (core->anal, name, lst);
+		break;
+	case R_SIGN_XREFS:
+		lst = r_str_split_list (sig, " ", 0);
+		r_sign_add_xrefs (core->anal, name, lst);
+		break;
+	case R_SIGN_VARS:
+		r_sign_add_vars (core->anal, name, sig);
+		break;
 	case R_SIGN_BBHASH:
-		return addHashZign (core, name, type, args);
+		ret = r_sign_add_hash (core->anal, name, t, sig, strlen (sig));
+		break;
 	default:
 		eprintf ("error: unknown zignature type\n");
 	}
-
-	return false;
+	r_list_free (lst);
+	free (args);
+	return ret;
 }
 
 static int cmdAdd(void *data, const char *input) {
 	RCore *core = (RCore *)data;
 
 	switch (*input) {
-	case ' ':
-		{
-			bool retval = true;
-			char *args = r_str_trim_dup (input + 1);
-			if (!args) {
-				return false;
-			}
-			RList *lst = r_str_split_list (args, " ", 0);
-			if (!lst) {
-				goto out_case_manual;
-			}
-			if (r_list_length (lst) < 3) {
-				eprintf ("Usage: za zigname type params\n");
-				retval = false;
-				goto out_case_manual;
-			}
-			char *zigname = r_list_pop_head (lst);
-			char *type_str = r_list_pop_head (lst);
-			if (strlen (type_str) != 1) {
-				eprintf ("Usage: za zigname type params\n");
-				retval = false;
-				goto out_case_manual;
-			}
-
-			if (!addZign (core, zigname, type_str[0], lst)) {
-				retval = false;
-				goto out_case_manual;
-			}
-
-out_case_manual:
-			r_list_free (lst);
-			free (args);
-			return retval;
-		}
-		break;
+	case ' ': // "za"
+		return za_add (core, input);
 	case 'f': // "zaf"
 		{
-			RAnalFunction *fcni = NULL;
-			RListIter *iter = NULL;
-			const char *fcnname = NULL, *zigname = NULL;
-			char *args = NULL;
-			int n = 0;
-			bool retval = true;
-
-			args = r_str_trim_dup (input + 1);
-			n = r_str_word_set0 (args);
+			char *args = r_str_trim_dup (input + 1);
+			int n = r_str_word_set0 (args);
 
 			if (n > 2) {
 				eprintf ("Usage: zaf [fcnname] [zigname]\n");
-				retval = false;
-				goto out_case_fcn;
+				free (args);
+				return false;
 			}
 
-			switch (n) {
-			case 2:
-				zigname = r_str_word_get0 (args, 1);
-			case 1:
-				fcnname = r_str_word_get0 (args, 0);
-			}
-
-			r_cons_break_push (NULL, NULL);
-			r_list_foreach (core->anal->fcns, iter, fcni) {
-				if (r_cons_is_breaked ()) {
-					break;
-				}
-				if ((!fcnname && core->offset == fcni->addr) ||
-					(fcnname && !strcmp (fcnname, fcni->name))) {
-					addFcnZign (core, fcni, zigname);
-					break;
-				}
-			}
-			r_cons_break_pop ();
-
-out_case_fcn:
-			free (args);
-			return retval;
-		}
-		break;
-	case 'F':
-		{
 			RAnalFunction *fcni = NULL;
-			RListIter *iter = NULL;
-			int count = 0;
+			const char *zigname = (n == 2)? r_str_word_get0 (args, 1): NULL;
+			if (n > 0) {
+				fcni = r_anal_get_function_byname (core->anal, r_str_word_get0 (args, 0));
+			} else {
+				fcni = r_anal_get_function_at (core->anal, core->offset);
+			}
+			if (fcni) {
+				r_sign_add_func (core->anal, fcni, zigname);
+			}
 
-			r_cons_break_push (NULL, NULL);
-			r_list_foreach (core->anal->fcns, iter, fcni) {
-				if (r_cons_is_breaked ()) {
-					break;
-				}
-				addFcnZign (core, fcni, NULL);
-				count++;
+			free (args);
+			if (!fcni) {
+				eprintf ("Could not find function");
+				return false;
 			}
-			r_cons_break_pop ();
-			eprintf ("generated zignatures: %d\n", count);
 		}
 		break;
+	case 'c': // "zac"
+		r_cons_break_push (NULL, NULL);
+		r_sign_resolve_collisions (core->anal);
+		r_cons_break_pop ();
+		break;
+	case 'F': // "zaF"
+	{
+		int count = r_sign_all_functions (core->anal, false);
+		eprintf ("generated zignatures: %d\n", count);
+		break;
+	}
+	case 'M': // "zaM"
+	{
+		int count = r_sign_all_functions (core->anal, true);
+		eprintf ("generated zignatures: %d\n", count);
+		break;
+	}
 	case '?':
 		if (input[1] == '?') {
 			// TODO #7967 help refactor: move to detail
 			r_cons_printf ("Adding Zignatures (examples and documentation)\n\n"
 				"Zignature types:\n"
-				"  a: bytes pattern (anal mask)\n"
+				"  a: bytes pattern, r2 creates mask from analysis\n"
 				"  b: bytes pattern\n"
 				"  c: base64 comment\n"
 				"  n: real function name\n"
@@ -479,7 +286,7 @@ static int cmdOpen(void *data, const char *input) {
 	switch (*input) {
 	case ' ':
 		if (input[1]) {
-			return r_sign_load (core->anal, input + 1);
+			return r_sign_load (core->anal, input + 1, false);
 		}
 		eprintf ("Usage: zo filename\n");
 		return false;
@@ -491,7 +298,7 @@ static int cmdOpen(void *data, const char *input) {
 		return false;
 	case 'z':
 		if (input[1] == ' ' && input[2]) {
-			return r_sign_load_gz (core->anal, input + 2);
+			return r_sign_load_gz (core->anal, input + 2, false);
 		}
 		eprintf ("Usage: zoz filename\n");
 		return false;
@@ -516,7 +323,11 @@ static int cmdSpace(void *data, const char *input) {
 			eprintf ("Usage: zs+zignspace\n");
 			return false;
 		}
-		r_spaces_push (zs, input + 1);
+		char *sp = r_str_trim_dup (input + 1);
+		if (sp) {
+			r_spaces_push (zs, sp);
+			free (sp);
+		}
 		break;
 	case 'r':
 		if (input[1] != ' ' || !input[2]) {
@@ -578,7 +389,7 @@ static int cmdFlirt(void *data, const char *input) {
 		int depth = r_config_get_i (core->config, "dir.depth");
 		char *file;
 		RListIter *iter;
-		RList *files = r_file_globsearch (input + 2, depth);
+		RList *files = r_file_glob (input + 2, depth);
 		r_list_foreach (files, iter, file) {
 			r_sign_flirt_scan (core->anal, file);
 		}
@@ -599,9 +410,18 @@ static int cmdFlirt(void *data, const char *input) {
 
 struct ctxSearchCB {
 	RCore *core;
+	bool bytes_only;
 	bool rad;
+	int collisions;
+	int newfuncs;
 	int count;
-	const char *prefix;
+	int bytes_count;
+	int graph_count;
+	int offset_count;
+	int refs_count;
+	int types_count;
+	int bbhash_count;
+	int next_count;
 };
 
 static void apply_name(RCore *core, RAnalFunction *fcn, RSignItem *it, bool rad) {
@@ -625,36 +445,6 @@ static void apply_name(RCore *core, RAnalFunction *fcn, RSignItem *it, bool rad)
 	}
 }
 
-static void apply_types(RCore *core, RAnalFunction *fcn, RSignItem *it) {
-	r_return_if_fail (core && fcn && it && it->name);
-	if (!it->types) {
-		return;
-	}
-	const char *name = it->realname? it->realname: it->name;
-	RListIter *iter;
-	char *type;
-	char *start = r_str_newf ("func.%s.", name);
-	size_t startlen = strlen (start);
-	char *alltypes = NULL;
-	r_list_foreach (it->types, iter, type) {
-		if (strncmp (start, type, startlen)) {
-			eprintf ("Unexpected type: %s\n", type);
-			free (alltypes);
-			free (start);
-			return;
-		}
-		if (!(alltypes = r_str_appendf (alltypes, "%s\n", type))) {
-			free (alltypes);
-			free (start);
-			return;
-		}
-	}
-	r_str_remove_char (alltypes, '"');
-	r_anal_save_parsed_type (core->anal, alltypes);
-	free (start);
-	free (alltypes);
-}
-
 static void apply_flag(RCore *core, RSignItem *it, ut64 addr, int size, int count, const char *prefix, bool rad) {
 	const char *zign_prefix = r_config_get (core->config, "zign.prefix");
 	char *name = r_str_newf ("%s.%s.%s_%d", zign_prefix, prefix, it->name, count);
@@ -672,49 +462,103 @@ static void apply_flag(RCore *core, RSignItem *it, ut64 addr, int size, int coun
 	}
 }
 
-static const char *getprefix(RSignType t) {
-	switch (t) {
-	case R_SIGN_BYTES:
-		return "bytes";
-	case R_SIGN_GRAPH:
-		return "graph";
-	case R_SIGN_OFFSET:
-		return "offset";
-	case R_SIGN_REFS:
-		return "refs";
-	case R_SIGN_TYPES:
-		return "types";
-	case R_SIGN_BBHASH:
-		return "bbhash";
-	default:
-		break;
-	}
-	return "unknown";
-}
-
-static int searchHitCB(RSignItem *it, RSearchKeyword *kw, ut64 addr, void *user) {
+static int searchBytesHitCB(RSignItem *it, RSearchKeyword *kw, ut64 addr, void *user) {
 	struct ctxSearchCB *ctx = (struct ctxSearchCB *)user;
-	apply_flag (ctx->core, it, addr, kw->keyword_length, kw->count, ctx->prefix, ctx->rad);
 	RAnalFunction *fcn = r_anal_get_fcn_in (ctx->core->anal, addr, 0);
-	// TODO: create fcn if it does not exist
-	if (fcn) {
-		apply_name (ctx->core, fcn, it, ctx->rad);
-		apply_types (ctx->core, fcn, it);
+	if (!fcn) {
+		RCore *c = ctx->core;
+		r_core_af (c, c->offset, NULL, false);
+		fcn = r_anal_get_fcn_in (ctx->core->anal, addr, 0);
+		ctx->newfuncs++;
+		ctx->count++;
+	}
+	apply_flag (ctx->core, it, addr, kw->keyword_length, kw->count, "bytes", ctx->rad);
+	if (ctx->bytes_only) {
+		if (fcn) {
+			char *tmp = NULL;
+			apply_name (ctx->core, fcn, it, ctx->rad);
+			if (it->types && (tmp = r_str_newf ("%s;", it->types))) { // apply types
+				r_anal_str_to_fcn (ctx->core->anal, fcn, tmp);
+				free (tmp);
+			}
+			if (it->vars) {
+				r_anal_function_set_var_prot (fcn, it->vars);
+			}
+		}
+		ctx->bytes_count++;
 	}
-	ctx->count++;
 	return 1;
 }
 
-static int fcnMatchCB(RSignItem *it, RAnalFunction *fcn, RSignType type, bool seen, void *user) {
+static int fcnMatchCB(RSignItem *it, RAnalFunction *fcn, RSignType *types, void *user, RList *col) {
+	r_return_val_if_fail (types && *types != R_SIGN_END, 1);
 	struct ctxSearchCB *ctx = (struct ctxSearchCB *)user;
-	const char *prefix = getprefix (type);
-	// TODO(nibble): use one counter per metric zign instead of ctx->count
 	ut64 sz = r_anal_function_realsize (fcn);
-	apply_flag (ctx->core, it, fcn->addr, sz, ctx->count, prefix, ctx->rad);
-	if (!seen) {
+	RSignType t;
+	bool collides = false;
+	if (!col || !r_list_empty (col)) {
+		// NULL col implies collision computation err, so assume collides
+		collides = true;
+		ctx->collisions++;
+	}
+	int i = 0;
+	while ((t = types[i++]) != R_SIGN_END) {
+		const char *prefix = NULL;
+		switch (t) {
+		case R_SIGN_BYTES:
+			ctx->bytes_count++;
+			prefix = "bytes";
+			break;
+		case R_SIGN_GRAPH:
+			prefix = "graph";
+			ctx->graph_count++;
+			break;
+		case R_SIGN_OFFSET:
+			prefix = "offset";
+			ctx->offset_count++;
+			break;
+		case R_SIGN_REFS:
+			prefix = "refs";
+			ctx->refs_count++;
+			break;
+		case R_SIGN_TYPES:
+			prefix = "types";
+			ctx->types_count++;
+			break;
+		case R_SIGN_BBHASH:
+			prefix = "bbhash";
+			ctx->bbhash_count++;
+			break;
+		case R_SIGN_NEXT:
+			prefix = "next";
+			ctx->next_count++;
+			break;
+		default:
+			r_warn_if_reached ();
+			break;
+		}
+		if (prefix) {
+			char *tmp = NULL;
+			if (collides && (tmp = r_str_newf ("%s_collision", prefix))) {
+				apply_flag (ctx->core, it, fcn->addr, sz, ctx->count, tmp, ctx->rad);
+				free (tmp);
+			} else {
+				apply_flag (ctx->core, it, fcn->addr, sz, ctx->count, prefix, ctx->rad);
+			}
+			ctx->count++;
+		}
+	}
+
+	if (!collides) {
+		char *tmp = NULL;
 		apply_name (ctx->core, fcn, it, ctx->rad);
-		apply_types (ctx->core, fcn, it);
-		ctx->count++;
+		if (it->types && (tmp = r_str_newf ("%s;", it->types))) { // apply types
+			r_anal_str_to_fcn (ctx->core->anal, fcn, tmp);
+			free (tmp);
+		}
+		if (it->vars) {
+			r_anal_function_set_var_prot (fcn, it->vars);
+		}
 	}
 	return 1;
 }
@@ -731,7 +575,7 @@ static bool searchRange(RCore *core, ut64 from, ut64 to, bool rad, struct ctxSea
 	}
 	RSignSearch *ss = r_sign_search_new ();
 	ss->search->align = r_config_get_i (core->config, "search.align");
-	r_sign_search_init (core->anal, ss, minsz, searchHitCB, ctx);
+	r_sign_search_init (core->anal, ss, minsz, searchBytesHitCB, ctx);
 
 	r_cons_break_push (NULL, NULL);
 	for (at = from; at < to; at += core->blocksize) {
@@ -758,51 +602,20 @@ static bool searchRange(RCore *core, ut64 from, ut64 to, bool rad, struct ctxSea
 	return retval;
 }
 
-static bool searchRange2(RCore *core, RSignSearch *ss, ut64 from, ut64 to, bool rad, struct ctxSearchCB *ctx) {
-	ut8 *buf = malloc (core->blocksize);
-	ut64 at;
-	int rlen;
-	bool retval = true;
-
-	if (!buf) {
-		return false;
-	}
-	r_cons_break_push (NULL, NULL);
-	for (at = from; at < to; at += core->blocksize) {
-		if (r_cons_is_breaked ()) {
-			retval = false;
-			break;
-		}
-		rlen = R_MIN (core->blocksize, to - at);
-		if (!r_io_is_valid_offset (core->io, at, 0)) {
-			retval = false;
-			break;
-		}
-		(void)r_io_read_at (core->io, at, buf, rlen);
-		if (r_sign_search_update (core->anal, ss, &at, buf, rlen) == -1) {
-			eprintf ("search: update read error at 0x%08"PFMT64x"\n", at);
-			retval = false;
-			break;
-		}
-	}
-	r_cons_break_pop ();
-	free (buf);
-
-	return retval;
-}
-
 static void search_add_to_types(RCore *c, RSignSearchMetrics *sm, RSignType t, const char *str, unsigned int *i) {
 	unsigned int count = *i;
-	r_return_if_fail (count < sizeof (sm->types) / sizeof (RSignType) - 1);
+	r_return_if_fail (count < sizeof (sm->stypes) / sizeof (RSignType) - 1);
 	if (r_config_get_i (c->config, str)) {
-		sm->types[count++] = t;
-		sm->types[count] = 0;
+		sm->stypes[count++] = t;
+		sm->stypes[count] = 0;
 		*i = count;
 	}
 }
 
 static bool fill_search_metrics(RSignSearchMetrics *sm, RCore *c, void *user) {
 	unsigned int i = 0;
+	sm->stypes[0] = R_SIGN_END;
+	search_add_to_types (c, sm, R_SIGN_BYTES, "zign.bytes", &i);
 	search_add_to_types (c, sm, R_SIGN_GRAPH, "zign.graph", &i);
 	search_add_to_types (c, sm, R_SIGN_OFFSET, "zign.offset", &i);
 	search_add_to_types (c, sm, R_SIGN_REFS, "zign.refs", &i);
@@ -813,30 +626,56 @@ static bool fill_search_metrics(RSignSearchMetrics *sm, RCore *c, void *user) {
 	search_add_to_types(c, sm, R_SIGN_VARS, "zign.vars", &i);
 #endif
 	sm->mincc = r_config_get_i (c->config, "zign.mincc");
+	sm->minsz = r_config_get_i (c->config, "zign.minsz");
 	sm->anal = c->anal;
 	sm->cb = fcnMatchCB;
 	sm->user = user;
-	sm->fcn = NULL;
 	return (i > 0);
 }
 
+static void print_ctx_hits(struct ctxSearchCB *ctx) {
+	int prints = 0;
+	if (ctx->newfuncs) {
+		eprintf ("New functions:  %d\n", ctx->newfuncs);
+	}
+	if (ctx->collisions) {
+		eprintf ("collisions:  %d\n", ctx->collisions);
+	}
+	if (ctx->bytes_count) {
+		eprintf ("bytes:  %d\n", ctx->bytes_count);
+		prints++;
+	}
+	if (ctx->graph_count) {
+		eprintf ("graph:  %d\n", ctx->graph_count);
+		prints++;
+	}
+	if (ctx->offset_count) {
+		eprintf ("offset: %d\n", ctx->offset_count);
+		prints++;
+	}
+	if (ctx->refs_count) {
+		eprintf ("refs:   %d\n", ctx->refs_count);
+		prints++;
+	}
+	if (ctx->types_count) {
+		eprintf ("types:  %d\n", ctx->types_count);
+		prints++;
+	}
+	if (ctx->bbhash_count) {
+		eprintf ("bbhash: %d\n", ctx->bbhash_count);
+		prints++;
+	}
+	if (ctx->next_count) {
+		eprintf ("next: %d\n", ctx->next_count);
+		prints++;
+	}
+	if (prints > 1) {
+		eprintf ("total:  %d\n", ctx->count);
+	}
+}
+
 static bool search(RCore *core, bool rad, bool only_func) {
-	RList *list;
-	RListIter *iter;
-	RAnalFunction *fcni = NULL;
-	RIOMap *map;
-	bool retval = true;
-
-	struct ctxSearchCB bytes_search_ctx = { core, rad, 0, "bytes" };
-	const char *mode = r_config_get (core->config, "search.in");
-	bool useBytes = r_config_get_i (core->config, "zign.bytes");
 	const char *zign_prefix = r_config_get (core->config, "zign.prefix");
-	int maxsz = r_config_get_i (core->config, "zign.maxsz");
-
-	struct ctxSearchCB metsearch_ctx = { core, rad, 0, NULL };
-	RSignSearchMetrics sm;
-	bool metsearch = fill_search_metrics (&sm, core, (void *)&metsearch_ctx);
-
 	if (rad) {
 		r_cons_printf ("fs+%s\n", zign_prefix);
 	} else {
@@ -846,53 +685,41 @@ static bool search(RCore *core, bool rad, bool only_func) {
 		}
 	}
 
+	struct ctxSearchCB ctx;
+	memset (&ctx, 0, sizeof (struct ctxSearchCB));
+	ctx.rad = rad;
+	ctx.core = core;
+
+	RSignSearchMetrics sm;
+	bool metsearch = fill_search_metrics (&sm, core, (void *)&ctx);
+	if (!metsearch) {
+		eprintf ("No zign types enabled\n");
+		return false;
+	}
+	if (sm.stypes[0] == R_SIGN_BYTES && sm.stypes[1] == R_SIGN_END) {
+		ctx.bytes_only = true;
+	}
+
 	// Bytes search
-	if (useBytes && !only_func) {
-		list = r_core_get_boundaries_prot (core, -1, mode, "search");
+	if (r_config_get_i (core->config, "zign.bytes") && !only_func) {
+		const char *mode = r_config_get (core->config, "search.in");
+		RList *list = r_core_get_boundaries_prot (core, -1, mode, "search");
 		if (!list) {
 			return false;
 		}
+		RListIter *iter;
+		RIOMap *map;
 		r_list_foreach (list, iter, map) {
 			eprintf ("[+] searching 0x%08"PFMT64x" - 0x%08"PFMT64x"\n", r_io_map_begin (map), r_io_map_end (map));
-			retval &= searchRange (core, r_io_map_begin (map), r_io_map_end (map), rad, &bytes_search_ctx);
+			searchRange (core, r_io_map_begin (map), r_io_map_end (map), rad, &ctx);
 		}
 		r_list_free (list);
 	}
 
 	// Function search
-	int hits = 0;
-	if (metsearch) {
+	if (!ctx.bytes_only) {
 		eprintf ("[+] searching function metrics\n");
-		r_cons_break_push (NULL, NULL);
-		int count = 0;
-
-		RSignSearch *ss = NULL;
-
-		if (useBytes && only_func) {
-			ss = r_sign_search_new ();
-			ss->search->align = r_config_get_i (core->config, "search.align");
-			int minsz = r_config_get_i (core->config, "zign.minsz");
-			r_sign_search_init (core->anal, ss, minsz, searchHitCB, &bytes_search_ctx);
-		}
-
-		r_list_foreach (core->anal->fcns, iter, fcni) {
-			if (r_cons_is_breaked ()) {
-				break;
-			}
-			if (useBytes && only_func) {
-				eprintf ("Matching func %d / %d (hits %d)\n", count, r_list_length (core->anal->fcns), bytes_search_ctx.count);
-				int fcnlen = r_anal_function_realsize (fcni);
-				int len = R_MIN (core->io->addrbytes * fcnlen, maxsz);
-				retval &= searchRange2 (core, ss, fcni->addr, fcni->addr + len, rad, &bytes_search_ctx);
-			}
-			sm.fcn = fcni;
-			hits += r_sign_fcn_match_metrics (&sm);
-			sm.fcn = NULL;
-			count ++;
-			// TODO: add useXRefs, useName
-		}
-		r_cons_break_pop ();
-		r_sign_search_free (ss);
+		r_sign_metric_search (core->anal, &sm);
 	}
 
 	if (rad) {
@@ -903,28 +730,41 @@ static bool search(RCore *core, bool rad, bool only_func) {
 			return false;
 		}
 	}
-
-	hits += bytes_search_ctx.count;
-	eprintf ("hits: %d\n", hits);
-
-	return retval;
+	print_ctx_hits (&ctx);
+	return ctx.count > 0? true: false;
 }
 
-static void print_possible_matches(RList *list) {
+static void print_possible_matches(RList *list, bool json, RCore *core) {
 	RListIter *itr;
 	RSignCloseMatch *row;
-	r_list_foreach (list, itr, row) {
-		// total score
-		if (row->bscore > 0.0 && row->gscore > 0.0) {
-			r_cons_printf ("%02.5lf  ", row->score);
+	if (json) {
+		PJ *pj = core->anal->coreb.pjWithEncoding (core);
+		pj_a (pj);
+		r_list_foreach (list, itr, row) {
+			pj_o (pj);
+			pj_ks (pj, "name", row->item->name);
+			pj_kd (pj, "similarity", row->score);
+			pj_kd (pj, "byte similarity", row->bscore);
+			pj_kd (pj, "graph similarity", row->gscore);
+			pj_end (pj);
 		}
-		if (row->bscore > 0.0) {
-			r_cons_printf ("%02.5lf B  ", row->bscore);
+		pj_end (pj);
+		r_cons_printf ("%s\n", pj_string (pj));
+		pj_free (pj);
+	} else {
+		r_list_foreach (list, itr, row) {
+			// total score
+			if (row->bscore > 0.0 && row->gscore > 0.0) {
+				r_cons_printf ("%02.5lf  ", row->score);
+			}
+			if (row->bscore > 0.0) {
+				r_cons_printf ("%02.5lf B  ", row->bscore);
+			}
+			if (row->gscore > 0.0) {
+				r_cons_printf ("%02.5lf G  ", row->gscore);
+			}
+			r_cons_printf (" %s\n", row->item->name);
 		}
-		if (row->gscore > 0.0) {
-			r_cons_printf ("%02.5lf G  ", row->gscore);
-		}
-		r_cons_printf (" %s\n", row->item->name);
 	}
 }
 
@@ -964,7 +804,7 @@ static double get_zb_threshold(RCore *core) {
 	return thresh;
 }
 
-static bool bestmatch_fcn(RCore *core, const char *input) {
+static bool bestmatch_fcn(RCore *core, const char *input, bool json) {
 	r_return_val_if_fail (input && core, false);
 
 	char *argv = r_str_new (input);
@@ -1014,14 +854,14 @@ static bool bestmatch_fcn(RCore *core, const char *input) {
 	r_sign_item_free (it);
 
 	if (list) {
-		print_possible_matches (list);
+		print_possible_matches (list, json, core);
 		r_list_free (list);
 		return true;
 	}
 	return false;
 }
 
-static bool bestmatch_sig(RCore *core, const char *input) {
+static bool bestmatch_sig(RCore *core, const char *input, bool json) {
 	r_return_val_if_fail (input && core, false);
 	int count = 5;
 	if (!R_STR_ISEMPTY (input)) {
@@ -1048,9 +888,7 @@ static bool bestmatch_sig(RCore *core, const char *input) {
 		RSignBytes *b = item->bytes;
 		int minsz = r_config_get_i (core->config, "zign.minsz");
 		if (b && b->size < minsz) {
-			eprintf ("Warning: Function signature is too small (%d < %d) See e zign.minsz", b->size, minsz);
-			r_sign_item_free (item);
-			return false;
+			eprintf ("Warning: Function signature is too small (%d < %d) See e zign.minsz \n", b->size, minsz);
 		}
 	}
 	if (r_config_get_i (core->config, "zign.graph")) {
@@ -1064,7 +902,7 @@ static bool bestmatch_sig(RCore *core, const char *input) {
 		RList *list = r_sign_find_closest_sig (core->anal, item, count, th);
 		if (list) {
 			found = true;
-			print_possible_matches (list);
+			print_possible_matches (list, json, core);
 			r_list_free (list);
 		}
 		r_cons_break_pop ();
@@ -1078,17 +916,24 @@ static bool bestmatch_sig(RCore *core, const char *input) {
 
 static bool bestmatch(void *data, const char *input) {
 	r_return_val_if_fail (data && input, false);
+	bool json = false;
 	RCore *core = (RCore *)data;
 	switch (input[0]) {
 	case 'r':
 		input++;
-		return bestmatch_fcn (core, input);
-		break;
+		if (*input == 'j') {
+			input++;
+			json = true;
+		}
+		return bestmatch_fcn (core, input, json);
+	case 'j':
+		json = true;
+		/* fallthrough */
 	case ' ':
 		input++;
+		/* fallthrough */
 	case '\x00':
-		return bestmatch_sig (core, input);
-		break;
+		return bestmatch_sig (core, input, json);
 	case '?':
 	default:
 		r_core_cmd_help (core, help_msg_zb);
@@ -1096,9 +941,192 @@ static bool bestmatch(void *data, const char *input) {
 	}
 }
 
+static bool _sig_bytediff_cb(RLevBuf *va, RLevBuf *vb, ut32 ia, ut32 ib) {
+	RSignBytes *a = (RSignBytes *)va->buf;
+	RSignBytes *b = (RSignBytes *)vb->buf;
+
+	if ((a->bytes[ia] & a->mask[ia]) == (b->bytes[ib] & b->mask[ib])) {
+		return false;
+	}
+	return true;
+}
+
+#define lines_addbytesmask(l, sig, index, add, col) \
+	l.bytes = r_str_appendf (l.bytes, " %s%s%02x%s", r_str_get (col), r_str_get (add), sig->bytes[index], col? Color_RESET: ""); \
+	l.mask = r_str_appendf (l.mask, " %s%s%02x%s", r_str_get (col), r_str_get (add), sig->mask[index], col? Color_RESET: ""); \
+	l.land = r_str_appendf (l.land, " %s%s%02x%s", r_str_get (col), r_str_get (add), sig->bytes[index] & sig->mask[index], col?Color_RESET: ""); \
+	index++;
+
+#define lines_addblnk(l) \
+	l.bytes = r_str_append (l.bytes, "    "); \
+	l.mask = r_str_append (l.mask, "    "); \
+	l.land = r_str_append (l.land, "    ");
+
+#define freelines(x) \
+	free (x.bytes); \
+	free (x.mask); \
+	free (x.land); \
+	memset (&x, 0, sizeof (x));
+
+static void print_zig_diff(RCore *c, RSignBytes *ab, RSignBytes *bb, RLevOp *ops) {
+	struct lines {
+		char *mask, *bytes, *land;
+	} al, bl;
+	memset (&al, 0, sizeof (al));
+	memset (&bl, 0, sizeof (bl));
+
+	char *colsub, *coladd, *coldel;
+	colsub = coladd = coldel = NULL;
+	if (r_config_get_b (c->config, "scr.color")) {
+		coldel = "\x1b[1;31m";
+		coladd = "\x1b[1;32m";
+		colsub = "\x1b[1;33m";
+	}
+
+	int i, ia, ib, iastart, ibstart;
+	ia = ib = iastart = ibstart = 0;
+	bool printb = false;
+	for (i = 0; ops[i] != LEVEND; i++) {
+		switch (ops[i]) {
+		case LEVNOP:
+			// lines_addbytesmask macro does ia++ so test must before
+			if (!printb && (ab->bytes[ia] != bb->bytes[ib] || ab->mask[ia] != bb->mask[ib])) {
+				printb = true;
+			}
+			lines_addbytesmask (al, ab, ia, " ", (const char *)NULL);
+			lines_addbytesmask (bl, bb, ib, " ", (const char *)NULL);
+			break;
+		case LEVSUB:
+			lines_addbytesmask (al, ab, ia, " ", colsub);
+			lines_addbytesmask (bl, bb, ib, "^", colsub);
+			printb = true;
+			break;
+		case LEVADD:
+			lines_addblnk (al);
+			lines_addbytesmask (bl, bb, ib, "+", coladd);
+			printb = true;
+			break;
+		case LEVDEL:
+			lines_addbytesmask (al, ab, ia, "-", coldel);
+			lines_addblnk (bl);
+			printb = true;
+			break;
+		default:
+			r_warn_if_reached ();
+			freelines (al);
+			freelines (bl);
+			return;
+		}
+		// when alloc fails
+		if (!(al.bytes && al.mask && al.land && bl.bytes && bl.mask && bl.land)) {
+			freelines (al);
+			freelines (bl);
+			return;
+		}
+
+		if (i % 16 == 15 || ops[i + 1] == LEVEND) {
+			if (i > 16) {
+				r_cons_printf ("\n");
+			}
+			r_cons_printf ("Fnc cmp   0x%04x %s\n", iastart, al.land);
+			if (printb) {
+				r_cons_printf ("Sig cmp   0x%04x %s\n", ibstart, bl.land);
+			}
+			r_cons_printf ("Fnc Mask  0x%04x %s\n", iastart, al.mask);
+			if (printb) {
+				r_cons_printf ("Sig Mask  0x%04x %s\n", ibstart, bl.mask);
+			}
+			r_cons_printf ("Fnc Bytes 0x%04x %s\n", iastart, al.bytes);
+			if (printb) {
+				r_cons_printf ("Sig Bytes 0x%04x %s\n", ibstart, bl.bytes);
+			} else {
+				r_cons_printf ("== Signature was same ==\n");
+			}
+			freelines (al);
+			freelines (bl);
+			iastart = ia;
+			ibstart = ib;
+			printb = false;
+		}
+	}
+}
+#undef lines_addbytesmask
+#undef lines_addblnk
+#undef freelines
+
+static bool diff_zig(void *data, const char *input) {
+	r_return_val_if_fail (data && input, false);
+	RCore *core = (RCore *)data;
+
+	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+	if (!fcn) {
+		eprintf ("No function at 0x%08" PFMT64x "\n", core->offset);
+		return false;
+	}
+
+	char *argv = r_str_new (input);
+	if (!argv) {
+		return false;
+	}
+
+	char *zigname = strtok (argv, " ");
+	if (!zigname) {
+		eprintf ("Need a signature\n");
+		free (argv);
+		return false;
+	}
+
+	if (strtok (NULL, " ")) {
+		eprintf ("too many arguments");
+		free (argv);
+		return false;
+	}
+
+	RSignItem *it = item_frm_signame (core->anal, zigname);
+	if (!it) {
+		eprintf ("Couldn't get signature for %s\n", zigname);
+		free (argv);
+		return false;
+	}
+	free (argv);
+
+	if (!it->bytes) {
+		eprintf ("Signature %s missing bytes\n", it->name);
+		return false;
+	}
+
+	RLevBuf b;
+	b.buf = it->bytes;
+	b.len = it->bytes->size;
+
+	RSignItem *fit = r_sign_item_new ();
+	if (!fit) {
+		r_sign_item_free (it);
+		return false;
+	}
+	r_sign_addto_item (core->anal, fit, fcn, R_SIGN_BYTES);
+
+	RLevBuf a;
+	a.buf = fit->bytes;
+	a.len = fit->bytes->size;
+
+	RLevOp *ops = NULL;
+
+	if (r_diff_levenshtein_path (&a, &b, UT32_MAX, _sig_bytediff_cb, &ops) < 0) {
+		eprintf ("Diff failed\n");
+	} else {
+		print_zig_diff (core, fit->bytes, it->bytes, ops);
+	}
+
+	free (ops);
+	r_sign_item_free (fit);
+	r_sign_item_free (it);
+	return false;
+}
+
 static int cmdCompare(void *data, const char *input) {
 	int result = true;
-	RCore *core = (RCore *) data;
+	RCore *core = (RCore *)data;
 	const char *raw_bytes_thresh = r_config_get (core->config, "zign.diff.bthresh");
 	const char *raw_graph_thresh = r_config_get (core->config, "zign.diff.gthresh");
 	RSignOptions *options = r_sign_options_new (raw_bytes_thresh, raw_graph_thresh);
@@ -1150,24 +1178,19 @@ static int cmdCompare(void *data, const char *input) {
 
 static int cmdCheck(void *data, const char *input) {
 	RCore *core = (RCore *) data;
-	RSignSearch *ss;
-	RListIter *iter;
-	RAnalFunction *fcni = NULL;
-	ut64 at = core->offset;
-	bool retval = true;
-	bool rad = input[0] == '*';
+	struct ctxSearchCB ctx;
+	memset (&ctx, 0, sizeof (struct ctxSearchCB));
+	ctx.rad = input[0] == '*';
+	ctx.core = core;
 
-	struct ctxSearchCB bytes_search_ctx = { core, rad, 0, "bytes" };
+	RSignSearchMetrics sm;
+	if (!fill_search_metrics (&sm, core, (void *)&ctx)) {
+		eprintf ("Nothing to search for\n");
+		return 0;
+	}
 
 	const char *zign_prefix = r_config_get (core->config, "zign.prefix");
-	int minsz = r_config_get_i (core->config, "zign.minsz");
-	bool useBytes = r_config_get_i (core->config, "zign.bytes");
-
-	struct ctxSearchCB metsearch_ctx = { core, rad, 0, NULL };
-	RSignSearchMetrics sm;
-	bool metsearch = fill_search_metrics (&sm, core, (void *)&metsearch_ctx);
-
-	if (rad) {
+	if (ctx.rad) {
 		r_cons_printf ("fs+%s\n", zign_prefix);
 	} else {
 		if (!r_flag_space_push (core->flags, zign_prefix)) {
@@ -1176,37 +1199,17 @@ static int cmdCheck(void *data, const char *input) {
 		}
 	}
 
-	// Bytes search
-	if (useBytes) {
-		eprintf ("[+] searching 0x%08"PFMT64x" - 0x%08"PFMT64x"\n", at, at + core->blocksize);
-		ss = r_sign_search_new ();
-		r_sign_search_init (core->anal, ss, minsz, searchHitCB, &bytes_search_ctx);
-		if (r_sign_search_update (core->anal, ss, &at, core->block, core->blocksize) == -1) {
-			eprintf ("search: update read error at 0x%08"PFMT64x"\n", at);
-			retval = false;
-		}
-		r_sign_search_free (ss);
-	}
-
-	// Function search
-	int hits = 0;
-	if (metsearch) {
-		eprintf ("[+] searching function metrics\n");
+	eprintf ("[+] searching function metrics\n");
+	RAnalFunction *fcn = r_anal_get_function_at (core->anal, core->offset);
+	if (fcn) {
 		r_cons_break_push (NULL, NULL);
-		r_list_foreach (core->anal->fcns, iter, fcni) {
-			if (r_cons_is_breaked ()) {
-				break;
-			}
-			if (fcni->addr == core->offset) {
-				sm.fcn = fcni;
-				hits += r_sign_fcn_match_metrics (&sm);
-				break;
-			}
-		}
+		r_sign_fcn_match_metrics (&sm, fcn);
 		r_cons_break_pop ();
+	} else {
+		eprintf ("No function at 0x%08" PFMT64x "\n", core->offset);
 	}
 
-	if (rad) {
+	if (ctx.rad) {
 		r_cons_printf ("fs-\n");
 	} else {
 		if (!r_flag_space_pop (core->flags)) {
@@ -1215,10 +1218,8 @@ static int cmdCheck(void *data, const char *input) {
 		}
 	}
 
-	hits += bytes_search_ctx.count;
-	eprintf ("hits: %d\n", hits);
-
-	return retval;
+	print_ctx_hits (&ctx);
+	return ctx.count;
 }
 
 static int cmdSearch(void *data, const char *input) {
@@ -1226,9 +1227,9 @@ static int cmdSearch(void *data, const char *input) {
 
 	switch (*input) {
 	case 0:
-	case '*':
+	case '*': // "z/*"
 		return search (core, input[0] == '*', false);
-	case 'f':
+	case 'f': // "z/f"
 		switch (input[1]) {
 		case 0:
 		case '*':
@@ -1241,7 +1242,7 @@ static int cmdSearch(void *data, const char *input) {
 		r_core_cmd_help (core, help_msg_z_slash);
 		break;
 	default:
-		eprintf ("Usage: z/[*]\n");
+		eprintf ("Usage: z/[f*]\n");
 		return false;
 	}
 	return true;
@@ -1264,8 +1265,8 @@ static int cmd_zign(void *data, const char *input) {
 
 	switch (*input) {
 	case '\0':
-	case '*':
-	case 'q':
+	case '*': // "z*"
+	case 'q': // "zq"
 	case 'j': // "zj"
 		r_sign_list (core->anal, *input);
 		break;
@@ -1279,6 +1280,8 @@ static int cmd_zign(void *data, const char *input) {
 		return cmdCheck (data, arg);
 	case 'b': // "zb"
 		return bestmatch (data, arg);
+	case 'd': // "zb"
+		return diff_zig (data, arg);
 	case 'o': // "zo"
 		return cmdOpen (data, arg);
 	case 'g': // "zg"
