@@ -1,9 +1,10 @@
-/* radare - LGPL - Copyright 2010-2020 - pancake, maijin */
+/* radare - LGPL - Copyright 2010-2021 - pancake, rhl */
 
 #include <r_types.h>
 #include <r_list.h>
 #include <r_flag.h>
 #include <r_core.h>
+#include <rvc.h>
 #define USE_R2 1
 #include <spp/spp.h>
 
@@ -15,8 +16,7 @@ static bool is_valid_project_name(const char *name) {
 	if (r_str_len_utf8 (name) >= 16) {
 		return false;
 	}
-	const char  *extention = r_str_endswith (name, ".zip") ?
-		r_str_last (name, ".zip") : NULL;
+	const char *extention = r_str_endswith (name, ".zip") ? r_str_last (name, ".zip") : NULL;
 	for (; *name && name != extention; name++) {
 		if (IS_DIGIT (*name) || IS_LOWER (*name) || *name == '_') {
 			continue;
@@ -27,7 +27,10 @@ static bool is_valid_project_name(const char *name) {
 }
 
 static char *get_project_script_path(RCore *core, const char *file) {
-	r_return_val_if_fail (core && !R_STR_ISEMPTY (file), NULL);
+	r_return_val_if_fail (core && file, NULL);
+	if (!*file) {
+		return NULL;
+	}
 	const char *magic = "# r2 rdb project file";
 	char *data, *prjfile;
 	if (r_file_is_abspath (file)) {
@@ -136,46 +139,12 @@ R_API int r_core_project_list(RCore *core, int mode) {
 	return 0;
 }
 
-static inline void remove_project_file(char * path) {
-	if (r_file_exists (path)) {
-		r_file_rm (path);
-		eprintf ("rm %s\n", path);
-	}
+R_API void r_core_project_undirty(RCore *core) {
+	core->config->is_dirty = false;
+	core->anal->is_dirty = false;
+	core->flags->is_dirty = false;
 }
 
-static inline void remove_notes_file(const char *pd) {
-	char *notes_txt = r_file_new (pd, "notes.txt", NULL);
-	if (r_file_exists (notes_txt)) {
-		r_file_rm (notes_txt);
-		eprintf ("rm %s\n", notes_txt);
-	}
-	free (notes_txt);
-}
-
-static inline void remove_rop_directory(const char *prj_dir) {
-	char *rop_d = r_file_new (prj_dir, "rop.d", NULL);
-
-	if (r_file_is_directory (rop_d)) {
-		char *f;
-		RListIter *iter;
-		RList *files = r_sys_dir (rop_d);
-		r_list_foreach (files, iter, f) {
-			char *filepath = r_str_append (strdup (rop_d), R_SYS_DIR);
-			filepath = r_str_append (filepath, f);
-			if (!r_file_is_directory (filepath)) {
-				eprintf ("rm %s\n", filepath);
-				r_file_rm (filepath);
-			}
-
-			free (filepath);
-		}
-		r_file_rm (rop_d);
-		eprintf ("rm %s\n", rop_d);
-		r_list_free (files);
-	}
-
-	free (rop_d);
-}
 R_API int r_core_project_delete(RCore *core, const char *prjfile) {
 	if (r_sandbox_enable (0)) {
 		eprintf ("Cannot delete project in sandbox mode\n");
@@ -193,11 +162,7 @@ R_API int r_core_project_delete(RCore *core, const char *prjfile) {
 			free (path);
 			return false;
 		}
-		remove_project_file (path);
-		remove_notes_file (prj_dir);
-		remove_rop_directory (prj_dir);
-		// remove directory only if it's empty
-		r_file_rm (prj_dir);
+		r_file_rm_rf (prj_dir);
 		free (prj_dir);
 	}
 	free (path);
@@ -222,8 +187,6 @@ static bool load_project_rop(RCore *core, const char *prjfile) {
 	char *prj_dir = r_file_dirname (rc_path);
 	R_FREE (rc_path);
 	if (r_str_endswith (prjfile, R_SYS_DIR "rc.r2")) {
-		// XXX
-		eprintf ("ENDS WITH\n");
 		path = strdup (prjfile);
 		path[strlen (path) - 3] = 0;
 	} else if (r_file_fexists ("%s%s%src.r2", R_SYS_DIR, prj_dir, prjfile)) {
@@ -251,7 +214,7 @@ static bool load_project_rop(RCore *core, const char *prjfile) {
 		return false;
 	}
 	if (rop_db) {
-		ls_foreach (core->sdb->ns, it, ns){
+		ls_foreach (core->sdb->ns, it, ns) {
 			if (ns->sdb == rop_db) {
 				ls_delete (core->sdb->ns, it);
 				found = true;
@@ -338,14 +301,27 @@ static bool r_core_project_load(RCore *core, const char *prj_name, const char *r
 	if (!r_project_open (core->prj, prj_name, rcpath)) {
 		return false;
 	}
-	const bool cfg_fortunes = r_config_get_i (core->config, "cfg.fortunes");
+	const bool cfg_fortunes = r_config_get_b (core->config, "cfg.fortunes");
 	const bool scr_interactive = r_cons_is_interactive ();
-	const bool scr_prompt = r_config_get_i (core->config, "scr.prompt");
+	const bool scr_prompt = r_config_get_b (core->config, "scr.prompt");
 	(void) load_project_rop (core, prj_name);
-	bool ret = r_core_cmd_file (core, rcpath);
-	r_config_set_i (core->config, "cfg.fortunes", cfg_fortunes);
-	r_config_set_i (core->config, "scr.interactive", scr_interactive);
-	r_config_set_i (core->config, "scr.prompt", scr_prompt);
+	const bool sandy = r_config_get_b (core->config, "prj.sandbox");
+	bool ret = false;
+	if (sandy) {
+		// enable sandbox (only allow file access, no network or program exec)
+		// projects can also tweak the cmd. eval vars to run code after the project is loaded
+		// users must be careful on that.
+		int oldgrain = r_sandbox_grain (R_SANDBOX_GRAIN_DISK | R_SANDBOX_GRAIN_FILES);
+		r_sandbox_enable (true);
+		ret = r_core_cmd_file (core, rcpath);
+		r_sandbox_disable (true);
+		r_sandbox_grain (oldgrain);
+	} else {
+		ret = r_core_cmd_file (core, rcpath);
+	}
+	r_config_set_b (core->config, "cfg.fortunes", cfg_fortunes);
+	r_config_set_b (core->config, "scr.interactive", scr_interactive);
+	r_config_set_b (core->config, "scr.prompt", scr_prompt);
 	r_config_bump (core->config, "asm.arch");
 	r_config_set (core->config, "prj.name", prj_name);
 	return ret;
@@ -407,11 +383,13 @@ R_API bool r_core_project_open(RCore *core, const char *prj_path) {
 	ret = r_core_project_load (core, prj_name, prj_script);
 	free (prj_name);
 	free (prj_script);
+	if (ret)  {
+		r_core_project_undirty(core);
+	}
 	return ret;
 }
 
 R_API char *r_core_project_name(RCore *core, const char *prjfile) {
-	FILE *fd;
 	char buf[256], *file = NULL;
 	if (*prjfile != '/') {
 		return strdup (prjfile);
@@ -421,7 +399,7 @@ R_API char *r_core_project_name(RCore *core, const char *prjfile) {
 		eprintf ("Invalid project name '%s'\n", prjfile);
 		return NULL;
 	}
-	fd = r_sandbox_fopen (prj, "r");
+	FILE *fd = r_sandbox_fopen (prj, "r");
 	if (fd) {
 		for (;;) {
 			if (!fgets (buf, sizeof (buf), fd)) {
@@ -472,10 +450,10 @@ static bool store_files_and_maps(RCore *core, RIODesc *desc, ut32 id) {
 	if (desc) {
 		// reload bin info
 		r_cons_printf ("\"obf %s\"\n", desc->uri);
-		r_cons_printf ("\"ofs \\\"%s\\\" %s\"\n", desc->uri, r_str_rwx_i (desc->perm));
-		if ((maps = r_io_map_get_by_fd (core->io, id))) {	//wtf
+		r_cons_printf ("\"of \\\"%s\\\" %s\"\n", desc->uri, r_str_rwx_i (desc->perm));
+		if ((maps = r_io_map_get_by_fd (core->io, id))) { //wtf
 			r_list_foreach (maps, iter, map) {
-				r_cons_printf ("om %d 0x%"PFMT64x" 0x%"PFMT64x" 0x%"PFMT64x" %s%s%s\n", fdc,
+				r_cons_printf ("om %d 0x%" PFMT64x " 0x%" PFMT64x " 0x%" PFMT64x " %s%s%s\n", fdc,
 					r_io_map_begin (map), r_io_map_size (map), map->delta, r_str_rwx_i (map->perm),
 					map->name ? " " : "", r_str_get (map->name));
 			}
@@ -487,7 +465,7 @@ static bool store_files_and_maps(RCore *core, RIODesc *desc, ut32 id) {
 }
 #endif
 
-static bool project_save_script(RCore *core, const char *file, int opts) {
+R_API bool r_core_project_save_script(RCore *core, const char *file, int opts) {
 	char *filename, *hl, *ohl = NULL;
 	int fdold;
 
@@ -582,7 +560,8 @@ static bool project_save_script(RCore *core, const char *file, int opts) {
 	r_core_cmd (core, "wc*", 0);
 	if (opts & R_CORE_PRJ_ANAL_SEEK) {
 		r_cons_printf ("# seek\n"
-			"s 0x%08"PFMT64x "\n", core->offset);
+			       "s 0x%08" PFMT64x "\n",
+			core->offset);
 		r_cons_flush ();
 	}
 
@@ -600,32 +579,25 @@ static bool project_save_script(RCore *core, const char *file, int opts) {
 	return true;
 }
 
-R_API bool r_core_project_save_script(RCore *core, const char *file, int opts) {
-	return project_save_script (core, file, opts);
-}
-
 R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 	bool scr_null = false;
 	bool ret = true;
 	SdbListIter *it;
 	SdbNs *ns;
 	r_return_val_if_fail (prj_name && *prj_name, false);
-	char *script_path = get_project_script_path (core, prj_name);
+
 	if (r_config_get_b (core->config, "cfg.debug")) {
 		eprintf ("radare2 does not support projects on debugged bins.\n");
 		return false;
 	}
+	char *script_path = get_project_script_path (core, prj_name);
 	if (!script_path) {
 		eprintf ("Invalid project name '%s'\n", prj_name);
 		return false;
 	}
-	char *prj_dir = NULL;
-	if (r_str_endswith (script_path, R_SYS_DIR "rc.r2")) {
-		/* new project format */
-		prj_dir = r_file_dirname (script_path);
-	} else {
-		prj_dir = r_str_newf ("%s.d", script_path);
-	}
+	char *prj_dir = r_str_endswith (script_path, R_SYS_DIR "rc.r2")
+		? r_file_dirname (script_path)
+		: r_str_newf ("%s.d", script_path);
 	if (r_file_exists (script_path)) {
 		if (r_file_is_directory (script_path)) {
 			eprintf ("Structural error: rc.r2 shouldnt be a directory.\n");
@@ -634,7 +606,13 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 	if (!prj_dir) {
 		prj_dir = strdup (prj_name);
 	}
-	if (!r_file_exists (prj_dir)) {
+	if (r_core_is_project (core, prj_name) && strcmp (prj_name, r_config_get (core->config, "prj.name"))) {
+		eprintf ("A project with this name already exists\n");
+		free (script_path);
+		free (prj_dir);
+		return false;
+	}
+	if (!r_file_is_directory (prj_dir)) {
 		r_sys_mkdirp (prj_dir);
 	}
 	if (r_config_get_i (core->config, "scr.null")) {
@@ -654,9 +632,11 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 		}
 	}
 
-	if (!project_save_script (core, script_path, R_CORE_PRJ_ALL)) {
+	r_config_set (core->config, "prj.name", prj_name);
+	if (!r_core_project_save_script (core, script_path, R_CORE_PRJ_ALL)) {
 		eprintf ("Cannot open '%s' for writing\n", prj_name);
 		ret = false;
+		r_config_set (core->config, "prj.name", "");
 	}
 
 	if (r_config_get_i (core->config, "prj.files")) {
@@ -673,24 +653,34 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 		free (prj_bin_dir);
 		free (bin_file);
 	}
-	if (r_config_get_i (core->config, "prj.git")) {
-		char *cwd = r_sys_getdir ();
-		char *git_dir = r_str_newf ("%s" R_SYS_DIR ".git", prj_dir);
-		if (r_sys_chdir (prj_dir)) {
-			if (!r_file_is_directory (git_dir)) {
-				r_sys_cmd ("git init");
+	if (r_config_get_b (core->config, "prj.vc")) {
+		if (!rvc_git_repo_exists (core, prj_dir)) {
+			if (!rvc_git_init (core, prj_dir)) {
+				free (prj_dir);
+				free (script_path);
+				return false;
 			}
-			if (r_cons_is_interactive ()) {
-				r_sys_cmd ("git add * ; git commit -a");
+		}
+		RList *paths = r_list_new ();
+		if (paths) {
+			if (r_list_append (paths, prj_dir)) {
+				if (!rvc_git_commit (core, prj_dir, NULL, NULL, paths)) {
+					r_list_free (paths);
+					free (prj_dir);
+					free (script_path);
+					return false;
+				}
 			} else {
-				r_sys_cmd ("git add * ; git commit -a -m commit");
+				r_list_free (paths);
+				free (prj_dir);
+				free (script_path);
+				return false;
 			}
 		} else {
-			eprintf ("Cannot chdir %s\n", prj_dir);
+			free (prj_dir);
+			free (script_path);
+			return false;
 		}
-		r_sys_chdir (cwd);
-		free (git_dir);
-		free (cwd);
 	}
 	if (r_config_get_i (core->config, "prj.zip")) {
 		char *cwd = r_sys_getdir ();
@@ -716,6 +706,7 @@ R_API bool r_core_project_save(RCore *core, const char *prj_name) {
 	}
 	free (script_path);
 	r_config_set (core->config, "prj.name", prj_name);
+	r_core_project_undirty(core);
 	return ret;
 }
 
@@ -726,3 +717,9 @@ R_API char *r_core_project_notes_file(RCore *core, const char *prj_name) {
 	free (prjpath);
 	return notes_txt;
 }
+
+R_API bool r_core_project_is_saved(RCore *core) {
+	return !R_IS_DIRTY (core->config)
+		&& !R_IS_DIRTY (core->anal)
+		&& !R_IS_DIRTY (core->flags);
+}
