@@ -1,4 +1,4 @@
-/* radare - Copyright 2009-2020 - pancake, nibble */
+/* radare - Copyright 2009-2021 - pancake, nibble */
 
 #include "r_core.h"
 #include "r_socket.h"
@@ -47,7 +47,7 @@ typedef struct {
 } RapThread;
 
 R_API void r_core_wait(RCore *core) {
-	r_cons_singleton ()->context->breaked = true;
+	r_cons_context ()->breaked = true;
 	r_th_kill (httpthread, true);
 	r_th_kill (rapthread, true);
 	r_th_wait (httpthread);
@@ -74,7 +74,7 @@ static void http_logf(RCore *core, const char *fmt, ...) {
 	va_end (ap);
 }
 
-static char *rtrcmd (TextLog T, const char *str) {
+static char *rtrcmd(TextLog T, const char *str) {
 	char *res, *ptr2;
 	char *ptr = r_str_uri_encode (str);
 	char *uri = r_str_newf ("http://%s:%s/%s%s", T.host, T.port, T.file, ptr? ptr: str);
@@ -105,7 +105,7 @@ static void showcursor(RCore *core, int x) {
 
 // TODO: rename /name to /nick or /so?
 // clone of textlog_chat () using rtrcmd()
-static void rtr_textlog_chat (RCore *core, TextLog T) {
+static void rtr_textlog_chat(RCore *core, TextLog T) {
 	char prompt[64];
 	char buf[1024];
 	int lastmsg = 0;
@@ -180,7 +180,7 @@ R_API int r_core_rtr_http_stop(RCore *u) {
 	RSocket* sock;
 
 #if __WINDOWS__
-	r_socket_http_server_set_breaked (&r_cons_singleton ()->context->breaked);
+	r_socket_http_server_set_breaked (&r_cons_context ()->breaked);
 #endif
 	if (((size_t)u) > 0xff) {
 		port = listenport? listenport: r_config_get (
@@ -223,7 +223,7 @@ static void dietime(int sig) {
 static void activateDieTime(RCore *core) {
 	int dt = r_config_get_i (core->config, "http.dietime");
 	if (dt > 0) {
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 		r_sys_signal (SIGALRM, dietime);
 		alarm (dt);
 #else
@@ -352,7 +352,7 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 	ut64 m_off, reg_val;
 	bool be;
 	RDebugPid *dbgpid;
-	if (!core_ptr || ! cmd) {
+	if (!core_ptr || !cmd) {
 		return -1;
 	}
 	RCore *core = (RCore*) core_ptr;
@@ -426,7 +426,7 @@ static int r_core_rtr_gdb_cb(libgdbr_t *g, void *core_ptr, const char *cmd,
 			be = r_config_get_i (core->config, "cfg.bigendian");
 			if (isspace ((ut8)cmd[2])) { // dr reg
 				const char *name, *val_ptr;
-				char new_cmd[128] = { 0 };
+				char new_cmd[128] = {0};
 				int off = 0;
 				name = cmd + 3;
 				// Temporarily using new_cmd to store reg name
@@ -888,6 +888,55 @@ R_API void r_core_rtr_remove(RCore *core, const char *input) {
 	}
 }
 
+static char *errmsg_tmpfile = NULL;
+static int errmsg_fd = -1;
+
+R_API void r_core_rtr_event(RCore *core, const char *input) {
+	if (*input == '-') {
+		input++;
+		if (!strcmp (input, "errmsg")) {
+			if (errmsg_tmpfile) {
+				r_file_rm (errmsg_tmpfile);
+				errmsg_tmpfile = NULL;
+				if (errmsg_fd != -1) {
+					close (errmsg_fd);
+				}
+			}
+		}
+		return;
+	}
+	if (!strcmp (input, "errmsg")) {
+		// TODO: support udp, tcp, rap, ...
+#if __UNIX__ && !__wasi__
+		char *f = r_file_temp ("errmsg");
+		r_cons_printf ("%s\n", f);
+		r_file_rm (f);
+		errmsg_tmpfile = strdup (f);
+		int e = mkfifo (f, 0644);
+		if (e == -1) {
+			r_sys_perror ("mkfifo");
+		} else {
+			int ff = open (f, O_RDWR);
+			if (ff != -1) {
+				dup2 (ff, 2);
+				errmsg_fd = ff;
+			} else {
+				eprintf ("Cannot open fifo: %s\n", f);
+			}
+		}
+		// r_core_event (core, );
+		free (s);
+		free (f);
+		// TODO: those files are leaked when closing r_core_free() should be deleted
+#else
+		eprintf ("Not supported for your platform.\n");
+#endif
+	} else {
+		eprintf ("(%s)\n", input);
+		eprintf ("Event types: errmsg, stdin, stdout, stderr, #fdn\n");
+	}
+}
+
 R_API void r_core_rtr_session(RCore *core, const char *input) {
 	__rtr_shell (core, atoi (input));
 }
@@ -899,16 +948,16 @@ static bool r_core_rtr_rap_run(RCore *core, const char *input) {
 	if (fd) {
 		if (r_io_is_listener (core->io)) {
 			if (!r_core_serve (core, fd)) {
-				r_cons_singleton ()->context->breaked = true;
+				r_cons_context ()->breaked = true;
 			}
 			r_io_desc_close (fd);
 			// avoid double free, we are not the owners of this fd so we can't destroy it
 			//r_io_desc_free (fd);
 		}
 	} else {
-		r_cons_singleton ()->context->breaked = true;
+		r_cons_context ()->breaked = true;
 	}
-	return !r_cons_singleton ()->context->breaked;
+	return !r_cons_context ()->breaked;
 	// r_core_cmdf (core, "o rap://%s", input);
 }
 
@@ -988,7 +1037,8 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 			return;
 		}
 		r_socket_close (s);
-		if (!r_socket_connect (s, rh->host, sdb_fmt ("%d", rh->port), R_SOCKET_PROTO_TCP, 0)) {
+		r_strf_var (portstr, 32, "%d", rh->port);
+		if (!r_socket_connect (s, rh->host, portstr, R_SOCKET_PROTO_TCP, 0)) {
 			eprintf ("Error: Cannot connect to '%s' (%d)\n", rh->host, rh->port);
 			r_socket_free (s);
 			return;
@@ -1048,7 +1098,7 @@ R_API void r_core_rtr_cmd(RCore *core, const char *input) {
 }
 
 // TODO: support len for binary data?
-R_API char *r_core_rtr_cmds_query (RCore *core, const char *host, const char *port, const char *cmd) {
+R_API char *r_core_rtr_cmds_query(RCore *core, const char *host, const char *port, const char *cmd) {
 	RSocket *s = r_socket_new (0);
 	const int timeout = 0;
 	char *rbuf = NULL;
@@ -1280,7 +1330,7 @@ beach:
 
 #else
 
-R_API int r_core_rtr_cmds (RCore *core, const char *port) {
+R_API int r_core_rtr_cmds(RCore *core, const char *port) {
 	unsigned char buf[4097];
 	RSocket *ch = NULL;
 	int i, ret;
