@@ -3,7 +3,7 @@
 // return 1 on error
 static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *path) {
 	RConfig *newcfg = NULL, *origcfg = NULL;
-	char headers[128] = R_EMPTY;
+	char headers[128] = {0};
 	RSocketHTTPRequest *rs;
 	char buf[32];
 	int ret = 0;
@@ -17,6 +17,7 @@ static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *
 	const char *homeroot = r_config_get (core->config, "http.homeroot");
 	const char *port = r_config_get (core->config, "http.port");
 	const char *allow = r_config_get (core->config, "http.allow");
+	const char *basepath = r_config_get (core->config, "http.basepath");
 	const char *httpui = r_config_get (core->config, "http.ui");
 	const char *httpauthfile = r_config_get (core->config, "http.authfile");
 	char *pfile = NULL;
@@ -39,12 +40,7 @@ static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *
 		path = NULL;
 	} else {
 		if (core->io->desc && (!path || !*path)) {
-			if (!strcmp (httpui, "p")
-			|| !strcmp (httpui, "m")
-			|| !strcmp (httpui, "enyo")
-			|| !strcmp (httpui, "t")) {
-				path = httpui;
-			}
+			path = httpui;
 		}
 	}
 
@@ -66,7 +62,7 @@ static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *
 			} else if (!strcmp (host, "local")) {
 				s->local = true;
 				r_config_set (core->config, "http.bind", "localhost");
-			} else if (host[0]=='0' || !strcmp (host, "public")) {
+			} else if (R_STR_ISEMPTY (host) || !strcmp (host, "public")) {
 				// public
 				host = "127.0.0.1";
 				r_config_set (core->config, "http.bind", "0.0.0.0");
@@ -178,6 +174,20 @@ static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *
 		rs = r_socket_http_accept (s, &so);
 		r_cons_sleep_end (bed);
 
+		if (*basepath && strcmp (basepath, "/")) {
+			if (R_STR_ISEMPTY (rs->path) || !strcmp (rs->path, "/")) {
+				char *res = r_str_newf ("Location: %s/\n%s", basepath, headers);
+				r_socket_http_response (rs, 302, NULL, 0, res);
+				r_socket_http_close (rs);
+				continue;
+			}
+			if (r_str_startswith (rs->path, basepath)) {
+				char *p = strdup (rs->path + strlen (basepath));
+				free (rs->path);
+				rs->path = p;
+			}
+		}
+
 		origoff = core->offset;
 		origblk = core->block;
 		origblksz = core->blocksize;
@@ -336,6 +346,8 @@ static int r_core_rtr_http_run(RCore *core, int launch, int browse, const char *
 								r_core_cmd0 (core, cmd + 1);
 								out = NULL;
 							} else {
+								RConsContext *ctx = r_cons_context ();
+								ctx->noflush = false;
 								out = r_core_cmd_str_pipe (core, cmd);
 							}
 
@@ -512,7 +524,7 @@ the_end:
 }
 
 #if 0
-static RThreadFunctionRet r_core_rtr_http_thread (RThread *th) {
+static RThreadFunctionRet r_core_rtr_http_thread(RThread *th) {
 	if (!th) {
 		return false;
 	}
