@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2018 - pancake, jduck, TheLemonMan, saucec0de */
+/* radare - LGPL - Copyright 2009-2021 - pancake, jduck, TheLemonMan, saucec0de */
 
 #include <r_debug.h>
 #include <r_drx.h>
@@ -11,13 +11,11 @@ R_LIB_VERSION(r_debug);
 #define DBG_BUF_SIZE 512
 
 R_API RDebugInfo *r_debug_info(RDebug *dbg, const char *arg) {
-	if (!dbg || !dbg->h || !dbg->h->info) {
-		return NULL;
-	}
+	r_return_val_if_fail (dbg && dbg->h, NULL);
 	if (dbg->pid < 0) {
 		return NULL;
 	}
-	return dbg->h->info (dbg, arg);
+	return dbg->h->info? dbg->h->info (dbg, arg): NULL;
 }
 
 R_API void r_debug_info_free(RDebugInfo *rdi) {
@@ -105,6 +103,7 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 		if (!b) { /* we don't. nothing left to do */
 			/* Some targets set pc to breakpoint */
 			b = r_bp_get_at (dbg->bp, pc);
+#if __i386__ || __x86_64__
 			if (!b) {
 				/* handle the case of hw breakpoints - notify the user */
 				int drx_reg_idx = r_debug_drx_at (dbg, pc);
@@ -115,10 +114,9 @@ static int r_debug_bp_hit(RDebug *dbg, RRegItem *pc_ri, ut64 pc, RBreakpointItem
 				/* Couldn't find the break point. Nothing more to do... */
 				return true;
 			}
-			else {
-				dbg->pc_at_bp_set = true;
-				dbg->pc_at_bp = true;
-			}
+#endif
+			dbg->pc_at_bp_set = true;
+			dbg->pc_at_bp = true;
 		} else {
 			dbg->pc_at_bp_set = true;
 			dbg->pc_at_bp = false;
@@ -214,7 +212,7 @@ static int r_debug_bps_enable(RDebug *dbg) {
  *
  * if the user wants to step, the single step here does the job.
  */
-static int r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
+static bool r_debug_recoil(RDebug *dbg, RDebugRecoilMode rc_mode) {
 	/* if bp_addr is not set, we must not have actually hit a breakpoint */
 	if (!dbg->reason.bp_addr) {
 		return r_debug_bps_enable (dbg);
@@ -326,7 +324,7 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 		}
 	}
 	if (watch) {
-		hw = 1; //XXX
+		hw = 1; // XXX
 		bpi = r_bp_watch_add (dbg->bp, addr, bpsz, hw, rw);
 	} else {
 		bpi = hw
@@ -347,7 +345,7 @@ R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch
 
 static const char *r_debug_str_callback(RNum *userptr, ut64 off, int *ok) {
 	// RDebug *dbg = (RDebug *)userptr;
-	eprintf ("STR CALLBACK WTF WTF WTF\n");
+	// TODO: implement the rnum callback for str or just get rid of it as we dont need it
 	return NULL;
 }
 
@@ -404,7 +402,7 @@ R_API RDebug *r_debug_new(int hard) {
 
 static int free_tracenodes_entry(RDebug *dbg, const char *k, const char *v) {
 	ut64 v_num = r_num_get (NULL, v);
-	free((void *)(size_t)v_num);
+	free ((void *)(size_t)v_num);
 	return true;
 }
 
@@ -413,7 +411,7 @@ R_API void r_debug_tracenodes_reset(RDebug *dbg) {
 	sdb_reset (dbg->tracenodes);
 }
 
-R_API RDebug *r_debug_free(RDebug *dbg) {
+R_API void r_debug_free(RDebug *dbg) {
 	if (dbg) {
 		// TODO: free it correctly.. we must ensure this is an instance and not a reference..
 		r_bp_free (dbg->bp);
@@ -440,17 +438,24 @@ R_API RDebug *r_debug_free(RDebug *dbg) {
 		free (dbg->glob_unlibs);
 		free (dbg);
 	}
-	return NULL;
 }
 
-R_API int r_debug_attach(RDebug *dbg, int pid) {
-	int ret = false;
-	if (dbg && dbg->h && dbg->h->attach) {
+R_API bool r_debug_attach(RDebug *dbg, int pid) {
+	r_return_val_if_fail (dbg, false);
+	if (pid < 0) {
+		return false;
+	}
+	bool ret = false;
+	if (dbg->h && dbg->h->attach) {
 		ret = dbg->h->attach (dbg, pid);
-		if (ret != -1) {
-			r_debug_select (dbg, pid, ret); //dbg->pid, dbg->tid);
+		if (ret) {
+			dbg->tid = pid;
+			// dbg->pid = pid;
+			// r_debug_select (dbg, pid, ret);
+			r_debug_select (dbg, dbg->pid, dbg->tid);
 		}
 	}
+	dbg->reason.type = R_DEBUG_REASON_STOPPED;
 	return ret;
 }
 
@@ -465,9 +470,14 @@ R_API int r_debug_stop(RDebug *dbg) {
 R_API bool r_debug_set_arch(RDebug *dbg, const char *arch, int bits) {
 	if (arch && dbg && dbg->h) {
 		switch (bits) {
+		case 16:
+			if (dbg->h->bits == 16) {
+				dbg->bits = R_SYS_BITS_16;
+			}
+			break;
 		case 27:
 			if (dbg->h->bits == 27) {
-				dbg->bits = 27;
+				dbg->bits = R_SYS_BITS_27;
 			}
 			break;
 		case 32:
@@ -588,21 +598,18 @@ R_API int r_debug_detach(RDebug *dbg, int pid) {
 }
 
 R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
-	ut64 pc = 0;
-	int prev_pid = dbg->pid;
-	int prev_tid = dbg->tid;
-
+	r_return_val_if_fail (dbg, false);
 	if (pid < 0) {
 		return false;
 	}
 	if (tid < 0) {
 		tid = pid;
 	}
-	if (pid != -1 && tid != -1) {
-		if ((pid != dbg->pid || tid != dbg->tid) && dbg->verbose) {
-			eprintf ("= attach %d %d\n", pid, tid);
-		}
-	} else {
+#if 0
+	pid = r_io_desc_get_pid (dbg->iob.io->desc);
+	tid = r_io_desc_get_tid (dbg->iob.io->desc);
+#endif
+	if (pid == -1 && tid == -1) {
 		if (dbg->pid != -1) {
 			eprintf ("Child %d is dead\n", dbg->pid);
 		}
@@ -610,20 +617,22 @@ R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
 	if (pid < 0 || tid < 0) {
 		return false;
 	}
-
-	if (dbg->h && dbg->h->select && !dbg->h->select (dbg, pid, tid)) {
-		return false;
+	if (dbg->h && dbg->h->select) {
+		if (!dbg->h->select (dbg, pid, tid)) {
+			return false;
+		}
 	}
-
-	// Don't change the pid/tid if the plugin already modified it due to internal constraints
-	if (dbg->pid == prev_pid) {
-		dbg->pid = pid;
+	dbg->pid = pid;
+	dbg->tid = tid;
+	if (dbg->tid != -1) {
+		char *pidcmd = r_str_newf ("pid %d", dbg->tid);
+		if (pidcmd) {
+			free (r_io_system (dbg->iob.io, pidcmd));
+			free (pidcmd);
+		}
+	} else {
+		eprintf ("Cannot find pid for child %d\n", dbg->pid);
 	}
-	if (dbg->tid == prev_tid) {
-		dbg->tid = tid;
-	}
-
-	free (r_io_system (dbg->iob.io, sdb_fmt ("pid %d", dbg->tid)));
 
 	// Synchronize with the current thread's data
 	if (dbg->corebind.core) {
@@ -632,40 +641,43 @@ R_API bool r_debug_select(RDebug *dbg, int pid, int tid) {
 		r_reg_arena_swap (core->dbg->reg, true);
 		r_debug_reg_sync (dbg, R_REG_TYPE_ALL, false);
 
-		pc = r_debug_reg_get (dbg, "PC");
-		core->offset = pc;
+		core->offset = r_debug_reg_get (dbg, "PC");
 	}
 
+	dbg->main_arena_resolved = false;
+
 	return true;
 }
 
+// TODO type should be enum so we can ensure to not miss an item
 R_API const char *r_debug_reason_to_string(int type) {
 	switch (type) {
-	case R_DEBUG_REASON_DEAD: return "dead";
 	case R_DEBUG_REASON_ABORT: return "abort";
-	case R_DEBUG_REASON_SEGFAULT: return "segfault";
-	case R_DEBUG_REASON_NONE: return "none";
-	case R_DEBUG_REASON_SIGNAL: return "signal";
 	case R_DEBUG_REASON_BREAKPOINT: return "breakpoint";
-	case R_DEBUG_REASON_TRACEPOINT: return "tracepoint";
-	case R_DEBUG_REASON_READERR: return "read-error";
-	case R_DEBUG_REASON_WRITERR: return "write-error";
+	case R_DEBUG_REASON_DEAD: return "dead";
 	case R_DEBUG_REASON_DIVBYZERO: return "div-by-zero";
-	case R_DEBUG_REASON_ILLEGAL: return "illegal";
-	case R_DEBUG_REASON_UNKNOWN: return "unknown";
 	case R_DEBUG_REASON_ERROR: return "error";
-	case R_DEBUG_REASON_NEW_PID: return "new-pid";
-	case R_DEBUG_REASON_NEW_TID: return "new-tid";
-	case R_DEBUG_REASON_NEW_LIB: return "new-lib";
+	case R_DEBUG_REASON_EXIT_LIB: return "exit-lib";
 	case R_DEBUG_REASON_EXIT_PID: return "exit-pid";
 	case R_DEBUG_REASON_EXIT_TID: return "exit-tid";
-	case R_DEBUG_REASON_EXIT_LIB: return "exit-lib";
-	case R_DEBUG_REASON_TRAP: return "trap";
-	case R_DEBUG_REASON_SWI: return "software-interrupt";
-	case R_DEBUG_REASON_INT: return "interrupt";
 	case R_DEBUG_REASON_FPU: return "fpu";
+	case R_DEBUG_REASON_ILLEGAL: return "illegal";
+	case R_DEBUG_REASON_INT: return "interrupt";
+	case R_DEBUG_REASON_NEW_LIB: return "new-lib";
+	case R_DEBUG_REASON_NEW_PID: return "new-pid";
+	case R_DEBUG_REASON_NEW_TID: return "new-tid";
+	case R_DEBUG_REASON_NONE: return "none";
+	case R_DEBUG_REASON_READERR: return "read-error";
+	case R_DEBUG_REASON_SEGFAULT: return "segfault";
+	case R_DEBUG_REASON_SIGNAL: return "signal";
 	case R_DEBUG_REASON_STEP: return "step";
+	case R_DEBUG_REASON_STOPPED: return "stopped";
+	case R_DEBUG_REASON_SWI: return "software-interrupt";
+	case R_DEBUG_REASON_TRACEPOINT: return "tracepoint";
+	case R_DEBUG_REASON_TRAP: return "trap";
+	case R_DEBUG_REASON_UNKNOWN: return "unknown";
 	case R_DEBUG_REASON_USERSUSP: return "suspended-by-user";
+	case R_DEBUG_REASON_WRITERR: return "write-error";
 	}
 	return "unhandled";
 }
@@ -690,7 +702,6 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 	if (!dbg) {
 		return reason;
 	}
-
 	if (bp) {
 		*bp = NULL;
 	}
@@ -713,7 +724,6 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			//r_debug_select (dbg, -1, -1);
 			return R_DEBUG_REASON_DEAD;
 		}
-
 #if __linux__
 		// Letting other threads running will cause ptrace commands to fail
 		// when writing to the same process memory to set/unset breakpoints
@@ -722,7 +732,6 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			r_debug_stop (dbg);
 		}
 #endif
-
 		/* propagate errors from the plugin */
 		if (reason == R_DEBUG_REASON_ERROR) {
 			return R_DEBUG_REASON_ERROR;
@@ -745,7 +754,7 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			/* get the program coounter */
 			pc_ri = r_reg_get (dbg->reg, dbg->reg->name[R_REG_NAME_PC], -1);
 			if (!pc_ri) { /* couldn't find PC?! */
-				eprintf ("Couldn't find PC!\n");
+				eprintf ("Couldn't find the program counter!\n");
 				return R_DEBUG_REASON_ERROR;
 			}
 
@@ -778,9 +787,10 @@ R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp) {
 			/* handle signal on continuations here */
 			int what = r_debug_signal_what (dbg, dbg->reason.signum);
 			const char *name = r_signal_to_string (dbg->reason.signum);
+			const char *humn = r_signal_to_human (dbg->reason.signum);
 			if (name && strcmp ("SIGTRAP", name)) {
-				r_cons_printf ("[+] signal %d aka %s received %d\n",
-						dbg->reason.signum, name, what);
+				r_cons_printf ("[+] signal %d aka %s received %d (%s)\n",
+						dbg->reason.signum, name, what, humn);
 			}
 		}
 	}
@@ -792,7 +802,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 	ut64 pc, sp, r;
 	ut64 next[2];
 	RAnalOp op;
-	int br, i, ret;
+	int br, i;
 	union {
 		ut64 r64;
 		ut32 r32[2];
@@ -886,7 +896,7 @@ R_API int r_debug_step_soft(RDebug *dbg) {
 		}
 	}
 
-	ret = r_debug_continue (dbg);
+	bool ret = r_debug_continue (dbg);
 
 	for (i = 0; i < br; i++) {
 		r_bp_del (dbg->bp, next[i]);
@@ -1147,7 +1157,7 @@ R_API int r_debug_continue_kill(RDebug *dbg, int sig) {
 			if (reg->cnum <= dbg->session->cnum) {
 				continue;
 			}
-			has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC) != NULL;
+			has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC);
 			if (has_bp) {
 				eprintf ("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
 				r_debug_goto_cnum (dbg, reg->cnum);
@@ -1320,6 +1330,9 @@ repeat:
 }
 
 R_API int r_debug_continue(RDebug *dbg) {
+	if (dbg->pid < 0) {
+		return -1;
+	}
 	return r_debug_continue_kill (dbg, 0); //dbg->reason.signum);
 }
 
@@ -1396,7 +1409,7 @@ static int r_debug_continue_until_internal(RDebug *dbg, ut64 addr, bool block) {
 		return false;
 	}
 	// Check if there was another breakpoint set at addr
-	bool has_bp = r_bp_get_in (dbg->bp, addr, R_BP_PROT_EXEC) != NULL;
+	bool has_bp = r_bp_get_in (dbg->bp, addr, R_BP_PROT_EXEC);
 	if (!has_bp) {
 		r_bp_add_sw (dbg->bp, addr, dbg->bpsize, R_BP_PROT_EXEC);
 	}
@@ -1446,7 +1459,7 @@ R_API bool r_debug_continue_back(RDebug *dbg) {
 		if (reg->cnum >= dbg->session->cnum) {
 			continue;
 		}
-		has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC) != NULL;
+		has_bp = r_bp_get_in (dbg->bp, reg->data, R_BP_PROT_EXEC);
 		if (has_bp) {
 			cnum = reg->cnum;
 			eprintf ("hit breakpoint at: 0x%" PFMT64x " cnum: %d\n", reg->data, reg->cnum);
@@ -1624,7 +1637,7 @@ R_API bool r_debug_is_dead(RDebug *dbg) {
 	if (!strcmp (dbg->h->name, "io")) {
 		return false;
 	}
-	bool is_dead = (dbg->pid == -1 && strncmp (dbg->h->name, "gdb", 3)) || (dbg->reason.type == R_DEBUG_REASON_DEAD);
+	bool is_dead = (dbg->pid < 0 && strncmp (dbg->h->name, "gdb", 3)) || (dbg->reason.type == R_DEBUG_REASON_DEAD);
 	if (dbg->pid > 0 && dbg->h && dbg->h->kill) {
 		is_dead = !dbg->h->kill (dbg, dbg->pid, false, 0);
 	}
@@ -1679,7 +1692,7 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 	if (pid < 0 || tid < 0) {
 		return 0LL;
 	}
-	if (r_debug_attach (dbg, pid) == -1) {
+	if (!r_debug_attach (dbg, pid)) {
 		return 0LL;
 	}
 #if __WINDOWS__
@@ -1694,13 +1707,15 @@ R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file) {
 	r_debug_select (dbg, pid, tid);
 	r_debug_map_sync (dbg);
 	char *abspath = r_sys_pid_to_path (pid);
+	if (file) {
 #if !__WINDOWS__
-	if (!abspath) {
-		abspath = r_file_abspath (file);
-	}
+		if (!abspath) {
+			abspath = r_file_abspath (file);
+		}
 #endif
-	if (!abspath) {
-		abspath = strdup (file);
+		if (!abspath) {
+			abspath = strdup (file);
+		}
 	}
 	if (abspath) {
 		r_list_foreach (dbg->maps, iter, map) {
