@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2021 - pancake */
+/* radare - LGPL - Copyright 2009-2022 - pancake */
 
 #define USE_THREADS 1
 #define ALLOW_THREADED 0
@@ -73,11 +73,7 @@ static int r_main_version_verify(int show) {
 		}
 	}
 	if (ret) {
-		if (show) {
-			eprintf ("Warning: r2 library versions mismatch!\n");
-		} else {
-			eprintf ("Warning: r2 library versions mismatch! See r2 -V\n");
-		}
+		eprintf ("Warning: r2 library versions mismatch! Check r2 -V");
 	}
 	return ret;
 }
@@ -161,7 +157,7 @@ static int main_help(int line) {
 		" R2_NOPLUGINS do not load r2 shared plugins\n"
 		" R2_RCFILE    ~/.radare2rc (user preferences, batch script)\n" // TOO GENERIC
 		" R2_RDATAHOME %s\n" // TODO: rename to RHOME R2HOME?
-		" R2_VERSION   contains the current version of r2\n" 
+		" R2_VERSION   contains the current version of r2\n"
 		"Paths:\n"
 		" R2_PREFIX    "R2_PREFIX"\n"
 		" R2_INCDIR    "R2_INCDIR"\n"
@@ -280,7 +276,7 @@ beach:
 }
 
 static bool mustSaveHistory(RConfig *c) {
-	if (!r_config_get_i (c, "scr.histsave")) {
+	if (!r_config_get_i (c, "scr.hist.save")) {
 		return false;
 	}
 	if (!r_cons_is_interactive ()) {
@@ -391,11 +387,11 @@ R_API int r_main_radare2(int argc, const char **argv) {
 	if (r_sys_getenv_asbool ("R2_DEBUG")) {
 		char *sysdbg = r_sys_getenv ("R2_DEBUG_TOOL");
 		char *fmt = (sysdbg && *sysdbg)
-			? r_str_newf ("%s %%d", sysdbg)
+			? strdup (sysdbg)
 #if __APPLE__
-			: r_str_newf ("lldb -p %%d");
+			: strdup ("lldb -p");
 #else
-			: r_str_newf ("gdb --pid %%d");
+			: strdup ("gdb --pid");
 #endif
 		r_sys_crash_handler (fmt);
 		free (fmt);
@@ -541,7 +537,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 		case 'H':
 			main_print_var (opt.arg);
 			LISTS_FREE ();
-            free (debugbackend);
+			free (debugbackend);
 			return 0;
 		case 'i':
 			if (R_STR_ISEMPTY (opt.arg)) {
@@ -569,6 +565,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			do_list_io_plugins = true;
 			break;
 		case 'm':
+			r_config_set_i (r->config, "io.va", 1);
 			mapaddr = r_num_math (r->num, opt.arg);
 			s_seek = opt.arg;
 			r_config_set_i (r->config, "file.offset", mapaddr);
@@ -720,6 +717,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			return 1;
 		}
 		if (2 != new_stderr) {
+#if !__wasi__
 			if (-1 == dup2 (new_stderr, 2)) {
 				eprintf ("Failed to dup2 stderr");
 				free (envprofile);
@@ -727,6 +725,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 				R_FREE (debugbackend);
 				return 1;
 			}
+#endif
 			if (-1 == close (new_stderr)) {
 				eprintf ("Failed to close %s", nul);
 				LISTS_FREE ();
@@ -817,7 +816,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 		if (opt.ind >= argc && !haveRarunProfile) {
 			eprintf ("Missing argument for -d\n");
 			LISTS_FREE ();
-			R_FREE (debugbackend);
+			free (debugbackend);
 			free (envprofile);
 			return 1;
 		}
@@ -857,7 +856,12 @@ R_API int r_main_radare2(int argc, const char **argv) {
 	r_bin_force_plugin (r->bin, forcebin);
 
 	if (project_name) {
-		r_core_project_open (r, project_name);
+		if (!r_core_project_open (r, project_name)) {
+			eprintf ("Cannot find project.\n");
+			free (debugbackend);
+			free (envprofile);
+			return 1;
+		}
 	}
 
 	if (do_connect) {
@@ -865,7 +869,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 		if (opt.ind >= argc) {
 			eprintf ("Missing URI for -C\n");
 			LISTS_FREE ();
-			R_FREE (debugbackend);
+			free (debugbackend);
 			free (envprofile);
 			return 1;
 		}
@@ -908,9 +912,9 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			if (file && *file && *file != '.') {
 				complete_path = r_str_newf ("%s"R_SYS_DIR"%s", path, file);
 				if (r_str_endswith (complete_path, "gz")) {
-					r_sign_load_gz (r->anal, complete_path);
+					r_sign_load_gz (r->anal, complete_path, false);
 				} else {
-					r_sign_load (r->anal, complete_path);
+					r_sign_load (r->anal, complete_path, false);
 				}
 				free (complete_path);
 			}
@@ -926,6 +930,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			free (pfile);
 			R_FREE (debugbackend);
 			free (envprofile);
+			free (debugbackend);
 			return 1;
 		}
 		if (r_sys_chdir (argv[opt.ind])) {
@@ -967,7 +972,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 				R_FREE (debugbackend);
 				return 1;
 			}
-			r_io_map_new (r->io, fh->fd, 7, 0LL, mapaddr,
+			r_io_map_add (r->io, fh->fd, 7, 0LL, mapaddr,
 					r_io_fd_size (r->io, fh->fd));
 			r_io_write_at (r->io, mapaddr, (const ut8 *)buf, sz);
 			r_core_block_read (r);
@@ -979,6 +984,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			free (buf);
 			LISTS_FREE ();
 			free (envprofile);
+			free (debugbackend);
 			return 1;
 		}
 	} else if (strcmp (argv[opt.ind - 1], "--") && !project_name) {
@@ -992,7 +998,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			if (opt.ind >= argc) {
 				eprintf ("No program given to -d\n");
 				LISTS_FREE ();
-				R_FREE (debugbackend);
+				free (debugbackend);
 				free (envprofile);
 				return 1;
 			}
@@ -1015,39 +1021,41 @@ R_API int r_main_radare2(int argc, const char **argv) {
 					iod = (r->io && fh) ? r_io_desc_get (r->io, fh->fd) : NULL;
 					if (!strcmp (debugbackend, "gdb")) {
 						const char *filepath = r_config_get (r->config, "dbg.exe.path");
-						ut64 addr = baddr;
-						if (addr == UINT64_MAX) {
-							addr = r_config_get_i (r->config, "bin.baddr");
-						}
-						if (r_file_exists (filepath) && !r_file_is_directory (filepath)) {
-							char *newpath = r_file_abspath (filepath);
-							if (newpath) {
-								if (iod) {
-									free (iod->name);
-									iod->name = newpath;
-								}
-								if (addr == UINT64_MAX) {
-									addr = r_debug_get_baddr (r->dbg, newpath);
-								}
-								r_core_bin_load (r, NULL, addr);
+						if (!R_STR_ISEMPTY (filepath)) {
+							ut64 addr = baddr;
+							if (addr == UINT64_MAX) {
+								addr = r_config_get_i (r->config, "bin.baddr");
 							}
-						} else if (is_valid_gdb_file (fh)) {
-							filepath = iod->name;
 							if (r_file_exists (filepath) && !r_file_is_directory (filepath)) {
-								if (addr == UINT64_MAX) {
-									addr = r_debug_get_baddr (r->dbg, filepath);
+								char *newpath = r_file_abspath (filepath);
+								if (newpath) {
+									if (iod) {
+										free (iod->name);
+										iod->name = newpath;
+									}
+									if (addr == UINT64_MAX) {
+										addr = r_debug_get_baddr (r->dbg, newpath);
+									}
+									r_core_bin_load (r, NULL, addr);
 								}
-								r_core_bin_load (r, filepath, addr);
-							} else if ((filepath = get_file_in_cur_dir (filepath))) {
-								// Present in local directory
-								if (iod) {
-									free (iod->name);
-									iod->name = (char*) filepath;
+							} else if (is_valid_gdb_file (fh)) {
+								filepath = iod->name;
+								if (r_file_exists (filepath) && !r_file_is_directory (filepath)) {
+									if (addr == UINT64_MAX) {
+										addr = r_debug_get_baddr (r->dbg, filepath);
+									}
+									r_core_bin_load (r, filepath, addr);
+								} else if ((filepath = get_file_in_cur_dir (filepath))) {
+									// Present in local directory
+									if (iod) {
+										free (iod->name);
+										iod->name = (char*) filepath;
+									}
+									if (addr == UINT64_MAX) {
+										addr = r_debug_get_baddr (r->dbg, filepath);
+									}
+									r_core_bin_load (r, NULL, addr);
 								}
-								if (addr == UINT64_MAX) {
-									addr = r_debug_get_baddr (r->dbg, filepath);
-								}
-								r_core_bin_load (r, NULL, addr);
 							}
 						}
 					}
@@ -1132,6 +1140,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			if (opt.ind < argc) {
 				R_FREE (pfile);
 				while (opt.ind < argc) {
+					R_FREE (pfile);
 					pfile = strdup (argv[opt.ind++]);
 #if __WINDOWS__
 					pfile = r_acp_to_utf8 (pfile);
@@ -1171,15 +1180,17 @@ R_API int r_main_radare2(int argc, const char **argv) {
 							}
 							/* Load rbin info from r2 dbg:// or r2 /bin/ls */
 							/* the baddr should be set manually here */
-							(void)r_core_bin_load (r, filepath, baddr);
-							// check if bin info is loaded and complain if -B was used
-							RBinFile *bi = r_bin_cur (r->bin);
-							bool haveBinInfo = bi && bi->o && bi->o->info && bi->o->info->type;
-							if (!haveBinInfo && baddr != UT64_MAX) {
-								eprintf ("Warning: Don't use -B on unknown files. Consider using -m.\n");
+							if (filepath) {
+								(void)r_core_bin_load (r, filepath, baddr);
+								// check if bin info is loaded and complain if -B was used
+								RBinFile *bi = r_bin_cur (r->bin);
+								bool haveBinInfo = bi && bi->o && bi->o->info && bi->o->info->type;
+								if (!haveBinInfo && baddr != UT64_MAX) {
+									eprintf ("Warning: Don't use -B on unknown files. Consider using -m.\n");
+								}
 							}
 						} else {
-							r_io_map_new (r->io, iod->fd, perms, 0LL, mapaddr, r_io_desc_size (iod));
+							r_io_map_add (r->io, iod->fd, perms, 0LL, mapaddr, r_io_desc_size (iod));
 							if (load_bin == LOAD_BIN_STRUCTURES_ONLY) {
 								r_core_bin_load_structs (r, iod->name);
 							}
@@ -1203,7 +1214,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 						iod = r->io ? r_io_desc_get (r->io, fh->fd) : NULL;
 						if (iod) {
 							perms = iod->perm;
-							r_io_map_new (r->io, iod->fd, perms, 0LL, 0LL, r_io_desc_size (iod));
+							r_io_map_add (r->io, iod->fd, perms, 0LL, 0LL, r_io_desc_size (iod));
 						}
 					}
 				}
@@ -1228,19 +1239,10 @@ R_API int r_main_radare2(int argc, const char **argv) {
 			// NOTE: the baddr is redefined to support PIE/ASLR
 			baddr = r_debug_get_baddr (r->dbg, pfile);
 
-			if (baddr != UT64_MAX && baddr != 0 && r->dbg->verbose) {
-				eprintf ("bin.baddr 0x%08" PFMT64x "\n", baddr);
-			}
 			if (load_bin == LOAD_BIN_ALL) {
-				if (baddr && baddr != UT64_MAX && r->dbg->verbose) {
-					eprintf ("Using 0x%" PFMT64x "\n", baddr);
-				}
 				if (r_core_bin_load (r, pfile, baddr)) {
 					RBinObject *obj = r_bin_cur_object (r->bin);
 					if (obj && obj->info) {
-						if (r->dbg->verbose) {
-							eprintf ("asm.bits %d\n", obj->info->bits);
-						}
 #if __linux__ && __GNU_LIBRARY__ && __GLIBC__ && __GLIBC_MINOR__ && __x86_64__
 						ut64 bitness = r_config_get_i (r->config, "asm.bits");
 						if (bitness == 32) {
@@ -1252,8 +1254,10 @@ R_API int r_main_radare2(int argc, const char **argv) {
 				}
 			}
 			r_core_cmd0 (r, ".dm*");
-			// Set Thumb Mode if necessary
-			r_core_cmd0 (r, "dr? thumb;?? e asm.bits=16");
+			if (asmarch && r_str_startswith (asmarch, "arm") && r_config_get_i (r->config, "asm.bits") < 64) {
+				// Set Thumb Mode if necessary
+				r_core_cmd0 (r, "dr? thumb;?? e asm.bits=16");
+			}
 			r_cons_reset ();
 		}
 		if (!pfile) {
@@ -1398,7 +1402,7 @@ R_API int r_main_radare2(int argc, const char **argv) {
 	}
 #if UNCOLORIZE_NONTTY
 #if __UNIX__
-	if (!r_cons_isatty ()) {
+	if (!r_cons_is_tty ()) {
 		r_config_set_i (r->config, "scr.color", COLOR_MODE_DISABLED);
 	}
 #endif
@@ -1461,8 +1465,11 @@ R_API int r_main_radare2(int argc, const char **argv) {
 				r_core_cmd0 (r, "aeip");
 			}
 		}
+		r_core_project_undirty (r);
 		for (;;) {
-			r_core_prompt_loop (r);
+			if (!r_core_prompt_loop (r)) {
+				quietLeak = true;
+			}
 			ret = r->num->value;
 			debug = r_config_get_i (r->config, "cfg.debug");
 			if (ret != -1 && r_cons_is_interactive ()) {
@@ -1501,26 +1508,30 @@ R_API int r_main_radare2(int argc, const char **argv) {
 				}
 
 				const char *prj = r_config_get (r->config, "prj.name");
-				if (no_question_save) {
-					if (prj && *prj && y_save_project){
-						r_core_project_save (r, prj);
+				if (R_STR_ISNOTEMPTY (prj)) {
+					if (r_core_project_is_saved (r) && !r_config_get_b (r->config, "prj.alwaysprompt")) {
+						break;
 					}
-				} else {
-					question = r_str_newf ("Do you want to save the '%s' project? (Y/n)", prj);
-					if (prj && *prj && r_cons_yesno ('y', "%s", question)) {
-						r_core_project_save (r, prj);
+					if (no_question_save) {
+						if (y_save_project) {
+							r_core_project_save (r, prj);
+						}
+					} else {
+						question = r_str_newf ("Do you want to save the '%s' project? (Y/n)", prj);
+						if (r_cons_yesno ('y', "%s", question)) {
+							r_core_project_save (r, prj);
+						}
+						free (question);
 					}
-					free (question);
 				}
-
-				if (r_config_get_i (r->config, "scr.confirmquit")) {
+				if (r_config_get_b (r->config, "scr.confirmquit")) {
 					if (!r_cons_yesno ('n', "Do you want to quit? (Y/n)")) {
 						continue;
 					}
 				}
 			} else {
 				// r_core_project_save (r, prj);
-				if (debug && r_config_get_i (r->config, "dbg.exitkills")) {
+				if (debug && r_config_get_b (r->config, "dbg.exitkills")) {
 					r_debug_kill (r->dbg, 0, false, 9); // KILL
 				}
 
