@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2020 - pancake */
+/* radare - LGPL - Copyright 2007-2022 - pancake */
 
 #include "r_types.h"
 #include "r_util.h"
@@ -25,6 +25,12 @@
 #include <process.h>
 #endif
 
+#if __UNIX__ && !defined(__serenity__)
+#define RDWR_FLAGS O_RDWR | O_SYNC
+#else
+#define RDWR_FLAGS O_RDWR
+#endif
+
 #define BS 1024
 
 static int file_stat(const char *file, struct stat* const pStat) {
@@ -37,9 +43,9 @@ static int file_stat(const char *file, struct stat* const pStat) {
 	int ret = _wstat (wfile, pStat);
 	free (wfile);
 	return ret;
-#else // __WINDOWS__
+#else
 	return stat (file, pStat);
-#endif // __WINDOWS__
+#endif
 }
 
 // r_file_new("", "bin", NULL) -> /bin
@@ -49,20 +55,26 @@ R_API char *r_file_new(const char *root, ...) {
 	va_list ap;
 	va_start (ap, root);
 	RStrBuf *sb = r_strbuf_new ("");
-	char *home = r_str_home (NULL);
-	const char *arg = va_arg (ap, char *);
-	r_strbuf_append (sb, arg);
-	arg = va_arg (ap, char *);
-	while (arg) {
-		if (!strcmp (arg, "~")) {
-			arg = home;
+	if (!strcmp (root, "~")) {
+		char *h = r_str_home (NULL);
+		if (!h) {
+			va_end (ap);
+			r_strbuf_free (sb);
+			return NULL;
 		}
+		r_strbuf_append (sb, h);
+		free (h);
+	} else {
+		r_strbuf_append (sb, root);
+	}
+	r_strbuf_append (sb, R_SYS_DIR);
+	const char *arg = va_arg (ap, char *);
+	while (arg) {
 		r_strbuf_append (sb, R_SYS_DIR);
 		r_strbuf_append (sb, arg);
 		arg = va_arg (ap, char *);
 	}
 	va_end (ap);
-	free (home);
 	char *path = r_strbuf_drain (sb);
 	char *abs = r_file_abspath (path);
 	free (path);
@@ -78,15 +90,11 @@ R_API bool r_file_truncate(const char *filename, ut64 newsize) {
 	if (!r_file_exists (filename) || !r_file_is_regular (filename)) {
 		return false;
 	}
-#if __WINDOWS__
-	fd = r_sandbox_open (filename, O_RDWR, 0644);
-#else
-	fd = r_sandbox_open (filename, O_RDWR | O_SYNC, 0644);
-#endif
+	fd = r_sandbox_open (filename, RDWR_FLAGS, 0644);
 	if (fd == -1) {
 		return false;
 	}
-#ifdef _MSC_VER
+#if defined(_MSC_VER) || __WINDOWS__
 	int r = _chsize (fd, newsize);
 #else
 	int r = ftruncate (fd, newsize);
@@ -187,14 +195,20 @@ R_API bool r_file_fexists(const char *fmt, ...) {
 }
 
 R_API bool r_file_exists(const char *str) {
-	char *absfile = r_file_abspath (str);
 	struct stat buf = {0};
+#if 1
+	if (file_stat (str, &buf) == -1) {
+		return false;
+	}
+#else
+	char *absfile = r_file_abspath (str);
 	r_return_val_if_fail (!R_STR_ISEMPTY (str), false);
 	if (file_stat (absfile, &buf) == -1) {
 		free (absfile);
 		return false;
 	}
 	free (absfile);
+#endif
 	return S_IFREG == (S_IFREG & buf.st_mode);
 }
 
@@ -238,7 +252,8 @@ R_API char *r_file_abspath_rel(const char *cwd, const char *file) {
 				PTCHAR f = r_sys_conv_utf8_to_win (file);
 				int s = GetFullPathName (f, MAX_PATH, abspath, NULL);
 				if (s > MAX_PATH) {
-					R_LOG_ERROR ("r_file_abspath/GetFullPathName: Path to file too long.\n");
+					// R_LOG_ERROR ("r_file_abspath/GetFullPathName: Path to file too long.\n");
+					eprintf ("r_file_abspath/GetFullPathName: Path to file too long.\n");
 				} else if (!s) {
 					r_sys_perror ("r_file_abspath/GetFullPathName");
 				} else {
@@ -253,7 +268,7 @@ R_API char *r_file_abspath_rel(const char *cwd, const char *file) {
 	if (!ret) {
 		ret = strdup (file);
 	}
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	char *abspath = realpath (ret, NULL);
 	if (abspath) {
 		free (ret);
@@ -278,10 +293,10 @@ R_API char *r_file_binsh(void) {
 	char *bin_sh = r_sys_getenv ("SHELL");
 	if (R_STR_ISEMPTY (bin_sh)) {
 		free (bin_sh);
-		bin_sh = r_file_path("sh");
+		bin_sh = r_file_path ("sh");
 		if (R_STR_ISEMPTY (bin_sh)) {
 			free (bin_sh);
-			bin_sh = strdup ("/bin/sh");
+			bin_sh = strdup (SHELL_PATH);
 		}
 	}
 	return bin_sh;
@@ -325,8 +340,11 @@ R_API char *r_file_path(const char *bin) {
 	return strdup (bin);
 }
 
-R_API char *r_stdin_slurp (int *sz) {
-#if __UNIX__ || __WINDOWS__
+R_API char *r_stdin_slurp(int *sz) {
+#if __wasi__
+#warning r_stdin_slurp not available for wasi
+	return NULL;
+#elif __UNIX__ || __WINDOWS__
 	int i, ret, newfd;
 	if ((newfd = dup (0)) < 0) {
 		return NULL;
@@ -390,6 +408,13 @@ R_API char *r_file_slurp(const char *str, R_NULLABLE size_t *usz) {
 		fclose (fd);
 		return NULL;
 	}
+#if __wasi__
+	fclose (fd);
+	fd = r_sandbox_fopen (str, "rb");
+	if (!fd) {
+		return NULL;
+	}
+#endif
 	if (!sz) {
 		if (r_file_is_regular (str)) {
 			char *buf = NULL;
@@ -411,6 +436,7 @@ R_API char *r_file_slurp(const char *str, R_NULLABLE size_t *usz) {
 			char *nbuf = realloc (buf, size + 1);
 			if (!nbuf) {
 				free (buf);
+				fclose (fd);
 				return NULL;
 			}
 			buf = nbuf;
@@ -426,14 +452,14 @@ R_API char *r_file_slurp(const char *str, R_NULLABLE size_t *usz) {
 		sz = UT16_MAX;
 	}
 	rewind (fd);
-	char *ret = (char *)malloc (sz + 1);
+	char *ret = (char *)calloc (sz + 1, 1);
 	if (!ret) {
 		fclose (fd);
 		return NULL;
 	}
 	size_t rsz = fread (ret, 1, sz, fd);
 	if (rsz != sz) {
-		eprintf ("Warning: r_file_slurp: fread: truncated read\n");
+		eprintf ("Warning: r_file_slurp: fread: truncated read (%d / %d)\n", (int)rsz, (int)sz);
 		sz = rsz;
 	}
 	fclose (fd);
@@ -636,7 +662,7 @@ R_API char *r_file_slurp_lines_from_bottom(const char *file, int line) {
 			}
 		}
 		if (line > lines) {
-			return strdup (str);	// number of lines requested in more than present, return all
+			return str;	// number of lines requested in more than present, return all
 		}
 		i--;
 		for (; str[i] && line; i--) {
@@ -644,7 +670,7 @@ R_API char *r_file_slurp_lines_from_bottom(const char *file, int line) {
 				line--;
 			}
 		}
-		ptr = str+i;
+		ptr = str + i;
 		ptr = strdup (ptr);
 		free (str);
 	}
@@ -881,25 +907,29 @@ err_r_file_mmap_write:
 		CloseHandle (fh);
 	}
 	return ret;
+#elif __wasi__ || EMSCRIPTEN
+	return -1;
 #elif __UNIX__
-	int fd = r_sandbox_open (file, O_RDWR|O_SYNC, 0644);
+	int fd = r_sandbox_open (file, RDWR_FLAGS, 0644);
 	const int pagesize = getpagesize ();
 	int mmlen = len + pagesize;
 	int rest = addr % pagesize;
-        ut8 *mmap_buf;
+	ut8 *mmap_buf;
 	if (fd == -1) {
 		return -1;
 	}
 	if ((st64)addr < 0) {
 		return -1;
 	}
-	mmap_buf = mmap (NULL, mmlen*2, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)addr - rest);
+	mmap_buf = mmap (NULL, mmlen * 2, PROT_READ|PROT_WRITE, MAP_SHARED, fd, (off_t)addr - rest);
 	if (((int)(size_t)mmap_buf) == -1) {
 		return -1;
 	}
-	memcpy (mmap_buf+rest, buf, len);
-	msync (mmap_buf+rest, len, MS_INVALIDATE);
-	munmap (mmap_buf, mmlen*2);
+	memcpy (mmap_buf + rest, buf, len);
+#if !defined(__serenity__)
+	msync (mmap_buf + rest, len, MS_INVALIDATE);
+#endif
+	munmap (mmap_buf, mmlen * 2);
 	close (fd);
 	return len;
 #else
@@ -907,7 +937,7 @@ err_r_file_mmap_write:
 #endif
 }
 
-R_API int r_file_mmap_read (const char *file, ut64 addr, ut8 *buf, int len) {
+R_API int r_file_mmap_read(const char *file, ut64 addr, ut8 *buf, int len) {
 #if __WINDOWS__
 	HANDLE fm = NULL, fh = INVALID_HANDLE_VALUE;
 	LPTSTR file_ = NULL;
@@ -942,6 +972,8 @@ err_r_file_mmap_read:
 	}
 	free (file_);
 	return ret;
+#elif __wasi__ || EMSCRIPTEN
+	return 0;
 #elif __UNIX__
 	int fd = r_sandbox_open (file, O_RDONLY, 0644);
 	const int pagesize = 4096;
@@ -959,12 +991,17 @@ err_r_file_mmap_read:
 	munmap (mmap_buf, mmlen*2);
 	close (fd);
 	return len;
-#endif
+#else
 	return 0;
+#endif
 }
 
-#if __UNIX__
-static RMmap *r_file_mmap_unix (RMmap *m, int fd) {
+#if __wasi__ || EMSCRIPTEN
+static RMmap *r_file_mmap_unix(RMmap *m, int fd) {
+	return NULL;
+}
+#elif __UNIX__
+static RMmap *r_file_mmap_unix(RMmap *m, int fd) {
 	ut8 empty = m->len == 0;
 	m->buf = mmap (NULL, (empty?BS:m->len) ,
 		m->rw?PROT_READ|PROT_WRITE:PROT_READ,
@@ -1009,7 +1046,7 @@ err_r_file_mmap_windows:
 	return m;
 }
 #else
-static RMmap *file_mmap_other (RMmap *m) {
+static RMmap *file_mmap_other(RMmap *m) {
 	ut8 empty = m->len == 0;
 	m->buf = malloc ((empty? BS: m->len));
 	if (!empty && m->buf) {
@@ -1043,7 +1080,7 @@ R_API RMmap *r_file_mmap(const char *file, bool rw, ut64 base) {
 	if (!rw && !r_file_exists (file)) {
 		return m;
 	}
-	fd = r_sandbox_open (file, rw? O_RDWR: O_RDONLY, 0644);
+	fd = r_sandbox_open (file, rw? RDWR_FLAGS: O_RDONLY, 0644);
 	if (fd == -1 && !rw) {
 		eprintf ("r_file_mmap: file does not exis.\n");
 		//m->buf = malloc (m->len);
@@ -1102,7 +1139,7 @@ R_API void r_file_mmap_free(RMmap *m) {
 		return;
 	}
 	free (m->filename);
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	munmap (m->buf, m->len);
 #endif
 	close (m->fd);
@@ -1119,15 +1156,51 @@ R_API char *r_file_temp(const char *prefix) {
 	return res;
 }
 
+R_API char *r_file_temp_ex(R_NULLABLE const char *prefix, R_NULLABLE const char *ex) {
+	prefix = R_STR_ISEMPTY (prefix)? "r2": prefix;
+	ex = R_STR_ISEMPTY (ex)? "": ex;
+	char *path = r_file_tmpdir ();
+	char *res = NULL;
+	if (path) {
+		ut64 t = r_time_now ();
+		res = r_str_newf ("%s/%s.%" PFMT64x "%s", path, prefix, t, ex);
+		free (path);
+	}
+	return res;
+}
+
+static inline char *file_fmt_split(const char *fmt) {
+	if (R_STR_ISEMPTY (fmt)) {
+		return r_file_temp_ex (NULL, NULL);
+	}
+	char *name = NULL;
+	char *dup = strdup (fmt);
+	if (dup) {
+		RList *splt = r_str_split_list (dup, "*", 2);
+		if (splt && r_list_length (splt)) {
+			char *pref = r_list_pop_head (splt);
+			char *ex = r_list_pop_head (splt);
+			name = r_file_temp_ex (pref, ex);
+		}
+		r_list_free (splt);
+		free (dup);
+	}
+	return name;
+}
+
 R_API int r_file_mkstemp(R_NULLABLE const char *prefix, char **oname) {
 	int h = -1;
-	char *path = r_file_tmpdir ();
 	if (!prefix) {
 		prefix = "r2";
 	}
 #if __WINDOWS__
 	LPTSTR name = NULL;
+	char *path = r_file_tmpdir ();
+	if (!path) {
+		return -1;
+	}
 	LPTSTR path_ = r_sys_conv_utf8_to_win (path);
+	free (path);
 	LPTSTR prefix_ = r_sys_conv_utf8_to_win (prefix);
 
 	name = (LPTSTR)malloc (sizeof (TCHAR) * (MAX_PATH + 1));
@@ -1136,7 +1209,7 @@ R_API int r_file_mkstemp(R_NULLABLE const char *prefix, char **oname) {
 	}
 	if (GetTempFileName (path_, prefix_, 0, name)) {
 		char *name_ = r_sys_conv_win_to_utf8 (name);
-		h = r_sandbox_open (name_, O_RDWR|O_EXCL|O_BINARY, 0644);
+		h = r_sandbox_open (name_, RDWR_FLAGS | O_EXCL | O_BINARY, 0644);
 		if (oname) {
 			if (h != -1) {
 				*oname = name_;
@@ -1152,44 +1225,20 @@ err_r_file_mkstemp:
 	free (name);
 	free (path_);
 	free (prefix_);
+#elif __wasi__
+	// nothing to do for wasm, drops to return -1
 #else
-	char pfxx[1024];
-	const char *suffix = strchr (prefix, '*');
-
-	if (suffix) {
-		suffix++;
-		r_str_ncpy (pfxx, prefix, (size_t)(suffix - prefix));
-		prefix = pfxx;
-	} else {
-		suffix = "";
-	}
-
-	char *name = r_str_newf ("%s/r2.%s.XXXXXX%s", path, prefix, suffix);
-	mode_t mask = umask (S_IWGRP | S_IWOTH);
-	if (suffix && *suffix) {
-#if defined(__GLIBC__) && defined(__GLIBC_MINOR__) && 2 <= __GLIBC__ && 19 <= __GLIBC__MINOR__
-		h = mkstemps (name, strlen (suffix));
-#else
-		char *const xpos = strrchr (name, 'X');
-		const char c = (char)(NULL != xpos ? *(xpos + 1) : 0);
-		if (0 != c) {
-			xpos[1] = 0;
-			h = mkstemp (name);
-			xpos[1] = c;
+	char *name = file_fmt_split (prefix);
+	if (name) {
+		int perm = RDWR_FLAGS | O_CREAT | O_EXCL | O_BINARY;
+		h = r_sandbox_open (name, perm, 0644);
+		if (h != -1) {
+			*oname = name;
 		} else {
-			h = -1;
+			free (name);
 		}
-#endif
-	} else {
-		h = mkstemp (name);
 	}
-	umask (mask);
-	if (oname) {
-		*oname = (h!=-1)? strdup (name): NULL;
-	}
-	free (name);
 #endif
-	free (path);
 	return h;
 }
 
@@ -1232,19 +1281,30 @@ R_API char *r_file_tmpdir(void) {
 	}
 	if (!path) {
 #if __ANDROID__
-		path = strdup ("/data/data/org.radare.radare2installer/radare2/tmp");
+		if (r_file_is_directory (TERMUX_PREFIX "/tmp")) {
+			path = strdup (TERMUX_PREFIX "/tmp");
+		} else {
+			path = strdup ("/data/local/tmp");
+		}
 #else
 		path = strdup ("/tmp");
 #endif
 	}
 #endif
 	if (!r_file_is_directory (path)) {
-		eprintf ("Cannot find temporary directory '%s'\n", path);
+		free (path);
+		return NULL;
+		//eprintf ("Cannot find dir.tmp '%s'\n", path);
 	}
 	return path;
 }
 
 R_API bool r_file_copy(const char *src, const char *dst) {
+	r_return_val_if_fail (R_STR_ISNOTEMPTY (src) && R_STR_ISNOTEMPTY (dst), false);
+	if (!strcmp (src, dst)) {
+		eprintf ("Cannot copy file '%s' to itself.\n", src);
+		return false;
+	}
 	/* TODO: implement in C */
 	/* TODO: Use NO_CACHE for iOS dyldcache copying */
 #if HAVE_COPYFILE_H
@@ -1253,7 +1313,6 @@ R_API bool r_file_copy(const char *src, const char *dst) {
 	PTCHAR s = r_sys_conv_utf8_to_win (src);
 	PTCHAR d = r_sys_conv_utf8_to_win (dst);
 	if (!s || !d) {
-		R_LOG_ERROR ("r_file_copy: Failed to allocate memory\n");
 		free (s);
 		free (d);
 		return false;
@@ -1276,7 +1335,7 @@ R_API bool r_file_copy(const char *src, const char *dst) {
 }
 
 static bool dir_recursive(RList *dst, const char *dir) {
-	char *name;
+	char *name, *path = NULL;
 	RListIter *iter;
 	bool ret = true;
 	RList *files = r_sys_dir (dir);
@@ -1284,7 +1343,6 @@ static bool dir_recursive(RList *dst, const char *dir) {
 		return false;
 	}
 	r_list_foreach (files, iter, name) {
-		char *path;
 		if (!strcmp (name, "..") || !strcmp (name, ".")) {
 			continue;
 		}
@@ -1293,9 +1351,8 @@ static bool dir_recursive(RList *dst, const char *dir) {
 			ret = false;
 			break;
 		}
-		if (!r_list_append (dst, path)) {
+		if (!r_list_append (dst, strdup (path))) {
 			ret = false;
-			free (path);
 			break;
 		}
 		if (r_file_is_directory (path)) {
@@ -1304,13 +1361,15 @@ static bool dir_recursive(RList *dst, const char *dir) {
 				break;
 			}
 		}
+		R_FREE (path);
 	}
+	free (path);
 	r_list_free (files);
 	return ret;
 }
 
 R_API RList *r_file_lsrf(const char *dir) {
-	RList *ret = r_list_new ();
+	RList *ret = r_list_newf (free);
 	if (!ret) {
 		return NULL;
 	}
@@ -1321,24 +1380,38 @@ R_API RList *r_file_lsrf(const char *dir) {
 	return ret;
 }
 
+R_API bool r_file_rm_rf(const char *dir) {
+	RList *files = r_file_lsrf (dir);
+	if (r_file_exists (dir)) {
+		return r_file_rm (dir);
+	}
+	if (!files) {
+		return false;
+	}
+	r_list_sort (files, (RListComparator) strcmp);
+	RListIter *iter;
+	char *f;
+	r_list_foreach_prev (files, iter, f)  {
+		r_file_rm (f);
+	}
+	return r_file_rm (dir);
+}
 
-static void recursive_search_glob(const char *path, const char *glob, RList* list, int depth) {
+
+static void recursive_glob(const char *path, const char *glob, RList* list, int depth) {
 	if (depth < 1) {
 		return;
 	}
 	char* file;
 	RListIter *iter;
-	RList *dir = r_sys_dir (path);
-	r_list_foreach (dir, iter, file) {
+	RList *files = r_sys_dir (path);
+	r_list_foreach (files, iter, file) {
 		if (!strcmp (file, ".") || !strcmp (file, "..")) {
 			continue;
 		}
-		char *filename = malloc (strlen (path) + strlen (file) + 2);
-		strcpy (filename, path);
-		strcat (filename, file);
+		char *filename = r_str_newf ("%s%s", path, file);
 		if (r_file_is_directory (filename)) {
-			strcat (filename, R_SYS_DIR);
-			recursive_search_glob (filename, glob, list, depth - 1);
+			recursive_glob (filename, glob, list, depth - 1);
 			free (filename);
 		} else if (r_str_glob (file, glob)) {
 			r_list_append (list, filename);
@@ -1346,10 +1419,10 @@ static void recursive_search_glob(const char *path, const char *glob, RList* lis
 			free (filename);
 		}
 	}
-	r_list_free (dir);
+	r_list_free (files);
 }
 
-R_API RList* r_file_globsearch(const char *_globbed_path, int maxdepth) {
+R_API RList* r_file_glob(const char *_globbed_path, int maxdepth) {
 	char *globbed_path = strdup (_globbed_path);
 	RList *files = r_list_newf (free);
 	char *glob = strchr (globbed_path, '*');
@@ -1381,9 +1454,9 @@ R_API RList* r_file_globsearch(const char *_globbed_path, int maxdepth) {
 		}
 
 		if (*(glob + 1) == '*') {  // "**"
-			recursive_search_glob (path, glob_ptr, files, maxdepth);
-		} else {                   // "*"
-			recursive_search_glob (path, glob_ptr, files, 1);
+			recursive_glob (path, glob_ptr, files, maxdepth);
+		} else {            // "*"
+			recursive_glob (path, glob_ptr, files, 1);
 		}
 		free (path);
 	}
