@@ -278,16 +278,16 @@ R_API char *r_str_home(const char *str) {
 		}
 	}
 	length = strlen (home) + 1;
-	if (str) {
+	if (R_STR_ISNOTEMPTY (str)) {
 		length += strlen (R_SYS_DIR) + strlen (str);
 	}
-	dst = (char *)malloc (length);
+	dst = (char *)calloc (1, length);
 	if (!dst) {
 		goto fail;
 	}
 	int home_len = strlen (home);
 	memcpy (dst, home, home_len + 1);
-	if (str) {
+	if (R_STR_ISNOTEMPTY (str)) {
 		dst[home_len] = R_SYS_DIR[0];
 		strcpy (dst + home_len + 1, str);
 	}
@@ -716,6 +716,19 @@ R_API char *r_str_trunc_ellipsis(const char *str, int len) {
 	return buf;
 }
 
+R_API char *r_str_newvf(const char *fmt, va_list ap) {
+	va_list ap2;
+	va_copy (ap2, ap);
+	int ret = vsnprintf (NULL, 0, fmt, ap2);
+	ret++;
+	char *p = calloc (1, ret);
+	if (p) {
+		(void)vsnprintf (p, ret, fmt, ap);
+	}
+	va_end (ap2);
+	return p;
+}
+
 R_API char *r_str_newf(const char *fmt, ...) {
 	va_list ap, ap2;
 
@@ -881,7 +894,7 @@ R_API char *r_str_append_owned(char *ptr, char *string) {
 	if (!ptr) {
 		return string;
 	}
-	char *r = r_str_append(ptr, string);
+	char *r = r_str_append (ptr, string);
 	free (string);
 	return r;
 }
@@ -949,6 +962,9 @@ R_API char* r_str_replace(char *str, const char *key, const char *val, int g) {
 		r_str_replace_char (str, *key, *val);
 		return str;
 	}
+	if (klen == 0) {
+		return str;
+	}
 	if (klen == vlen && !strcmp (key, val)) {
 		return str;
 	}
@@ -1163,6 +1179,15 @@ R_API int r_str_unescape(char *buf) {
 		case 'f':
 			buf[i] = 0x0c;
 			break;
+		case '"':
+			buf[i] = '"';
+			break;
+		case '\'':
+			buf[i] = '\'';
+			break;
+		case '`':
+			buf[i] = '`';
+			break;
 		case 'x':
 			err = ch2 = ch = 0;
 			if (!buf[i + 2] || !buf[i + 3]) {
@@ -1195,8 +1220,7 @@ R_API int r_str_unescape(char *buf) {
 				}
 				esc_seq_len = 1 + num_digits;
 			} else {
-				eprintf ("Error: Unknown escape sequence.\n");
-				return 0; // -1?
+				esc_seq_len = 1;
 			}
 		}
 		memmove (buf + i + 1, buf + i + esc_seq_len, strlen (buf + i + esc_seq_len) + 1);
@@ -1360,6 +1384,24 @@ out:
 	return new_buf;
 }
 
+/* hex-escape unprintable characters in a raw buffer (null-safe) */
+R_API char *r_str_escape_raw(const ut8 *buf, int sz) {
+	r_return_val_if_fail (buf, NULL);
+
+	/* Worst case scenario, we convert every byte to a \xhh escape */
+	char *new_buf = malloc (1 + sz * 4);
+	if (!new_buf) {
+		return NULL;
+	}
+	char *q = new_buf;
+	int i;
+	for (i = 0; i < sz; i++) {
+		r_str_byte_escape ((char *)&buf[i], &q, false, false, true);
+	}
+	*q = '\0';
+	return new_buf;
+}
+
 R_API char *r_str_escape(const char *buf) {
 	return r_str_escape_ (buf, false, true, true, false, true);
 }
@@ -1823,8 +1865,8 @@ R_API char *r_str_format_msvc_argv(size_t argc, const char **argv) {
 			r_strbuf_append (&sb, " ");
 		}
 		const char *arg = argv[i];
-		bool must_escape = strchr (arg, '\"') != NULL;
-		bool must_quote = strpbrk (arg, " \t") != NULL || !*arg;
+		bool must_escape = strchr (arg, '\"');
+		bool must_quote = strpbrk (arg, " \t") || !*arg;
 		if (!must_escape && must_quote && *arg && arg[strlen (arg) - 1] == '\\') {
 			// if the last char is a bs and we would quote it, we must also escape
 			must_escape = true;
@@ -1911,7 +1953,7 @@ R_API size_t r_str_ansi_nlen(const char *str, size_t slen) {
 		}
 		i += chlen;
 	}
-	return len > 0 ? len: 1;
+	return len; // len > 0 ? len: 1;
 }
 
 R_API size_t r_str_ansi_len(const char *str) {
@@ -2024,7 +2066,12 @@ R_API size_t r_wstr_clen(const char *s) {
 	return len + 1;
 }
 
+// TODO: rename to r_str_ansi_at() ? or find better name?
 R_API const char *r_str_ansi_chrn(const char *str, size_t n) {
+#if 0
+	size_t pos = r_str_ansi_nlen (str, at);
+	return str + pos;
+#endif
 	int len, i, li;
 	for (li = i = len = 0; str[i] && (n != len); i++) {
 		size_t chlen = __str_ansi_length (str + i);
@@ -2218,7 +2265,6 @@ R_API bool r_str_char_fullwidth(const char* s, size_t left) {
 		 R_BETWEEN (0xffe0, codepoint, 0xffe6) ||
 		 R_BETWEEN (0x20000, codepoint, 0x2fffd) ||
 		 R_BETWEEN (0x30000, codepoint, 0x3fffd)));
-
 }
 
 /**
@@ -2594,6 +2640,46 @@ R_API const char *r_str_firstbut(const char *s, char ch, const char *but) {
 	return NULL;
 }
 
+R_API const char *r_str_firstbut_escape(const char *s, char ch, const char *but) {
+	int idx, _b = 0;
+	ut8 *b = (ut8*)&_b;
+	const char *isbut, *p;
+	const int bsz = sizeof (_b) * 8;
+	if (!but) {
+		return strchr (s, ch);
+	}
+	if (strlen (but) >= bsz) {
+		eprintf ("r_str_firstbut: but string too long\n");
+		return NULL;
+	}
+	for (p = s; *p; p++) {
+		if (*p == '\\') {
+			p++;
+			if (*p == ch || strchr(but, *p)) {
+				if (!*p) {
+					break;
+				}
+				continue;
+			} else if (!*p) {
+				break;
+			}
+		}
+		isbut = strchr (but, *p);
+		if (isbut) {
+			idx = (int)(size_t)(isbut - but);
+			_b = R_BIT_TOGGLE (b, idx);
+			if (_b && (_b & (_b - 1))) {
+				_b = R_BIT_TOGGLE (b, idx); // cancel a but char if a but is already toggle
+			}
+			continue;
+		}
+		if (*p == ch && !_b) {
+			return p;
+		}
+	}
+	return NULL;
+}
+
 R_API const char *r_str_lastbut(const char *s, char ch, const char *but) {
 	int idx, _b = 0;
 	ut8 *b = (ut8*)&_b;
@@ -3166,7 +3252,7 @@ R_API char *r_str_wrap(const char *str, int w) {
 	return ret;
 }
 
-R_API const char * r_str_tok(const char *str1, const char b, size_t len) {
+R_API const char *r_str_tok(const char *str1, const char b, size_t len) {
 	const char *p = str1;
 	size_t i = 0;
 	if (!p || !*p) {
@@ -3595,6 +3681,10 @@ R_API char *r_str_from_ut64(ut64 val) {
 	if (!str) {
 		return NULL;
 	}
+	while (!*v && i < 8) {
+		v++;
+		i++;
+	}
 	while (i < 8 && *v) {
 		str[i++] = *v++;
 	}
@@ -3604,8 +3694,13 @@ R_API char *r_str_from_ut64(ut64 val) {
 R_API int r_snprintf(char *string, int len, const char *fmt, ...) {
 	va_list ap;
 	va_start (ap, fmt);
-	int ret = vsnprintf (string, len, fmt, ap);
-	string[len - 1] = 0;
+	int ret = 0;
+	if (len > 0) {
+		ret = vsnprintf (string, len, fmt, ap);
+		string[len - 1] = 0;
+	} else {
+		*string = 0;
+	}
 	va_end (ap);
 	return ret;
 }
@@ -3748,11 +3843,11 @@ R_API char *r_str_scale(const char *s, int w, int h) {
 	int curline = -1;
 	char *linetext = (char*)r_str_pad (' ', w);
 	for (i = 0; i < h; i++) {
-		int zoomedline = i * ((float)rows / h);
+		int zoomedline = (int)(i * ((double)rows / h));
 		const char *srcline = r_list_get_n (lines, zoomedline);
 		int cols = strlen (srcline);
 		for (j = 0; j < w; j++) {
-			int zoomedcol = j * ( (float)cols / w);
+			int zoomedcol = (int)(j * ( (double)cols / w));
 			linetext[j] = srcline[zoomedcol];
 		}
 		if (curline != zoomedline) {
