@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2020 - pancake */
+/* radare - LGPL - Copyright 2007-2021 - pancake */
 
 #if __WINDOWS__
 #include <stdlib.h>
@@ -117,7 +117,7 @@ R_API char *r_num_units(char *buf, size_t len, ut64 num) {
 			return NULL;
 		}
 	}
-	fnum = num;
+	fnum = (long double)num;
 	if (num >= EB) { unit = 'E'; fnum /= EB; } else
 	if (num >= PB) { unit = 'P'; fnum /= PB; } else
 	if (num >= TB) { unit = 'T'; fnum /= TB; } else
@@ -156,6 +156,41 @@ static void error(RNum *num, const char *err_str) {
 	}
 }
 
+static ut64 r_num_from_binary(const char *str) {
+	int i, j;
+	ut64 ret = 0;
+	for (j = 0, i = strlen (str) - 1; i > 0; i--, j++) {
+		if (str[i] == '1') {
+			ret |= (ut64) (1ULL << j);
+		} else if (str[i] != '0') {
+			break;
+		}
+	}
+	sscanf (str, "0x%"PFMT64x, &ret);
+	return ret;
+}
+
+R_API ut64 r_num_from_ternary(const char *inp) {
+	if (!inp) {
+		return 0LL;
+	}
+	const char *p;
+	int pos = strlen (inp);
+	ut64 fr = 0;
+	for (p = inp; *p ; p++, pos--) {
+		ut64 n012 = 0;
+		switch (*p) {
+		case '0':
+		case '1':
+		case '2':
+			n012 = *p - '0';
+			fr += (ut64)(n012 * pow (3, pos - 1));
+			break;
+		}
+	}
+	return fr;
+}
+
 // TODO: try to avoid the use of sscanf
 /* old get_offset */
 R_API ut64 r_num_get(RNum *num, const char *str) {
@@ -218,21 +253,16 @@ R_API ut64 r_num_get(RNum *num, const char *str) {
 		}
 	}
 	if (str[0] == '0' && str[1] == 'b') {
-		ret = 0;
-		for (j = 0, i = strlen (str) - 1; i > 0; i--, j++) {
-			if (str[i] == '1') {
-				ret|=1 << j;
-			} else if (str[i] != '0') {
-				break;
-			}
-		}
-		sscanf (str, "0x%"PFMT64x, &ret);
+		ret = r_num_from_binary (str + 2);
 	} else if (str[0] == '\'') {
 		ret = str[1] & 0xff;
 	// ugly as hell
 	} else if (!strncmp (str, "0xff..", 6) || !strncmp (str, "0xFF..", 6)) {
 		ret = r_num_tailff (num, str + 6);
 	// ugly as hell
+	} else if (!strncmp (str, "0t", 2)) {
+		// parse ternary number
+		ret = r_num_from_ternary (str + 2);
 	} else if (!strncmp (str, "0o", 2)) {
 		if (sscanf (str + 2, "%"PFMT64o, &ret) != 1) {
 			error (num, "invalid octal number");
@@ -483,7 +513,7 @@ R_API ut64 r_num_math(RNum *num, const char *str) {
 			p = group + 1;
 			if (r_str_delta (p, '(', ')') < 0) {
 				char *p2 = strchr (p, '(');
-				if (p2 != NULL) {
+				if (p2) {
 					*p2 = '\0';
 					ret = r_num_op (op, ret, r_num_math_internal (num, p));
 					ret = r_num_op (op, ret, r_num_math (num, p2 + 1));
@@ -909,3 +939,28 @@ R_API double r_num_cos(double a) {
 R_API double r_num_sin(double a) {
 	return sin (a);
 }
+
+// sega dance dance revolution numbers
+// convert address into segmented address
+// takes addr, segment base and segment granurality
+R_API bool r_num_segaddr(ut64 addr, ut64 sb, int sg, ut32 *a, ut32 *b) {
+#if 0
+	s = n >> 16 << 12;
+	a = n & 0x0fff;
+#endif
+	if (sb) {
+		ut32 csbase = (sb << 4);
+		if (addr > csbase) {
+			*a = sb;
+			*b = addr - csbase;
+		} else {
+			int delta = csbase - addr;
+			*a = csbase + delta;
+			*b = addr - csbase+ delta;
+		}
+	} else {
+		*a = ((addr >> 16) << (16 - sg));
+		*b = (addr & 0xffff);
+	}
+	return *a <= 0xffff && *b <= 0xffff;
+}
