@@ -1,10 +1,7 @@
-/* sdb - MIT - Copyright 2011-2021 - pancake */
+/* sdb - MIT - Copyright 2011-2022 - pancake */
 
-#include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
-#include <string.h>
-#include <stdlib.h>
 #include <sys/stat.h>
 #include "sdb.h"
 
@@ -31,20 +28,11 @@ static inline SdbKv *next_kv(HtPP *ht, SdbKv *kv) {
 		     (j) < (bt)->count;					\
 		     (j) = (count) == (ht)->count? j + 1: j, (kv) = (count) == (ht)->count? next_kv (ht, kv): kv, (count) = (ht)->count)
 
-static inline int nextcas(void) {
-	static ut32 cas = 1;
-	if (!cas) {
-		cas++;
+static inline int nextcas(SdbKv const *kv) {
+	if (!kv->cas) {
+		return 1;
 	}
-	return cas++;
-}
-
-static SdbHook global_hook = NULL;
-static void* global_user = NULL;
-
-SDB_API void sdb_global_hook(SdbHook hook, void *user) {
-	global_hook = hook;
-	global_user = user;
+	return kv->cas + 1;
 }
 
 // TODO: use mmap instead of read.. much faster!
@@ -59,16 +47,19 @@ SDB_API Sdb* sdb_new(const char *path, const char *name, int lock) {
 	}
 	s->db.fd = -1;
 	s->fd = -1;
+	s->journal = -1;
 	s->refs = 1;
 	s->ht = sdb_ht_new ();
 	if (path && !*path) {
 		path = NULL;
 	}
 	if (name && *name && strcmp (name, "-")) {
+		char buf[SDB_MAX_PATH];
+
 		if (path && *path) {
 			size_t plen = strlen (path);
 			size_t nlen = strlen (name);
-			s->dir = malloc (plen + nlen + 2);
+			s->dir = (char *)malloc (plen + nlen + 2);
 			if (!s->dir) {
 				free (s);
 				return NULL;
@@ -82,12 +73,18 @@ SDB_API Sdb* sdb_new(const char *path, const char *name, int lock) {
 		}
 		switch (lock) {
 		case 1:
-			if (!sdb_lock (sdb_lock_file (s->dir))) {
+			if (!sdb_lock_file (s->dir, buf, sizeof (buf))) {
+				goto fail;
+			}
+			if (!sdb_lock (buf)) {
 				goto fail;
 			}
 			break;
 		case 2:
-			if (!sdb_lock_wait (sdb_lock_file (s->dir))) {
+			if (!sdb_lock_file (s->dir, buf, sizeof (buf))) {
+				goto fail;
+			}
+			if (!sdb_lock_wait (buf)) {
 				goto fail;
 			}
 			break;
@@ -114,10 +111,6 @@ SDB_API Sdb* sdb_new(const char *path, const char *name, int lock) {
 		goto fail;
 	}
 	s->lock = lock;
-	// if open fails ignore
-	if (global_hook) {
-		sdb_hook (s, global_hook, global_user);
-	}
 	cdb_init (&s->db, s->fd);
 	return s;
 fail:
@@ -134,18 +127,21 @@ fail:
 
 // XXX: this is wrong. stuff not stored in memory is lost
 SDB_API void sdb_file(Sdb* s, const char *dir) {
+	char buf[SDB_MAX_PATH];
 	if (s->lock) {
-		sdb_unlock (sdb_lock_file (s->dir));
+		sdb_lock_file (s->dir, buf, sizeof (buf));
+		sdb_unlock (buf);
 	}
 	free (s->dir);
 	s->dir = (dir && *dir)? strdup (dir): NULL;
 	if (s->lock) {
-		sdb_lock (sdb_lock_file (s->dir));
+		sdb_lock_file (s->dir, buf, sizeof (buf));
+		sdb_lock (buf);
 	}
 }
 
 static bool sdb_merge_cb(void *user, const char *k, const char *v) {
-	sdb_set (user, k, v, 0);
+	sdb_set ((Sdb*)user, k, v, 0);
 	return true;
 }
 
@@ -185,13 +181,15 @@ SDB_API int sdb_count(Sdb *s) {
 }
 
 static void sdb_fini(Sdb* s, bool donull) {
+	char buf[SDB_MAX_PATH];
 	if (!s) {
 		return;
 	}
 	sdb_hook_free (s);
 	cdb_free (&s->db);
 	if (s->lock) {
-		sdb_unlock (sdb_lock_file (s->dir));
+		sdb_lock_file (s->dir, buf, sizeof (buf));
+		sdb_unlock (buf);
 	}
 	sdb_ns_free (s);
 	s->refs = 0;
@@ -278,7 +276,7 @@ SDB_API const char *sdb_const_get_len(Sdb* s, const char *key, int *vlen, ut32 *
 		return NULL;
 	}
 	(void) cdb_findstart (&s->db);
-	if (cdb_findnext (&s->db, s->ht->opt.hashfn (key), key, keylen) < 1) {
+	if (!s->ht || cdb_findnext (&s->db, s->ht->opt.hashfn (key), key, keylen) < 1) {
 		return NULL;
 	}
 	len = cdb_datalen (&s->db);
@@ -354,7 +352,7 @@ SDB_API int sdb_concat(Sdb *s, const char *key, const char *value, ut32 cas) {
 		return sdb_set (s, key, value, cas);
 	}
 	vl = strlen (value);
-	o = malloc (kl + vl + 1);
+	o = (char *)malloc (kl + vl + 1);
 	if (o) {
 		memcpy (o, p, kl);
 		memcpy (o + kl, value, vl + 1);
@@ -404,6 +402,13 @@ SDB_API int sdb_open_gperf(Sdb *s, SdbGperf *gp) {
 	return 0;
 }
 
+static int sdb_open_text(Sdb *s, const char *file) {
+	if (!sdb_text_load (s, file)) {
+		return -1;
+	}
+	return s->fd;
+}
+
 SDB_API int sdb_open(Sdb *s, const char *file) {
         struct stat st;
 	if (!s) {
@@ -411,7 +416,7 @@ SDB_API int sdb_open(Sdb *s, const char *file) {
 	}
 	if (file) {
 		if (sdb_text_check (s, file)) {
-			return sdb_text_load (s, file);
+			return sdb_open_text (s, file);
 		}
 		if (s->fd != -1) {
 			close (s->fd);
@@ -427,7 +432,7 @@ SDB_API int sdb_open(Sdb *s, const char *file) {
 	s->last = 0LL;
 	if (s->fd != -1 && fstat (s->fd, &st) != -1) {
 		if ((S_IFREG & st.st_mode) != S_IFREG) {
-			eprintf ("Database must be a file\n");
+			// eprintf ("Database must be a file\n");
 			close (s->fd);
 			s->fd = -1;
 			return -1;
@@ -551,7 +556,7 @@ SDB_API SdbKv* sdbkv_new2(const char *k, int kl, const char *v, int vl) {
 		kv->base.value = NULL;
 		kv->base.value_len = 0;
 	}
-	kv->cas = nextcas ();
+	kv->cas = nextcas (kv);
 	kv->expire = 0LL;
 	return kv;
 }
@@ -575,7 +580,7 @@ static ut32 sdb_set_internal(Sdb* s, const char *key, char *val, bool owned, ut3
 		if (owned) {
 			val = strdup ("");
 		} else {
-			val = "";
+			val = (char *)"";
 		}
 	}
 	// XXX strlen computed twice.. because of check_*()
@@ -604,7 +609,7 @@ static ut32 sdb_set_internal(Sdb* s, const char *key, char *val, bool owned, ut3
 				sdb_hook_call (s, key, val);
 				return kv->cas;
 			}
-			kv->cas = cas = nextcas ();
+			kv->cas = cas = nextcas (kv);
 			if (owned) {
 				kv->base.value_len = vlen;
 				free (kv->base.value);
@@ -635,7 +640,7 @@ static ut32 sdb_set_internal(Sdb* s, const char *key, char *val, bool owned, ut3
 		kv = sdbkv_new2 (key, klen, val, vlen);
 	}
 	if (kv) {
-		ut32 cas = kv->cas = nextcas ();
+		ut32 cas = kv->cas = nextcas (kv);
 		sdb_ht_insert_kvp (s->ht, kv, true /*update*/);
 		free (kv);
 		sdb_hook_call (s, key, val);
@@ -664,7 +669,8 @@ static bool sdb_foreach_list_cb(void *user, const char *k, const char *v) {
 }
 
 static int __cmp_asc(const void *a, const void *b) {
-	const SdbKv *ka = a, *kb = b;
+	const SdbKv *ka = (SdbKv *)a;
+	const SdbKv *kb = (SdbKv *)b;
 	return strcmp (sdbkv_key (ka), sdbkv_key (kb));
 }
 
@@ -729,7 +735,9 @@ typedef struct {
 
 static bool sdb_foreach_match_cb(void *user, const char *k, const char *v) {
 	_match_sdb_user *o = (_match_sdb_user*)user;
-	SdbKv tkv = { .base.key = (char*)k, .base.value = (char*)v };
+	SdbKv tkv = {0};
+	tkv.base.key = (char *)k;
+	tkv.base.value = (char *)v;
 	if (sdbkv_match (&tkv, o->expr)) {
 		SdbKv *kv = R_NEW0 (SdbKv);
 		kv->base.key = strdup (k);
@@ -794,6 +802,9 @@ SDB_API bool sdb_foreach(Sdb* s, SdbForeachCallback cb, void *user) {
 	if (!s) {
 		return false;
 	}
+	if (s->gp) {
+		return s->gp->foreach ((GperfForeachCallback)cb, user);
+	}
 	s->depth++;
 	bool result = sdb_foreach_cdb (s, cb, NULL, user);
 	if (!result) {
@@ -955,7 +966,7 @@ SDB_API bool sdb_dump_dupnext(Sdb* s, char *key, char **value, int *_vlen) {
 	if (value) {
 		*value = 0;
 		if (vlen < SDB_MAX_VALUE) {
-			*value = malloc (vlen + 10);
+			*value = (char *)malloc (vlen + 10);
 			if (!*value) {
 				return false;
 			}
@@ -970,7 +981,7 @@ SDB_API bool sdb_dump_dupnext(Sdb* s, char *key, char **value, int *_vlen) {
 	return true;
 }
 
-static inline ut64 parse_expire (ut64 e) {
+static inline ut64 parse_expire(ut64 e) {
 	const ut64 month = 30 * 24 * 60 * 60;
 	if (e > 0 && e < month) {
 		e += sdb_now ();
@@ -1010,7 +1021,7 @@ SDB_API bool sdb_expire_set(Sdb* s, const char *key, ut64 expire, ut32 cas) {
 	if (len < 1 || len >= INT32_MAX) {
 		return false;
 	}
-	if (!(buf = calloc (1, len + 1))) {
+	if (!(buf = (char *)calloc (1, len + 1))) {
 		return false;
 	}
 	cdb_read (&s->db, buf, len, pos);
@@ -1036,7 +1047,7 @@ SDB_API bool sdb_hook(Sdb* s, SdbHook cb, void* user) {
 	SdbHook hook;
 	SdbListIter *iter;
 	if (s->hooks) {
-		ls_foreach (s->hooks, iter, hook) {
+		ls_foreach_cast (s->hooks, iter, SdbHook, hook) {
 			if (!(i % 2) && (hook == cb)) {
 				return false;
 			}
@@ -1055,7 +1066,7 @@ SDB_API bool sdb_unhook(Sdb* s, SdbHook h) {
 	int i = 0;
 	SdbHook hook;
 	SdbListIter *iter, *iter2;
-	ls_foreach (s->hooks, iter, hook) {
+	ls_foreach_cast (s->hooks, iter, SdbHook, hook) {
 		if (!(i % 2) && (hook == h)) {
 			iter2 = iter->n;
 			ls_delete (s->hooks, iter);
@@ -1074,7 +1085,7 @@ SDB_API int sdb_hook_call(Sdb *s, const char *k, const char *v) {
 	if (s->timestamped && s->last) {
 		s->last = sdb_now ();
 	}
-	ls_foreach (s->hooks, iter, hook) {
+	ls_foreach_cast (s->hooks, iter, SdbHook, hook) {
 		if (!(i % 2) && k && iter->n) {
 			void *u = iter->n->data;
 			hook (s, u, k, v);
@@ -1127,7 +1138,7 @@ SDB_API void sdb_drain(Sdb *s, Sdb *f) {
 }
 
 static bool copy_foreach_cb(void *user, const char *k, const char *v) {
-	Sdb *dst = user;
+	Sdb *dst = (Sdb *)user;
 	sdb_set (dst, k, v, 0);
 	return true;
 }
@@ -1136,7 +1147,7 @@ SDB_API void sdb_copy(Sdb *src, Sdb *dst) {
 	sdb_foreach (src, copy_foreach_cb, dst);
 	SdbListIter *it;
 	SdbNs *ns;
-	ls_foreach (src->ns, it, ns) {
+	ls_foreach_cast (src->ns, it, SdbNs*, ns) {
 		sdb_copy (ns->sdb, sdb_ns (dst, ns->name, true));
 	}
 }
@@ -1147,7 +1158,7 @@ typedef struct {
 } UnsetCallbackData;
 
 static bool unset_cb(void *user, const char *k, const char *v) {
-	UnsetCallbackData *ucd = user;
+	UnsetCallbackData *ucd = (UnsetCallbackData *)user;
 	if (sdb_match (k, ucd->key)) {
 		sdb_unset (ucd->sdb, k, 0);
 	}
@@ -1170,7 +1181,7 @@ typedef struct {
 } LikeCallbackData;
 
 static bool like_cb(void *user, const char *k, const char *v) {
-	LikeCallbackData *lcd = user;
+	LikeCallbackData *lcd = (LikeCallbackData *)user;
 	if (!user) {
 		return false;
 	}
@@ -1216,7 +1227,7 @@ SDB_API char** sdb_like(Sdb *s, const char *k, const char *v, SdbForeachCallback
 		lcd.val = NULL;
 	}
 	lcd.array_size = sizeof (char*) * 2;
-	lcd.array = calloc (lcd.array_size, 1);
+	lcd.array = (const char **)calloc (lcd.array_size, 1); // XXX shouldnt be const
 	if (!lcd.array) {
 		return NULL;
 	}
